/*
This file is part of FSDB: a higher level library to access a Firebird database using IBPP.

Copyright Fulvio Senore.

License:

Permission is hereby granted, free of charge, to any person or organization ("You") obtaining a copy of this software and 
associated documentation files covered by this license (the "Software") to use the Software as part of another work; 
to modify it for that purpose; to publish or distribute it, modified or not, for that same purpose; 
to permit persons to whom the other work using the Software is furnished to do so; subject to the following conditions: 
the above copyright notice and this complete and unmodified permission notice shall be included in all copies or substantial 
portions of the Software; You will not misrepresent modified versions of the Software as being the original.

The Software is provided "as is", without warranty of any kind, express or implied, including but not limited 
to the warranties of merchantability, fitness for a particular purpose and noninfringement. 
In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software 
or the use of other dealings in the Software.
*/

/*
main include file for FSDB
*/

/*

#define the following symbol:

_WX_FSDB_

to enable native conversion among wxWidgets types (wxString, wxDateTime) and FSDB::Variant

*/


#ifndef __FSDB_H__
#define __FSDB_H__


#include "variant.h"
#include "parameters.h"
#include "database_firebird.h"
#include "data_error.h"

/// Macros used to declare a Recordset object.
/// Each macro declares a pointer to a recordset using an unlikely variable name,
/// creates a recordset using then name to store a pointer, 
/// then declares a reference to the created recordset using the name received as a parameter
/// I do this because I need a reference to use the "." operator and I need a pointer to store the allocated object
#define FSDB_CREATE_RS_READONLY_FWDONLY(db, name) \
	FSDB::RecordsetPt ptr__$$##name = db->CreateRecordset( FSDB::rtReadOnlyForwardOnly ); \
	FSDB::Recordset name = *ptr__$$##name;

#define FSDB_CREATE_RS_READONLY_REPEATABLE_FWDONLY(db, name) \
	FSDB::RecordsetPt ptr__$$##name = db->CreateRecordset( FSDB::rtReadOnlyRepeatableForwardOnly ); \
	FSDB::Recordset name = *ptr__$$##name;

#define FSDB_CREATE_RS_READWRITE_FWDONLY(db, name) \
	FSDB::RecordsetPt ptr__$$##name = db->CreateRecordset( FSDB::rtReadWriteForwardOnly ); \
	FSDB::Recordset name = *ptr__$$##name;

#define FSDB_CREATE_RS_READWRITE(db, name) \
	FSDB::RecordsetPt ptr__$$##name = db->CreateRecordset( FSDB::rtReadWrite ); \
	FSDB::Recordset name = *ptr__$$##name;

/// Macros used to declare a RecordsetPt object
#define FSDB_CREATE_RSPT_READONLY_FWDONLY(db, name) \
	FSDB::RecordsetPt name = db->CreateRecordset( FSDB::rtReadOnlyForwardOnly );

#define FSDB_CREATE_RSPT_READONLY_REPEATABLE_FWDONLY(db, name) \
	FSDB::RecordsetPt name = db->CreateRecordset( FSDB::rtReadOnlyRepeatableForwardOnly );

#define FSDB_CREATE_RSPT_READWRITE_FWDONLY(db, name) \
	FSDB::RecordsetPt name = db->CreateRecordset( FSDB::rtReadWriteForwardOnly );

#define FSDB_CREATE_RSPT_READWRITE(db, name) \
	FSDB::RecordsetPt name = db->CreateRecordset( FSDB::rtReadWrite );


#endif
