/*
This file is part of FSDB: a higher level library to access a Firebird database using IBPP.

Copyright Fulvio Senore.

License:

Permission is hereby granted, free of charge, to any person or organization ("You") obtaining a copy of this software and 
associated documentation files covered by this license (the "Software") to use the Software as part of another work; 
to modify it for that purpose; to publish or distribute it, modified or not, for that same purpose; 
to permit persons to whom the other work using the Software is furnished to do so; subject to the following conditions: 
the above copyright notice and this complete and unmodified permission notice shall be included in all copies or substantial 
portions of the Software; You will not misrepresent modified versions of the Software as being the original.

The Software is provided "as is", without warranty of any kind, express or implied, including but not limited 
to the warranties of merchantability, fitness for a particular purpose and noninfringement. 
In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software 
or the use of other dealings in the Software.
*/

#ifndef __FSDB_PARAMETERS_H__
#define __FSDB_PARAMETERS_H__

#include "date_time.h"
#include "_fsdb.h"
#include "variant.h"

#include <boost/shared_ptr.hpp>
#include <string>
#include <vector>

namespace FSDB {

	/// a class that contains the parameters for a database command or sql statement
	class ParametersImpl {

	public:

		/// removes all data from the container
		void Clear() { buf.clear(); }

		/// returns the number of parameters stored in the container
		size_t Size() { return buf.size(); }

		/// returns a parameter as a Variant
		Variant& operator[]( int index ) { return buf[index]; }

		/// append a new parameter to the end of the container
		void Append( int16_t value );
		/// append a new parameter to the end of the container
		void Append( int32_t value );
		/// append a new parameter to the end of the container
		void Append( int64_t value );
		/// append a new parameter to the end of the container
		void Append( const std::string &value );
		/// append a new parameter to the end of the container
		void Append( float value );
		/// append a new parameter to the end of the container
		void Append( double value );
		/// append a new parameter to the end of the container
		void Append( const Timestamp &value );
		/// append a NULL parameter to the end of the container
		void AppendNull();

	protected:

		std::vector<Variant> buf;

	};

	/// the type name used by programs for a pointer to a Parameter object
	typedef boost::shared_ptr<ParametersImpl> Parameters;

	/// function to be called to return a newly created, empty Parameters object
	Parameters ParametersFactory();
}

#endif


