/*
This file is part of FSDB: a higher level library to access a Firebird database using IBPP.

Copyright Fulvio Senore.

License:

Permission is hereby granted, free of charge, to any person or organization ("You") obtaining a copy of this software and 
associated documentation files covered by this license (the "Software") to use the Software as part of another work; 
to modify it for that purpose; to publish or distribute it, modified or not, for that same purpose; 
to permit persons to whom the other work using the Software is furnished to do so; subject to the following conditions: 
the above copyright notice and this complete and unmodified permission notice shall be included in all copies or substantial 
portions of the Software; You will not misrepresent modified versions of the Software as being the original.

The Software is provided "as is", without warranty of any kind, express or implied, including but not limited 
to the warranties of merchantability, fitness for a particular purpose and noninfringement. 
In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
whether in an action of contract, tort or otherwise, arising from, out of or in connection with the software 
or the use of other dealings in the Software.
*/


#include "recordset_fo_ro.h"

#include <assert.h>

FSDB::Recordset_FO_RO::Recordset_FO_RO( RecordsetType rt, DbInterfaceBase *DbInterface ) {
	this->rt = rt;
	this->sql = "";
	this->DbInterface = DbInterface;
}

FSDB::Recordset_FO_RO::~Recordset_FO_RO() {
	if( DbInterface != NULL ) delete DbInterface;
}

void FSDB::Recordset_FO_RO::Open( const std::string &sql, const std::string &tableName, const std::string &primaryKeyName ) {

	this->sql = sql;
	// passes a collection with zero parameters
	DbInterface->OpenRecordset( rt, sql, ParametersFactory(), tableName, primaryKeyName );
	isOpened = true;
}


void FSDB::Recordset_FO_RO::Open( const std::string &sql, Parameters params, const std::string &tableName, const std::string &primaryKeyName ) {

	this->sql = sql;
	DbInterface->OpenRecordset( rt, sql, params, tableName, primaryKeyName );
	isOpened = true;
}


void FSDB::Recordset_FO_RO::Close() {

    CheckOpened();

	DbInterface->Close();
	isOpened = false;
}


bool FSDB::Recordset_FO_RO::IsBOF() {
	assert( DbInterface != NULL );
    CheckOpened();
	return DbInterface->IsBOF();
}

bool FSDB::Recordset_FO_RO::IsEOF() {
	assert( DbInterface != NULL );
    CheckOpened();
	return DbInterface->IsEOF();
}

bool FSDB::Recordset_FO_RO::IsEmpty() {
	return ( IsEOF() && IsBOF() );
}

FSDB::Variant& FSDB::Recordset_FO_RO::operator []( const std::string& column ) {
	return DbInterface->GetVariantByName( column );
}

void FSDB::Recordset_FO_RO::MoveNext() {
    CheckOpened();
    DbInterface->MoveNext();
}

void FSDB::Recordset_FO_RO::MovePrevious() {
	throw std::logic_error( "Undefined method for this object" );
}

void FSDB::Recordset_FO_RO::MoveLast() {
	throw std::logic_error( "Undefined method for this object" );
}

void FSDB::Recordset_FO_RO::MoveFirst() {
	throw std::logic_error( "Undefined method for this object" );
}

void FSDB::Recordset_FO_RO::Edit() {
	throw std::logic_error( "Undefined method for this object" );
}

void FSDB::Recordset_FO_RO::AddNew() {
	throw std::logic_error( "Undefined method for this object" );
}

void FSDB::Recordset_FO_RO::Update() {
	throw std::logic_error( "Undefined method for this object" );
}

void FSDB::Recordset_FO_RO::CancelUpdate() {
	throw std::logic_error( "Undefined method for this object" );
}

void FSDB::Recordset_FO_RO::Delete() {
	throw std::logic_error( "Undefined method for this object" );
}

void FSDB::Recordset_FO_RO::ReleaseMemory() {
	throw std::logic_error( "Undefined method for this object" );
}

void FSDB::Recordset_FO_RO::MoveAbsolute( long newPosition ) {
	throw std::logic_error( "Undefined method for this object" );
}

long FSDB::Recordset_FO_RO::GetAbsolutePosition() {
	throw std::logic_error( "Undefined method for this object" );
}

void FSDB::Recordset_FO_RO::SaveAbsolutePosition() {
	throw std::logic_error( "Undefined method for this object" );
}

void FSDB::Recordset_FO_RO::RestoreAbsolutePosition() {
	throw std::logic_error( "Undefined method for this object" );
}

long FSDB::Recordset_FO_RO::GetNumberOfCachedRows() {
	throw std::logic_error( "Undefined method for this object" );
}

bool FSDB::Recordset_FO_RO::HasReadAllRows() {
	throw std::logic_error( "Undefined method for this object" );
}

void FSDB::Recordset_FO_RO::CheckOpened()
{
    if( !isOpened )
	    throw std::runtime_error( "This recordset is not opened" );
}

