///////////////////////////////////////////////////////////////////////////////
//
//	File    : $Id: time.cpp 54 2006-03-27 16:07:44Z epocman $
//	Subject : IBPP, Time class implementation
//
///////////////////////////////////////////////////////////////////////////////
//
//	(C) Copyright 2000-2006 T.I.P. Group S.A. and the IBPP Team (www.ibpp.org)
//
//	The contents of this file are subject to the IBPP License (the "License");
//	you may not use this file except in compliance with the License.  You may
//	obtain a copy of the License at http://www.ibpp.org or in the 'license.txt'
//	file which must have been distributed along with this file.
//
//	This software, distributed under the License, is distributed on an "AS IS"
//	basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
//	License for the specific language governing rights and limitations
//	under the License.
//
///////////////////////////////////////////////////////////////////////////////
//
//	COMMENTS
//	* Tabulations should be set every four characters when editing this file.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef _MSC_VER
#pragma warning(disable: 4786 4996)
#ifndef _DEBUG
#pragma warning(disable: 4702)
#endif
#endif

#include "date_time.h"

#ifdef HAS_HDRSTOP
#pragma hdrstop
#endif

#include <time.h>		// Can't use <ctime> thanks to MSVC6 buggy library

//using namespace ibpp_internals;

void FSDB::Time::Now()
{
	time_t systime = time(0);
	tm* loctime = localtime(&systime);
	FSDB::itot(&mTime, loctime->tm_hour, loctime->tm_min, loctime->tm_sec, 0);
}

void FSDB::Time::SetTime(int tm)
{
	if (tm < 0 || tm > 863999999)
		throw LogicExceptionImpl("Time::SetTime - Invalid time value");
	mTime = tm;
}

void FSDB::Time::SetTime(int hour, int minute, int second, int tenthousandths)
{
	if (hour < 0 || hour > 23 ||
		minute < 0 || minute > 59 ||
			second < 0 || second > 59 ||
				tenthousandths < 0 || tenthousandths > 9999)
					throw LogicExceptionImpl("Time::SetTime - Invalid hour, minute, second values");
	FSDB::itot(&mTime, hour, minute, second, tenthousandths);
}

void FSDB::Time::GetTime(int& hour, int& minute, int& second) const
{
	FSDB::ttoi(mTime, &hour, &minute, &second, 0);
}

void FSDB::Time::GetTime(int& hour, int& minute, int& second, int& tenthousandths) const
{
	FSDB::ttoi(mTime, &hour, &minute, &second, &tenthousandths);
}

int FSDB::Time::Hours() const
{
	int hours;
	FSDB::ttoi(mTime, &hours, 0, 0, 0);
	return hours;
}

int FSDB::Time::Minutes() const
{
	int minutes;
	FSDB::ttoi(mTime, 0, &minutes, 0, 0);
	return minutes;
}

int FSDB::Time::Seconds() const
{
	int seconds;
	FSDB::ttoi(mTime, 0, 0, &seconds, 0);
	return seconds;
}

int FSDB::Time::SubSeconds() const	// Actually tenthousandths of seconds
{
	int tenthousandths;
	FSDB::ttoi(mTime, 0, 0, 0, &tenthousandths);
	return tenthousandths;
}

FSDB::Time::Time(int hour, int minute, int second, int tenthousandths)
{
	SetTime(hour, minute, second, tenthousandths);
}

FSDB::Time::Time(const FSDB::Time& copied)
{
	mTime = copied.mTime;
}

FSDB::Time& FSDB::Time::operator=(const FSDB::Timestamp& assigned)
{
	mTime = assigned.GetTime();
	return *this;
}

FSDB::Time& FSDB::Time::operator=(const FSDB::Time& assigned)
{
	mTime = assigned.mTime;
	return *this;
}

//	Time calculations. Internal format is the number of seconds elapsed since
//	midnight. Splits such a time in its hours, minutes, seconds components.

void FSDB::ttoi(int itime, int *h, int *m, int *s, int* t)
{
	int hh, mm, ss, tt;

	hh = (int) (itime / 36000000);	itime = itime - hh * 36000000;
	mm = (int) (itime / 600000);	itime = itime - mm * 600000;
	ss = (int) (itime / 10000);
	tt = (int) (itime - ss * 10000);

	if (h != 0) *h = hh;
	if (m != 0) *m = mm;
	if (s != 0) *s = ss;
	if (t != 0) *t = tt;

	return;
}

//	Get the internal time format, given hour, minute, second.

void FSDB::itot (int *ptime, int hour, int minute, int second, int tenthousandths)
{
	*ptime = hour * 36000000 + minute * 600000 + second * 10000 + tenthousandths;
	return;
}


//
//	EOF
//

