/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    aint with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        catalog_images.h
// Purpose:     
// Author:      
// Modified by: 
// Created:     31/12/2008 10:35:36
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _CATALOG_IMAGES_H_
#define _CATALOG_IMAGES_H_


/*!
 * Includes
 */

////@begin includes
////@end includes

#include "window_position.h"
#include "fsdb.h"

#include <wx/filename.h>

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxBoxSizer;
////@end forward declarations

class CCompareFolders;

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_DIALOG_CATALOG_IMAGES 10048
#define ID_HANDLE_REMOVABLE 10049
#define ID_PANEL_REMOVABLE 10050
#define ID_VOLUME_PATH 10004
#define ID_VOLUME_BROWSE 10005
#define ID_VOLUME_NAME 10006
#define ID_GET_VOLUME_NAME 10007
#define ID_REM_FOLDER_PATH 10051
#define ID_FOLDER_BROWSE 10052
#define ID_HANDLE_FIXED 10053
#define ID_PANEL_FIXED 10054
#define ID_CHOICE_FIXED_LOCATION 10055
#define ID_COMBO_FIXED_LOCATION 10134
#define ID_FIX_FOLDER_PATH 10056
#define ID_FIXED_FOLDER_BROWSE 10057
#define ID_BUTTON_CATALOG 10008
#define ID_LAST_CATALOGED 10003
#define ID_PROGRESS_BAR 10010
#define SYMBOL_CDIALOGCATALOGIMAGES_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CDIALOGCATALOGIMAGES_TITLE _("Catalog Images")
#define SYMBOL_CDIALOGCATALOGIMAGES_IDNAME ID_DIALOG_CATALOG_IMAGES
#define SYMBOL_CDIALOGCATALOGIMAGES_SIZE wxSize(400, 300)
#define SYMBOL_CDIALOGCATALOGIMAGES_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CDialogCatalogImages class declaration
 */

class CDialogCatalogImages: public wxDialog
{    
    DECLARE_DYNAMIC_CLASS( CDialogCatalogImages )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CDialogCatalogImages();
    CDialogCatalogImages( wxWindow* parent, wxWindowID id = SYMBOL_CDIALOGCATALOGIMAGES_IDNAME, const wxString& caption = SYMBOL_CDIALOGCATALOGIMAGES_TITLE, const wxPoint& pos = SYMBOL_CDIALOGCATALOGIMAGES_POSITION, const wxSize& size = SYMBOL_CDIALOGCATALOGIMAGES_SIZE, long style = SYMBOL_CDIALOGCATALOGIMAGES_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CDIALOGCATALOGIMAGES_IDNAME, const wxString& caption = SYMBOL_CDIALOGCATALOGIMAGES_TITLE, const wxPoint& pos = SYMBOL_CDIALOGCATALOGIMAGES_POSITION, const wxSize& size = SYMBOL_CDIALOGCATALOGIMAGES_SIZE, long style = SYMBOL_CDIALOGCATALOGIMAGES_STYLE );

    /// Destructor
    ~CDialogCatalogImages();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CDialogCatalogImages event handler declarations

    /// wxEVT_UPDATE_UI event handler for ID_PANEL_REMOVABLE
    void OnPanelRemovableUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_VOLUME_BROWSE
    void OnVolumeBrowseClick( wxCommandEvent& event );

#if defined(__WXMSW__)
    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_GET_VOLUME_NAME
    void OnGetVolumeNameClick( wxCommandEvent& event );

#endif
    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_FOLDER_BROWSE
    void OnFolderBrowseClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_HANDLE_FIXED
    void OnHandleFixedClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_PANEL_FIXED
    void OnPanelFixedUpdate( wxUpdateUIEvent& event );

#if defined(__WXMSW__) || defined(__WXMAC__)
    /// wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE_FIXED_LOCATION
    void OnChoiceFixedLocationSelected( wxCommandEvent& event );

#endif
#if defined(__WXGTK__)
    /// wxEVT_COMMAND_COMBOBOX_SELECTED event handler for ID_COMBO_FIXED_LOCATION
    void OnChoiceFixedLocationSelected( wxCommandEvent& event );

#endif
    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_FIXED_FOLDER_BROWSE
    void OnFixedFolderBrowseClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_HELP
    void OnHelpClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_CATALOG
    void OnButtonCatalogClick( wxCommandEvent& event );

////@end CDialogCatalogImages event handler declarations

////@begin CDialogCatalogImages member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CDialogCatalogImages member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

	// return true if this dialog has changed the database
	bool IsDatabaseChanged() { return m_DatabaseChanged; }

////@begin CDialogCatalogImages member variables
    wxBoxSizer* m_MainSizer;
    wxCheckBox* m_ChkHandleRemovable;
    wxPanel* m_PanelRemovable;
    wxTextCtrl* m_VolumePath;
    wxTextCtrl* m_VolumeName;
    wxTextCtrl* m_FolderPathRemovable;
    wxCheckBox* m_ChkHandleFixed;
    wxPanel* m_PanelFixed;
#if defined(__WXMSW__) || defined(__WXMAC__)
    wxChoice* m_FixedLocationsCombo;
#endif
#if defined(__WXGTK__)
    wxComboBox* m_FixedLocationsCombo;
#endif
    wxTextCtrl* m_FolderPathFixed;
    wxButton* m_HelpButton;
    wxButton* m_CatalogButton;
    wxButton* m_CloseButton;
    wxStaticText* m_LastCatalogedCtrl;
    wxGauge* m_ProgressBarCtrl;
////@end CDialogCatalogImages member variables

private:

	// the name read directly from the volume (useful in case the user will change it)
	wxString m_realVolumeName;

	CWindowPosition m_WindowPosition;

	// index of the currently selected fixed locations combo item
	int m_CurrentFixedLocationComboIndex;

	// report a list of files different in the physical and removable paths
	// asks the user if he wants to continue. Returns true if the user wants to continue, returns false if the user wants to abort
	bool ReportDifferentFiles( CCompareFolders cf );

	// show a warning if the user asks to catalog fixed locations but none of them are defined
	void ShowWarningMissingFixedLocation();

	// enable or disable all controls, used while cataloging to avoid recursion
	void EnableDisableControls( bool enabled );
	// used by EnableDisableControls to remember if the panel was enabled when disabling all the controls
	bool removablePanelEnabled;
	bool fixedPanelEnabled;

	// true if the database has been changed
	// used by the caller to decide if it needs to reload the treeview or not
	bool m_DatabaseChanged;

	// this variable is usually false: will be changed to true if the user wants to interrupt cataloging
	bool cancelCatalogTask;

	// flag: true while the program is cataloging images
	bool cataloging;
};

#endif
    // _CATALOG_IMAGES_H_
