/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2011, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
this class is used to retrieve a list of files from the database
*/

#ifndef __DB_FILE_LIST_H__
#define __DB_FILE_LIST_H__

#include "fsdb.h"

#include <vector>

struct CDbFileListData
{
	wxString path;	// path to the file
	wxString SHA1;	// SHA-1 hash of the file
};

class CDbFileList
{
public:

	CDbFileList( FSDB::Database db );
	~CDbFileList(void);

	// add to the internal list all the files contained in a volume, including subfolders
	// read the list from the database
	void CreateFileListVolume( const int volumeID );

	// add to the internal list all the files contained in a folder, including subfolders
	// folderPath is the full path to the folder in the current computer
	// read the list from the database
	void CreateFileListFolder( const int volumeID, const int folderID, const wxString &folderPath );

	// retrieve the files list, after it has been created
	std::vector<CDbFileListData> GetFileList() { return m_FileList; }

protected:

	// add filenames in a folder and recurse in subfolders
	// receives the folder ID in the database and the full physical path to that folder
	void AddSingleFolderToList( const int volumeID, const int pathID, const wxString &path );

	std::vector<CDbFileListData> m_FileList;

	FSDB::Database m_db;
};

#endif

