/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2009, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "db_utils.h"

#include "utils.h"
#include "sqlstrings.h"
#include "vvvpapp.h"
#include "fsdb.h"

CDbUtils::LocationsStatus CDbUtils::FixedLocationPathFromID( const int fileID, const wxString &fileName, wxString &fullPath )
{
	fullPath = wxEmptyString;

	wxString path;
	LocationsStatus ls = FixedLocationFolderFromID( fileID, path );
	
	if( ls != lsOk ) return ls;

	wxString sep = wxString(wxFileName::GetPathSeparator());
	fullPath = path + sep + fileName;

	return lsOk;
}

CDbUtils::LocationsStatus CDbUtils::FixedLocationFolderFromID( const int fileID, wxString &folderPath )
{
	folderPath = wxEmptyString;

	wxString sep = wxString(wxFileName::GetPathSeparator());
	FSDB::Database db = wxGetApp().GetDatabase();
	FSDB::Parameters params = FSDB::ParametersFactory();
	params->Append( fileID );
	params->Append( CUtils::wx2std( sep ) );
	params->Append( "F" );
	std::string sql = CSqlStrings::GetFullPhysicalPathFromFile();
	wxString path = CUtils::DBstd2wx( db->ExecuteReturnString(sql, params) );

	if( path.empty() ) return lsMissingFixedLocation;

	// translate the fixed location name (first part of the path) to the real path for the computer that is running the program
	int pos = path.Find( sep );
	if( pos == wxNOT_FOUND ) {
		// root folder for this volume
		folderPath = wxGetApp().GetFixedLocations().GetPathByName( path );
		if( folderPath.empty() ) return lsUndefinedFixedLocation;
	}
	else {
		wxString physLoc = path.Left( pos );	// physical location
		wxString physPath = wxGetApp().GetFixedLocations().GetPathByName( physLoc );
		if( physPath.empty() ) return lsUndefinedFixedLocation;
		folderPath = physPath + path.Mid( pos );
	}

	return lsOk;
}

CDbUtils::LocationsStatus CDbUtils::FixedLocationFolderFromPathID( const int pathID, wxString &folderPath )
{
	folderPath = wxEmptyString;

	wxString sep = wxString(wxFileName::GetPathSeparator());
	FSDB::Database db = wxGetApp().GetDatabase();
	FSDB::Parameters params = FSDB::ParametersFactory();
	params->Append( pathID );
	params->Append( CUtils::wx2std( sep ) );
	std::string sql = CSqlStrings::GetFullPhysicalPath();
	wxString path = CUtils::DBstd2wx( db->ExecuteReturnString(sql, params) );

	if( path.empty() ) return lsMissingFixedLocation;

	// translate the fixed location name (first part of the path) to the real path for the computer that is running the program
	int pos = path.Find( sep );
	if( pos == wxNOT_FOUND ) {
		// root folder for this volume
		folderPath = wxGetApp().GetFixedLocations().GetPathByName( path );
		if( folderPath.empty() ) return lsUndefinedFixedLocation;
	}
	else {
		wxString physLoc = path.Left( pos );	// physical location
		wxString physPath = wxGetApp().GetFixedLocations().GetPathByName( physLoc );
		if( physPath.empty() ) return lsUndefinedFixedLocation;
		folderPath = physPath + path.Mid( pos );
	}

	return lsOk;
}

