/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2010, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "external_programs.h"
#include "utils.h"

#include "wx/config.h"

void CSingleExternalProgram::Copy(const CSingleExternalProgram& data)
{
	m_CommandLine = data.m_CommandLine;
	m_Description = data.m_Description;
	m_MakeCopy =data.m_MakeCopy;
}

CSingleExternalProgram::CSingleExternalProgram( const CSingleExternalProgram& data )
{
	Copy( data );
}

void CSingleExternalProgram::operator=(const CSingleExternalProgram& data)
{
	Copy( data );
}

CExternalPrograms::CExternalPrograms(void)
{
}

CExternalPrograms::~CExternalPrograms(void)
{
	Clear();
}

CExternalPrograms::CExternalPrograms( const CExternalPrograms& data )
{
	Copy( data );
}

void CExternalPrograms::operator=(const CExternalPrograms& data)
{
	Copy( data );
}

void CExternalPrograms::Copy(const CExternalPrograms& data)
{
	Clear();
	for( unsigned int k = 0; k < data.size(); k++ ) {
		CSingleExternalProgram *sl = new CSingleExternalProgram;
		*sl = data[k];
		dataArray.push_back( sl );
	}
}

void CExternalPrograms::Clear() {

	for( unsigned int k = 0; k < dataArray.size(); k++ ) {
		delete dataArray[k];
	}
	dataArray.clear();
}

CExternalPrograms::Error CExternalPrograms::Append( wxString commandLine, wxString description, bool makeCopy ) {

	CExternalPrograms::Error err = CheckData( commandLine, description, makeCopy, -1 );
	if( err != eNoError )
		return err;

	CSingleExternalProgram *sl = new CSingleExternalProgram;
	sl->m_CommandLine = commandLine;
	sl->m_Description = description;
	sl->m_MakeCopy = makeCopy;
	dataArray.push_back( sl );

	return eNoError;
}

CExternalPrograms::Error CExternalPrograms::Edit( wxString commandLine, wxString description, bool makeCopy, int index ) {

	if( index < 0 || index >= (int) dataArray.size() )
		throw std::runtime_error( "Invalid index" );

	CExternalPrograms::Error err = CheckData( commandLine, description, makeCopy, index );
	if( err != eNoError )
		return err;

	CSingleExternalProgram *sl = dataArray[index];
	sl->m_CommandLine = commandLine;
	sl->m_Description = description;
	sl->m_MakeCopy = makeCopy;

	return eNoError;
}

CExternalPrograms::Error CExternalPrograms::CheckData( wxString commandLine, wxString description, bool makeCopy, int index ) {

	if( commandLine == wxEmptyString )
		return eMissingCommandLine;
	if( description == wxEmptyString )
		return eMissingDescription;

	for( unsigned int k = 0; k < dataArray.size(); k++ ) {
		if( k != index ) {

			if( dataArray[k]->m_CommandLine == commandLine && dataArray[k]->m_MakeCopy == makeCopy  )
				return eDuplicateCommandLine;

			if( dataArray[k]->m_Description == description )
				return eDuplicateDescription;

		}
	}

	return eNoError;
}

void CExternalPrograms::Remove( int index ) {

	if( index < 0 || index >= (int) dataArray.size() )
		throw std::runtime_error( "Invalid index" );

	delete dataArray[index];
	dataArray.erase( dataArray.begin() + index );
}

void CExternalPrograms::Load() {

	Clear();
	
	wxConfigBase *pConfig = wxConfigBase::Get();

	pConfig->SetPath( wxT("/ExternalPrograms") );
	int nPrograms = pConfig->Read( wxT("nPrograms"), (long) 0 );

	for( int k = 0; k < nPrograms; k++ ) {
		wxString commandLine = pConfig->Read( wxT("CommandLine") + CUtils::long2string(k), wxEmptyString );
		wxString description = pConfig->Read( wxT("Description") + CUtils::long2string(k), wxEmptyString );
		bool makeCopy;
		pConfig->Read( wxT("MakeCopy") + CUtils::long2string(k), &makeCopy, false );

		if( commandLine != wxEmptyString && description != wxEmptyString )
			Append( commandLine, description, makeCopy );
	}
}

void CExternalPrograms::Save() {

	wxConfigBase *pConfig = wxConfigBase::Get();

	pConfig->SetPath( wxT("/ExternalPrograms/") );
	pConfig->Write( wxT("nPrograms"), (long) dataArray.size() );

	for( unsigned int k = 0; k < dataArray.size(); k++ ) {
		pConfig->Write( wxT("CommandLine") + CUtils::long2string(k), dataArray[k]->m_CommandLine );
		pConfig->Write( wxT("Description") + CUtils::long2string(k), dataArray[k]->m_Description );
		pConfig->Write( wxT("MakeCopy") + CUtils::long2string(k), dataArray[k]->m_MakeCopy );

	}
}

