/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2010, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef _EXTERNAL_PROGRAMS_H_
#define _EXTERNAL_PROGRAMS_H_

#include <vector>

#include <wx/string.h>


// this class stores the data about a single external program
class CSingleExternalProgram {

	friend class CExternalPrograms;

public:

	CSingleExternalProgram() {}
	
	/// copy constructor
	CSingleExternalProgram( const CSingleExternalProgram& data );

	/// assignment operator
	void operator=(const CSingleExternalProgram& data);

    void Copy(const CSingleExternalProgram& data);

	const wxString& GetCommandLine() { return m_CommandLine; }
	const wxString& GetDescription() { return m_Description; }
	const bool GetMakeCopy() { return m_MakeCopy; }

private:

	wxString m_CommandLine;		// command line to execute
	wxString m_Description;		// description to show in the menu
	// if false send the selected image to the external program
	// if true make a copy of the file and send it to the external program
	bool m_MakeCopy;
};

/*
this class handles the list of fixed external programs
*/
class CExternalPrograms
{
public:

	enum Error {
		eNoError,
		eDuplicateCommandLine,		/* the new command line is already present in the list */
		eDuplicateDescription,		/* the new description is already present in the list */
		eMissingCommandLine,		/* the command line is missing*/
		eMissingDescription			/* the description is missing */
	};

	CExternalPrograms(void);
	~CExternalPrograms(void);

	/// copy constructor
	CExternalPrograms( const CExternalPrograms& data );

	/// assignment operator
	void operator=(const CExternalPrograms& data);

    void Copy(const CExternalPrograms& data);

	// remove everything from the array
	void Clear();

	// return an element of the array
	CSingleExternalProgram& operator[]( size_t index ) const { return *dataArray[index]; }

	// return the number of elements in the array
	size_t size() const { return dataArray.size(); }

	// append a new fixed location at the end of the container
	Error Append( wxString commandLine, wxString description, bool makeCopy );

	// insert the data at a given index, overwriting the old data
	Error Edit( wxString commandLine, wxString description, bool makeCopy, int index );

	// remove the row at the given index from the container
	void Remove( int index );

	// load locations from the user profile.
	void Load();

	// save locations to the user profile.
	void Save();

private:
	
	std::vector<CSingleExternalProgram*> dataArray;

	// check if the data do not conflict with the data already contained in the list
	// if this is a new row index = -1 (impossible value)
	// if this is an edited line index is the index of the data to be changed
	Error CheckData( wxString commandLine, wxString description, bool makeCopy, int index );

};

#endif
