/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2009, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        fs_imageviewer.cpp
// Purpose:     A control to show images
// Author:      Fulvio Senore
// Modified by: 
// Created:     04/08/2009 21:08:29
// RCS-ID:      
// Copyright:   
// Licence:     wxWidgets licence
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "fs_imageviewer.h"
#include "fs_image.h"
#include "vvvpapp.h"

////@begin XPM images
////@end XPM images

DEFINE_EVENT_TYPE(fsEVT_COMMAND_ZOOM_CHANGED)
IMPLEMENT_DYNAMIC_CLASS(fsImageViewerEvent, wxNotifyEvent)

/*
 * FS_ImageViewer type definition
 */

IMPLEMENT_DYNAMIC_CLASS( FS_ImageViewer, wxScrolledWindow )


/*
 * FS_ImageViewer event table definition
 */

BEGIN_EVENT_TABLE( FS_ImageViewer, wxScrolledWindow )

////@begin FS_ImageViewer event table entries
    EVT_PAINT( FS_ImageViewer::OnPaint )
    EVT_ERASE_BACKGROUND( FS_ImageViewer::OnEraseBackground )
    EVT_LEFT_DOWN( FS_ImageViewer::OnLeftDown )
    EVT_LEFT_UP( FS_ImageViewer::OnLeftUp )
    EVT_MOTION( FS_ImageViewer::OnMotion )
    EVT_MOUSEWHEEL( FS_ImageViewer::OnMouseWheel )

    EVT_CHAR( FS_ImageViewer::OnChar )
////@end FS_ImageViewer event table entries

	EVT_KEY_DOWN(FS_ImageViewer::OnKeyDown)
    EVT_KEY_UP(FS_ImageViewer::OnKeyUp)

    EVT_SIZE( FS_ImageViewer::OnSize )

END_EVENT_TABLE()


/*
 * FS_ImageViewer constructors
 */

FS_ImageViewer::FS_ImageViewer()
{
    Init();
}

FS_ImageViewer::FS_ImageViewer(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style)
{
    Init();
    Create(parent, id, pos, size, style);
}


/*
 * FS_ImageViewer creator
 */

bool FS_ImageViewer::Create(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style)
{
////@begin FS_ImageViewer creation
    wxScrolledWindow::Create(parent, id, pos, size, style);
    CreateControls();
////@end FS_ImageViewer creation

    SetScrollRate( 1, 1 );
	SetVirtualSize( 100000, 100000 );
    SetBackgroundColour( *wxBLACK );
	
	// this is a hack because in Linux if the viewer window goes full screen accelerator keys do not work anymore
	// setting the focus to the parent window makes them work again
#ifdef __WXGTK__
	GetParent()->SetFocus();
#endif
    
	return true;
}


/*
 * FS_ImageViewer destructor
 */

FS_ImageViewer::~FS_ImageViewer()
{
////@begin FS_ImageViewer destruction
////@end FS_ImageViewer destruction
	delete m_HandCursor;
}


/*
 * Member initialisation
 */

void FS_ImageViewer::Init()
{
////@begin FS_ImageViewer member initialisation
////@end FS_ImageViewer member initialisation
	m_ZoomLevel = 100;
	m_ZoomDelta = 20;
	m_MinZoomLevel = 10;
	m_MaxZoomLevel = 200;
	m_wScrollbarH = wxSystemSettings::GetMetric( wxSYS_HSCROLL_Y );
	m_wScrollbarV = wxSystemSettings::GetMetric( wxSYS_VSCROLL_X );
	m_HandCursor = new wxCursor( wxCURSOR_CROSS );
	m_LastCommandWasFit = false;
	m_AutoFit = false;
	m_ImageAndBitmapOutOfSync = true;
	m_OutputMode = omImage;
	m_OutputMessage = wxEmptyString;
}


/*
 * Control creation for FS_ImageViewer
 */

void FS_ImageViewer::CreateControls()
{    
////@begin FS_ImageViewer content construction
    FS_ImageViewer* itemScrolledWindow1 = this;

    this->SetScrollbars(1, 1, 0, 0);
////@end FS_ImageViewer content construction
}


/*
 * Should we show tooltips?
 */

bool FS_ImageViewer::ShowToolTips()
{
    return true;
}

/*
 * Get bitmap resources
 */

wxBitmap FS_ImageViewer::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin FS_ImageViewer bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end FS_ImageViewer bitmap retrieval
}

/*
 * Get icon resources
 */

wxIcon FS_ImageViewer::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin FS_ImageViewer icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end FS_ImageViewer icon retrieval
}


/*
 * wxEVT_PAINT event handler for ID_SCROLLEDWINDOW
 */

void FS_ImageViewer::OnPaint( wxPaintEvent& event )
{
    wxPaintDC dc(this);

	if( m_OutputMode == omText ) {
		// draw text and return
		SetVirtualSize( 10, 10 );
		dc.SetTextForeground( *wxWHITE );
		dc.SetTextBackground( GetBackgroundColour() );
		dc.DrawText( m_OutputMessage, 10, 10 );
		return;
	}

    DoPrepareDC( dc );

	if( !m_ScaledBitmap.Ok() ) return;

	// only draw the needed part of the bitmap
	
	// update rect in device coords
	wxRect rectUpdateDevice = GetUpdateRegion().GetBox();
	
	// update rect in logical coords (those of the bitmap)
	wxRect rectUpdateLogical = rectUpdateDevice;
    CalcUnscrolledPosition(rectUpdateDevice.x, rectUpdateDevice.y, &rectUpdateLogical.x, &rectUpdateLogical.y);

	// check that the update rect does no go outside the bitmap (it can happen if the window is larger than the image)
	if( rectUpdateLogical.x + rectUpdateLogical.width > m_ScaledBitmap.GetWidth() )
		rectUpdateLogical.width = m_ScaledBitmap.GetWidth() - rectUpdateLogical.x;
	if( rectUpdateLogical.y + rectUpdateLogical.height > m_ScaledBitmap.GetHeight() )
		rectUpdateLogical.height = m_ScaledBitmap.GetHeight() - rectUpdateLogical.y;

	if( rectUpdateLogical.width <= 0 || rectUpdateLogical.height <= 0 ) return;

	// portion of the bitmap to draw
	wxBitmap bmpDraw = m_ScaledBitmap.GetSubBitmap( rectUpdateLogical );

	// if the image is smaller than the window center it
	int h, w;
	GetClientSize( &w, &h );
	int x = rectUpdateLogical.x;
	if( m_ScaledBitmap.GetWidth() < w )
		x += (w - m_ScaledBitmap.GetWidth())/2;
	int y = rectUpdateLogical.y;
	if( m_ScaledBitmap.GetHeight() < h )
		y += (h - m_ScaledBitmap.GetHeight())/2;

	dc.DrawBitmap( bmpDraw, x, y );
}

void FS_ImageViewer::SetImage( wxImage img )
{
	m_OutputMode = omImage;
	m_OutputMessage = wxEmptyString;
	m_Image = img;
	m_ImageAndBitmapOutOfSync = true;
}

void FS_ImageViewer::CreateScaledBitmap( bool noScrollbars, bool verticalScrollbar, bool horizontalScrollbar )
{
	wxBusyCursor bc;

	wxImage scaledImage;
	if( m_ZoomLevel == 100 ) {
		scaledImage = m_Image;
	}
	else {
		wxSize scaledSize( static_cast<int>(m_Image.GetWidth()*m_ZoomLevel/100.0 + 0.5), static_cast<int>(m_Image.GetHeight()*m_ZoomLevel/100.0 + 0.5) );
//		scaledImage = m_Image.Scale( scaledSize.GetWidth(), scaledSize.GetHeight(), wxIMAGE_QUALITY_HIGH );
		fsImage *ip = static_cast<fsImage*>( &m_Image );
		scaledImage = ip->ScaleFS( scaledSize.GetWidth(), scaledSize.GetHeight() );
//		scaledImage = ip->Scale( scaledSize.GetWidth(), scaledSize.GetHeight(), wxIMAGE_QUALITY_HIGH );
	}

	// make sure that the image is not smaller than the window by adding a border if needed
	int h, w;
	GetClientSize( &w, &h );
	if( verticalScrollbar ) {
		// leave room for a vertical scrollbar
		w -= m_wScrollbarV;
	}
	if( horizontalScrollbar ) {
		// leave room for an horizontal scrollbar
		h -= m_wScrollbarH;
	}
	if( scaledImage.GetWidth() < w || scaledImage.GetHeight() < h ) {
		wxSize windowSize( w, h );
		wxSize imageSize( scaledImage.GetWidth(), scaledImage.GetHeight() );
		wxSize finalSize = imageSize;
		finalSize.IncTo( windowSize );

		wxPoint pos( 0, 0 );
		if( imageSize.x < w )
			pos.x = (w - imageSize.x) / 2;
		if( imageSize.y < h )
			pos.y = (h - imageSize.y) / 2;
		wxColour col = GetBackgroundColour();
		scaledImage.Resize( finalSize, pos, col.Red(), col.Green(), col.Blue() );
	}
	m_ScaledBitmap = wxBitmap( scaledImage );

	if( noScrollbars ) {
		// make sure that there are no scrollbars
		SetVirtualSize( 10, 10 );
		SetVirtualSize( m_ScaledBitmap.GetWidth(), m_ScaledBitmap.GetHeight() );
	}
	else {
		SetVirtualSize( m_ScaledBitmap.GetWidth(), m_ScaledBitmap.GetHeight() );
	}

	SetScrollRate( 1, 1 );
	Refresh();
	m_ImageAndBitmapOutOfSync = false;
}


/*
 * wxEVT_ERASE_BACKGROUND event handler for ID_SCROLLEDWINDOW
 */

void FS_ImageViewer::OnEraseBackground( wxEraseEvent& event )
{
	// empty handler to prevent flicker unless we are showing a text message
	if( m_OutputMode == omText )
		event.Skip();
}

void FS_ImageViewer::SetZoomLevel( float zoom )
{
	if( zoom == m_ZoomLevel ) return;

	m_ZoomLevel = zoom;
	m_LastCommandWasFit = false;
	CreateScaledBitmap();
	PostZoomChangedEvent();
}


/*
 * wxEVT_LEFT_DOWN event handler for ID_SCROLLEDWINDOW
 */

void FS_ImageViewer::OnLeftDown( wxMouseEvent& event )
{
	m_LastMouseCoords = event.GetPosition();
	if( !IsWholeImageVisible() ) {
		CaptureMouse();
		SetCursor( *m_HandCursor );
	}
}


/*
 * wxEVT_LEFT_UP event handler for ID_SCROLLEDWINDOW
 */

void FS_ImageViewer::OnLeftUp( wxMouseEvent& event )
{
//	if( !IsWholeImageVisible() ) {
	if( HasCapture() ) {
		ReleaseMouse();
		SetCursor( wxNullCursor );
	}
#ifdef __WXGTK__
	GetParent()->SetFocus();
#endif
}


/*
 * wxEVT_MOTION event handler for ID_SCROLLEDWINDOW
 */

void FS_ImageViewer::OnMotion( wxMouseEvent& event )
{
	if( IsWholeImageVisible() || !event.LeftIsDown() ) {
		event.Skip();
		return;
	}
	
	wxPoint curCoords = event.GetPosition();

	// compute how much the mouse has moved
	int deltaX = m_LastMouseCoords.x - curCoords.x;
	int deltaY = m_LastMouseCoords.y - curCoords.y;

	// get current view position
	int xTop, yTop;
	GetViewStart( &xTop, &yTop );

	// scroll to the new position
	Scroll( xTop + deltaX, yTop + deltaY );

	// remember the position
	m_LastMouseCoords = curCoords;
}

bool FS_ImageViewer::IsWholeImageVisible()
{
	int h, w;

	if( !m_ScaledBitmap.Ok() ) return true;

	GetClientSize( &w, &h );

	return (m_ScaledBitmap.GetWidth() <= w && m_ScaledBitmap.GetHeight() <= h);
}

bool FS_ImageViewer::IsBackgroundVisible()
{
	int h, w;

	if( !m_ScaledBitmap.Ok() ) return true;

	GetClientSize( &w, &h );

	return (m_ScaledBitmap.GetWidth() <= w || m_ScaledBitmap.GetHeight() <= h);
}

void FS_ImageViewer::FitHorizontal()
{
	bool verticalScrollbar;
	ComputeZoomLevelForHorizontalFit( m_ZoomLevel, verticalScrollbar );
	m_LastCommandWasFit = false;
	CreateScaledBitmap( false, verticalScrollbar, false );
	PostZoomChangedEvent();
}

void FS_ImageViewer::FitVertical()
{
	bool horizontalScrollbar;
	ComputeZoomLevelForVerticalFit( m_ZoomLevel, horizontalScrollbar );
	m_LastCommandWasFit = false;
	CreateScaledBitmap( false, false, horizontalScrollbar );
	PostZoomChangedEvent();
}

void FS_ImageViewer::Fit()
{
	int h, w;
	float newZoomLevel;

	if( !m_Image.Ok() ) return;
	
	// make sure that there are no scrollbars
	SetVirtualSize( 10, 10 );
	GetClientSize( &w, &h );

	float zoomH = w*100.0/m_Image.GetWidth();
	float zoomV = h*100.0/m_Image.GetHeight();

	if( zoomH < zoomV )
		newZoomLevel = zoomH;
	else
		newZoomLevel = zoomV;

	if( newZoomLevel != m_ZoomLevel || m_ImageAndBitmapOutOfSync ) {
		// to avoid unneeded recreation of the bitmap
		m_ZoomLevel = newZoomLevel;
		CreateScaledBitmap( true );
		PostZoomChangedEvent();
	}

	m_LastCommandWasFit = true;
}

void FS_ImageViewer::ComputeZoomLevelForHorizontalFit( float &zoom, bool &verticalScrollbar )
{
	int h, w;

	SetVirtualSize( 10, 10 );	// make sure to get the size without scrollbars
	GetClientSize( &w, &h );

	float imageRatio = m_Image.GetHeight()*1.0/m_Image.GetWidth();
	float windowRatio = (h + m_wScrollbarH)*1.0/(w + m_wScrollbarV);

	verticalScrollbar = false;
	if( imageRatio > windowRatio ) {
		// the image is taller than the window so we will have a vertical scrollbar if we fit horizontally
		w -= m_wScrollbarV;
		verticalScrollbar = true;
	}

	zoom = w*100.0/m_Image.GetWidth();
}

void FS_ImageViewer::ComputeZoomLevelForVerticalFit( float &zoom, bool &horizontalScrollbar )
{
	int h, w;

	SetVirtualSize( 10, 10 );	// make sure to get the size without scrollbars
	GetClientSize( &w, &h );

	float imageRatio = m_Image.GetHeight()*1.0/m_Image.GetWidth();
	float windowRatio = (h + m_wScrollbarH)*1.0/(w + m_wScrollbarV);

	horizontalScrollbar = false;
	if( imageRatio < windowRatio ) {
		// the image is not taller than the window so we will have an horizontal scrollbar if we fit vertically
		h -= m_wScrollbarH;
		horizontalScrollbar = true;
	}

	zoom = h*100.0/m_Image.GetHeight();
}

void FS_ImageViewer::ZoomIn()
{
//	float newZoom = m_ZoomLevel + m_ZoomDelta;
	float newZoom = m_ZoomLevel*(1 + m_ZoomDelta/100.0);
	if( newZoom > m_MaxZoomLevel ) return;

	m_ZoomLevel = newZoom;
	m_LastCommandWasFit = false;
	CreateScaledBitmap();
	PostZoomChangedEvent();
}


void FS_ImageViewer::ZoomOut()
{
//	float newZoom = m_ZoomLevel - m_ZoomDelta;
	float newZoom = m_ZoomLevel/(1 + m_ZoomDelta/100.0);
	if( newZoom < m_MinZoomLevel ) return;

	m_ZoomLevel = newZoom;
	m_LastCommandWasFit = false;
	CreateScaledBitmap();
	PostZoomChangedEvent();
}

/*
 * wxEVT_MOUSEWHEEL event handler for ID_FSVIEWER_WINDOW
 */

void FS_ImageViewer::OnMouseWheel( wxMouseEvent& event )
{
////@begin wxEVT_MOUSEWHEEL event handler for ID_FSVIEWER_WINDOW in FS_ImageViewer.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_MOUSEWHEEL event handler for ID_FSVIEWER_WINDOW in FS_ImageViewer. 
}

void FS_ImageViewer::PostZoomChangedEvent()
{
	fsImageViewerEvent event( fsEVT_COMMAND_ZOOM_CHANGED, GetId() );
	event.SetEventObject( this );
	GetEventHandler()->ProcessEvent( event );
}

void FS_ImageViewer::OnSize( wxSizeEvent& event )
{
	event.Skip();
	if( m_AutoFit && m_LastCommandWasFit )
		Fit();
}

void FS_ImageViewer::ShowMessage( wxString msg )
{
	m_OutputMode = omText;
	m_OutputMessage = msg;
	Refresh();
}

void FS_ImageViewer::OnKeyDown( wxKeyEvent& event )
{
	// keydown event is used to handle ctrl+ and ctrl- in the numeric keypad that do not cause EVT_CHAR
	int kc = event.GetKeyCode();
	switch( kc ) {
		case WXK_NUMPAD_ADD:
			ZoomIn();
			break;
		case WXK_NUMPAD_SUBTRACT:
			ZoomOut();
			break;
		default:
			event.Skip();
			break;
	}
}

void FS_ImageViewer::OnKeyUp( wxKeyEvent& event )
{
	// placeholder for future code
	event.Skip();
}

/*
 * wxEVT_CHAR event handler for ID_FSVIEWER_WINDOW
 */

void FS_ImageViewer::OnChar( wxKeyEvent& event )
{
	int kc = event.GetKeyCode();
	switch( kc ) {
		case '+':
			ZoomIn();
			break;
		case '-':
			ZoomOut();
			break;
		default:
			// make the event propagate up two levels until the frame that will eventually process it
			event.ResumePropagation( 2 );
			event.Skip();
			break;
	}
}


