/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2009, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        image_information_dialog.cpp
// Purpose:     
// Author:      
// Modified by: 
// Created:     17/04/2009 10:18:58
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
#include "wx/mstream.h"
#include "wx/imaglist.h"
////@end includes

#include "image_information_dialog.h"
#include "fsdb.h"
#include "vvvpapp.h"
#include "sqlstrings.h"
#include "filedata_array.h"

////@begin XPM images
////@end XPM images


/*!
 * CImageInformationDialog type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CImageInformationDialog, wxDialog )


/*!
 * CImageInformationDialog event table definition
 */

BEGIN_EVENT_TABLE( CImageInformationDialog, wxDialog )

////@begin CImageInformationDialog event table entries
    EVT_SIZE( CImageInformationDialog::OnSize )

    EVT_BUTTON( wxID_OK, CImageInformationDialog::OnOkClick )

    EVT_BUTTON( wxID_HELP, CImageInformationDialog::OnHelpClick )

////@end CImageInformationDialog event table entries

END_EVENT_TABLE()


/*!
 * CImageInformationDialog constructors
 */

CImageInformationDialog::CImageInformationDialog()
{
    Init();
}

CImageInformationDialog::CImageInformationDialog( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * CImageInformationDialog creator
 */

bool CImageInformationDialog::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CImageInformationDialog creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CImageInformationDialog creation

	m_WindowPosition.SetWindow( this );
	m_WindowPosition.RestorePosition();

	ResizeLCColumns();

    m_TextCtrl->SetFocus();

	return true;
}


/*!
 * CImageInformationDialog destructor
 */

CImageInformationDialog::~CImageInformationDialog()
{
////@begin CImageInformationDialog destruction
////@end CImageInformationDialog destruction

	m_WindowPosition.SavePosition();
}


/*!
 * Member initialisation
 */

void CImageInformationDialog::Init()
{
////@begin CImageInformationDialog member initialisation
    m_ListCtrl = NULL;
    m_TextCtrl = NULL;
////@end CImageInformationDialog member initialisation
	m_fileData = NULL;
}


/*!
 * Control creation for CImageInformationDialog
 */

void CImageInformationDialog::CreateControls()
{    
////@begin CImageInformationDialog content construction
    CImageInformationDialog* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    m_ListCtrl = new wxListCtrl( itemDialog1, ID_LISTCTRL1, wxDefaultPosition, wxSize(400, 200), wxLC_REPORT|wxLC_NO_SORT_HEADER|wxSUNKEN_BORDER );
    itemBoxSizer2->Add(m_ListCtrl, 3, wxGROW|wxALL, 5);

    wxStaticText* itemStaticText4 = new wxStaticText( itemDialog1, wxID_STATIC, _("Description:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer2->Add(itemStaticText4, 0, wxALIGN_LEFT|wxALL, 5);

    m_TextCtrl = new wxTextCtrl( itemDialog1, ID_TEXTCTRL, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE );
    itemBoxSizer2->Add(m_TextCtrl, 1, wxGROW|wxALL, 5);

    wxStdDialogButtonSizer* itemStdDialogButtonSizer6 = new wxStdDialogButtonSizer;

    itemBoxSizer2->Add(itemStdDialogButtonSizer6, 0, wxGROW|wxALL, 10);
    wxButton* itemButton7 = new wxButton( itemDialog1, wxID_OK, _("&OK"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer6->AddButton(itemButton7);

    wxButton* itemButton8 = new wxButton( itemDialog1, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer6->AddButton(itemButton8);

    wxButton* itemButton9 = new wxButton( itemDialog1, wxID_HELP, _("&Help"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer6->AddButton(itemButton9);

    itemStdDialogButtonSizer6->Realize();

////@end CImageInformationDialog content construction

	CreateLCHeaders();
	ShowFileData();
}


/*!
 * Should we show tooltips?
 */

bool CImageInformationDialog::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CImageInformationDialog::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CImageInformationDialog bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CImageInformationDialog bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CImageInformationDialog::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CImageInformationDialog icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CImageInformationDialog icon retrieval
}

void CImageInformationDialog::SetFileData( CFileData* fileData ) {

	m_fileData = fileData;
}

void CImageInformationDialog::CreateLCHeaders() {
	wxListItem itemCol;
	int i = 0;

	itemCol.SetText( _("Name") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	m_ListCtrl->InsertColumn( i++, itemCol );

	itemCol.SetText( _("Value") );
	itemCol.SetAlign( wxLIST_FORMAT_LEFT );
	m_ListCtrl->InsertColumn( i++, itemCol );
}

void CImageInformationDialog::ShowFileData() {

	wxASSERT( m_fileData != NULL );
	m_ListCtrl->DeleteAllItems();

	CMetadata md = m_fileData->GetMetadata();

	// each row is inserted at the top, so we add them in reverse order

	AddLCRow( _("White balance"), md.GetWhiteBalance() );
	AddLCRow( _("Image quality"), md.GetImageQuality() );
	AddLCRow( _("Metering mode"), md.GetMeteringMode() );
	AddLCRow( _("Exposure mode"), md.GetExposureMode() );
	AddLCRow( _("ISO"), md.GetISOSpeed() );
	AddLCRow( _("35mm eq."), md.GetFocalLength35mm() );
	AddLCRow( _("Focal length"), md.GetFocalLength() );
	AddLCRow( _("Flash"), md.GetFlash() );
	AddLCRow( _("Exposure bias"), md.GetExposureBias() );
	AddLCRow( _("Max aperture"), md.GetMaxAperture() );
	AddLCRow( _("Aperture"), md.GetAperture() );
	AddLCRow( _("Exposure time"), md.GetExposureTime() );
	AddLCRow( _("Software"), md.GetSoftware() );
	AddLCRow( _("Model"), md.GetCameraModel() );
	AddLCRow( _("Make"), md.GetCameraMake() );
	wxDateTime dt = md.GetImageTimeStamp();
	if( dt.IsValid() ) {
		AddLCRow( _("Image timestamp"), dt.FormatDate() + wxT(" ") + dt.FormatTime() );
	}

	AddLCRow( wxEmptyString, wxEmptyString );

	if( m_fileData->HasThumbnail() ) {
		wxString s;
		if( m_fileData->GetRotation() == CUtils::irNoRotation || m_fileData->GetRotation() == CUtils::irUpsideDown )
			s = CUtils::long2string(m_fileData->GetThumbnailWidth()) + wxT("x") + CUtils::long2string(m_fileData->GetThumbnailHeight());
		else
			s = CUtils::long2string(m_fileData->GetThumbnailHeight()) + wxT("x") + CUtils::long2string(m_fileData->GetThumbnailWidth());
		AddLCRow( _("Thumbnail size"), s );
	}
	else
		AddLCRow( _("Thumbnail size"), _("No thumbnail") );
	if( m_fileData->GetWidth() > 0 && m_fileData->GetHeight() > 0 ) {
		wxString s;
		if( m_fileData->GetRotation() == CUtils::irNoRotation || m_fileData->GetRotation() == CUtils::irUpsideDown )
			s = CUtils::long2string(m_fileData->GetWidth()) + wxT("x") + CUtils::long2string(m_fileData->GetHeight());
		else
			s = CUtils::long2string(m_fileData->GetHeight()) + wxT("x") + CUtils::long2string(m_fileData->GetWidth());
		AddLCRow( _("Image size"), s );
	}
	AddLCRow( _("File date"), m_fileData->GetFileDateTime().FormatDate() + wxT(" ") + m_fileData->GetFileDateTime().FormatTime() );
	AddLCRow( _("File size"), CUtils::HumanReadableFileSize(m_fileData->GetSize()) );
	AddLCRow( _("File name"), m_fileData->GetName() );

	AddLCRow( wxEmptyString, wxEmptyString );

	// virtual paths
	FSDB::Database db = wxGetApp().GetDatabase();
	{
		FSDB_CREATE_RS_READONLY_FWDONLY( db, rs );
		std::string sql = CSqlStrings::SelectVirtualPathsFromPhysicalFileID( m_fileData->GetID() );
		rs.Open( sql );
		while( !rs.IsEOF() ) {
			FSDB::Parameters params = FSDB::ParametersFactory();
			params->Append( static_cast<int>(rs["VIRTUAL_PATH_ID"]) );
			params->Append( CUtils::wx2std( wxString(wxFileName::GetPathSeparator()) ) );
			std::string sql2 = CSqlStrings::GetFullVirtualPath();
			std::string path = db->ExecuteReturnString( sql2, params );
			AddLCRow( _("Virtual location"), CUtils::DBstd2wx(path) );

			rs.MoveNext();
		}
	}

	// removable path
	{
		FSDB::Parameters params = FSDB::ParametersFactory();
		params->Append( m_fileData->GetID() );
		params->Append( CUtils::wx2std( wxString(wxFileName::GetPathSeparator()) ) );
		params->Append( "R" );
		std::string sql = CSqlStrings::GetFullPhysicalPathFromFile();
		std::string path = db->ExecuteReturnString( sql, params );
		AddLCRow( _("Removable location"), CUtils::DBstd2wx(path) );
	}
	// fixed path
	{
		FSDB::Parameters params = FSDB::ParametersFactory();
		params->Append( m_fileData->GetID() );
		params->Append( CUtils::wx2std( wxString(wxFileName::GetPathSeparator()) ) );
		params->Append( "F" );
		std::string sql = CSqlStrings::GetFullPhysicalPathFromFile();
		std::string path = db->ExecuteReturnString( sql, params );
		AddLCRow( _("Fixed location"), CUtils::DBstd2wx(path) );
	}

	m_TextCtrl->SetValue( m_fileData->GetDescription() );

}

int CImageInformationDialog::AddLCRow( wxString col0, wxString col1 ) 
{
  	int i = m_ListCtrl->InsertItem( 0, "" );

    // this code is used to set the item's mask
	wxListItem item;
	item.SetId( i );
	m_ListCtrl->GetItem( item );
	item.SetMask( wxLIST_MASK_STATE|wxLIST_MASK_TEXT|wxLIST_MASK_DATA );
	m_ListCtrl->SetItem( item );

	m_ListCtrl->SetItem( i, 0, col0 );
	m_ListCtrl->SetItem( i, 1, col1 );
	return i;
}

void CImageInformationDialog::ResizeLCColumns() {
	int x, y;
	int firstColumnWidth = 120;

    if( m_ListCtrl == NULL ) return;

    m_ListCtrl->GetClientSize( &x, &y );
	m_ListCtrl->SetColumnWidth( 0, firstColumnWidth );
	m_ListCtrl->SetColumnWidth( 1, x - firstColumnWidth - 2 );
}


/*!
 * wxEVT_SIZE event handler for ID_CIMAGEINFORMATIONDIALOG
 */

void CImageInformationDialog::OnSize( wxSizeEvent& event )
{
	event.Skip();
	ResizeLCColumns();
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_OK
 */

void CImageInformationDialog::OnOkClick( wxCommandEvent& WXUNUSED(event) )
{
	// if needed store the new description
	wxString curDescr = m_TextCtrl->GetValue();
	if( curDescr != m_fileData->GetDescription() ) {
		int fileID = m_fileData->GetID();
		std::string sql = CSqlStrings::UpdatePhysicalFileDescription( fileID, curDescr );
		FSDB::Database db = wxGetApp().GetDatabase();
		db->ExecuteNoReturn(sql);
		m_fileData->SetDescription( curDescr );
	}

	EndModal( wxID_OK );
}

wxString CImageInformationDialog::GetDescription( void ) {
	return m_TextCtrl->GetValue();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_HELP
 */

void CImageInformationDialog::OnHelpClick( wxCommandEvent& WXUNUSED(event) )
{
	wxGetApp().GetHelpController()->DisplaySection( wxT("image_information.htm") );
}

