/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2009, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// this class contains code to extract a preview from an image file

#ifndef __IMAGE_PREVIEW_H__
#define __IMAGE_PREVIEW_H__

#include "memory_chunk.h"
#include "metadata.h"

#include <wx/string.h>

// used to extract the largest possible preview
#define IMAGE_PREVIEW_LARGEST_SIZE -1

class CImagePreview
{
public:

	// return value for the following functions
	// rtNoPreviews: the file does not contain previews
	// rtPreviewsTooSmall: the previews are smaller than the requested size
	enum ReturnValue { rtOk = 0, rtNoPreviews, rtPreviewsTooSmall, rtPreviewCorrupted };

	// return a wxImage containing the preview immediately larger than the requested size, the embedded color profile and the color space defined by metadata (if the return value is rtOk )
	// the color space will be used if no color profile is embedded
	static ReturnValue ExtractImagePreview( const wxString &fileName, int requestedSize, wxImage &previewImage, CMemoryChunk &colorProfile, CMetadata::ColorSpace &exifColorSpace );

	// return a wxImage containing the largest preview available in the file, the embedded color profile and the color space defined by metadata (if the return value is rtOk )
	// the color space will be used if no color profile is embedded
	static ReturnValue ExtractLargestPreview( const wxString &fileName, wxImage &previewImage, CMemoryChunk &colorProfile, CMetadata::ColorSpace &exifColorSpace );

	// save the largest preview available in the file (if the return value is rtOk )
	// fileNameIn is the input file name
	// pathOut is the path where the preview file will be written. The file name is the same as the input file, the extension is determined by the preview type
	static ReturnValue ExtractLargestPreview( const wxString &fileNameIn, wxString &pathOut );

};


#endif
