/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2009, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    aint with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        image_viewer_frame.cpp
// Purpose:     
// Author:      
// Modified by: 
// Created:     10/08/2009 14:06:28
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
#include "wx/mstream.h"
#include "fs_imageviewer.h"
////@end includes

#include "image_information_dialog.h"
#include "image_viewer_frame.h"
#include "db_utils.h"
#include "mainframe.h"
#include "rightpanelist.h"
#include "vvvpapp.h"
#include "image_preview.h"
#include "catalog_images_functions.h"
#include "memory_chunk.h"
#include "fs_image.h"
#include "fs_lcms.h"

#include <wx/ffile.h>

////@begin XPM images
#include "graphics/vvvPicon32.inc"
#include "graphics/tlb_view_fullscreen.inc"
#include "graphics/tlb_go_first.inc"
#include "graphics/tlb_go_previous.inc"
#include "graphics/tlb_go_next.inc"
#include "graphics/tlb_go_last.inc"
#include "graphics/tlb_zoom_fit.inc"
#include "graphics/tlb_zoom_fit_horizontal.inc"
#include "graphics/tlb_zoom_fit_vertical.inc"
#include "graphics/tlb_zoom_original.inc"
#include "graphics/tlb_zoom_in.inc"
#include "graphics/tlb_zoom_out.inc"
////@end XPM images
#include "graphics/vvvPicon16.xpm"


/*!
 * CImageViewerFrame type definition
 */

IMPLEMENT_CLASS( CImageViewerFrame, wxFrame )


/*!
 * CImageViewerFrame event table definition
 */

BEGIN_EVENT_TABLE( CImageViewerFrame, wxFrame )

////@begin CImageViewerFrame event table entries
    EVT_CLOSE( CImageViewerFrame::OnCloseWindow )
    EVT_ERASE_BACKGROUND( CImageViewerFrame::OnEraseBackground )
    EVT_CHAR( CImageViewerFrame::OnChar )

    EVT_MENU( ID_IMGV_EDIT_IMAGE_INFORMATION, CImageViewerFrame::OnImgvEditImageInformationClick )

    EVT_MENU( ID_IMGV_VIEW_FULLSCREEN, CImageViewerFrame::OnImgvViewFullscreenClick )

    EVT_MENU( ID_IMGV_VIEW_FIT, CImageViewerFrame::OnImgvViewFitClick )

    EVT_MENU( ID_IMGV_VIEW_FIT_H, CImageViewerFrame::OnImgvViewFitHClick )

    EVT_MENU( ID_IMGV_VIEW_FIT_V, CImageViewerFrame::OnImgvViewFitVClick )

    EVT_MENU( ID_IMGV_VIEW_REAL_SIZE, CImageViewerFrame::OnImgvViewRealSizeClick )
    EVT_UPDATE_UI( ID_IMGV_VIEW_REAL_SIZE, CImageViewerFrame::OnImgvViewRealSizeUpdate )

    EVT_MENU( ID_IMGV_VIEW_ZOOM_IN, CImageViewerFrame::OnImgvViewZoomInClick )
    EVT_UPDATE_UI( ID_IMGV_VIEW_ZOOM_IN, CImageViewerFrame::OnImgvViewZoomInUpdate )

    EVT_MENU( ID_IMGV_VIEW_ZOOM_OUT, CImageViewerFrame::OnImgvViewZoomOutClick )
    EVT_UPDATE_UI( ID_IMGV_VIEW_ZOOM_OUT, CImageViewerFrame::OnImgvViewZoomOutUpdate )

    EVT_MENU( ID_IMGV_VIEW_FIRST, CImageViewerFrame::OnImgvViewFirstClick )
    EVT_UPDATE_UI( ID_IMGV_VIEW_FIRST, CImageViewerFrame::OnImgvViewFirstUpdate )

    EVT_MENU( ID_IMGV_VIEW_PREVIOUS, CImageViewerFrame::OnImgvViewPreviousClick )
    EVT_UPDATE_UI( ID_IMGV_VIEW_PREVIOUS, CImageViewerFrame::OnImgvViewPreviousUpdate )

    EVT_MENU( ID_IMGV_VIEW_NEXT, CImageViewerFrame::OnImgvViewNextClick )
    EVT_UPDATE_UI( ID_IMGV_VIEW_NEXT, CImageViewerFrame::OnImgvViewNextUpdate )

    EVT_MENU( ID_IMGV_VIEW_LAST, CImageViewerFrame::OnImgvViewLastClick )
    EVT_UPDATE_UI( ID_IMGV_VIEW_LAST, CImageViewerFrame::OnImgvViewLastUpdate )

    EVT_MENU( wxID_HELP, CImageViewerFrame::OnHelpClick )

////@end CImageViewerFrame event table entries

	EVT_COMMAND_ZOOM_CHANGED( ID_IMAGE_VIEWER_CTRL, CImageViewerFrame::OnViewerZoomChanged )

END_EVENT_TABLE()


/*!
 * CImageViewerFrame constructors
 */

CImageViewerFrame::CImageViewerFrame()
{
    Init();
}

CImageViewerFrame::CImageViewerFrame( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create( parent, id, caption, pos, size, style );
}


/*!
 * CImageViewerFrame creator
 */

bool CImageViewerFrame::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CImageViewerFrame creation
    wxFrame::Create( parent, id, caption, pos, size, style );

    CreateControls();
    SetIcon(GetIconResource(wxT("graphics/vvvPicon32.png")));
    Centre();
////@end CImageViewerFrame creation

#ifdef __WXGTK__
	// add more than one icon to the Linux version
	wxIconBundle ib( GetIcon() );
	ib.AddIcon( vvvPicon16_xpm );
	SetIcons( ib );
#endif

	m_ImageViewerCtrl->SetAutoFit( true );

	m_WindowPosition.SetWindow( this );
	m_WindowPosition.RestorePosition();

	Layout();
    return true;
}


/*!
 * CImageViewerFrame destructor
 */

CImageViewerFrame::~CImageViewerFrame()
{
////@begin CImageViewerFrame destruction
////@end CImageViewerFrame destruction

	m_WindowPosition.SavePosition();
}


/*!
 * Member initialisation
 */

void CImageViewerFrame::Init()
{
////@begin CImageViewerFrame member initialisation
    m_ToolbarCtrl = NULL;
    m_ViewerPanel = NULL;
    m_ViewerSizer = NULL;
    m_ImageViewerCtrl = NULL;
    m_StatusBar = NULL;
////@end CImageViewerFrame member initialisation

	m_FileDataArray = NULL;
	m_FileDataIndex = -1;
	m_WasMaximized = false;

	SetupColorManagement();
}


/*!
 * Control creation for CImageViewerFrame
 */

void CImageViewerFrame::CreateControls()
{    
////@begin CImageViewerFrame content construction
    CImageViewerFrame* itemFrame1 = this;

    wxMenuBar* menuBar = new wxMenuBar;
    wxMenu* itemMenu17 = new wxMenu;
    itemMenu17->Append(ID_IMGV_EDIT_IMAGE_INFORMATION, _("Image Information...\tCtrl-I"), wxEmptyString, wxITEM_NORMAL);
    menuBar->Append(itemMenu17, _("Edit"));
    wxMenu* itemMenu19 = new wxMenu;
    itemMenu19->Append(ID_IMGV_VIEW_FULLSCREEN, _("Full Screen\tCtrl-F"), wxEmptyString, wxITEM_NORMAL);
    itemMenu19->AppendSeparator();
    itemMenu19->Append(ID_IMGV_VIEW_FIT, _("Fit"), wxEmptyString, wxITEM_NORMAL);
    itemMenu19->Append(ID_IMGV_VIEW_FIT_H, _("Fit Width"), wxEmptyString, wxITEM_NORMAL);
    itemMenu19->Append(ID_IMGV_VIEW_FIT_V, _("Fit Height"), wxEmptyString, wxITEM_NORMAL);
    itemMenu19->Append(ID_IMGV_VIEW_REAL_SIZE, _("Real Size"), wxEmptyString, wxITEM_NORMAL);
    itemMenu19->Append(ID_IMGV_VIEW_ZOOM_IN, _("Zoom In"), wxEmptyString, wxITEM_NORMAL);
    itemMenu19->Append(ID_IMGV_VIEW_ZOOM_OUT, _("Zoom Out"), wxEmptyString, wxITEM_NORMAL);
    itemMenu19->AppendSeparator();
    itemMenu19->Append(ID_IMGV_VIEW_FIRST, _("First Image"), wxEmptyString, wxITEM_NORMAL);
    itemMenu19->Append(ID_IMGV_VIEW_PREVIOUS, _("Previous Image"), wxEmptyString, wxITEM_NORMAL);
    itemMenu19->Append(ID_IMGV_VIEW_NEXT, _("Next Image"), wxEmptyString, wxITEM_NORMAL);
    itemMenu19->Append(ID_IMGV_VIEW_LAST, _("Last Image"), wxEmptyString, wxITEM_NORMAL);
    menuBar->Append(itemMenu19, _("View"));
    wxMenu* itemMenu33 = new wxMenu;
    itemMenu33->Append(wxID_HELP, _("Image Viewer Help"), wxEmptyString, wxITEM_NORMAL);
    menuBar->Append(itemMenu33, _("Help"));
    itemFrame1->SetMenuBar(menuBar);

    m_ToolbarCtrl = CreateToolBar( wxTB_FLAT|wxTB_HORIZONTAL|wxTB_TEXT, ID_IMGVIEWER_TOOLBAR );
    m_ToolbarCtrl->SetToolBitmapSize(wxSize(16, 16));
    wxBitmap itemtool3Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_view_fullscreen.png")));
    wxBitmap itemtool3BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_FULLSCREEN, _("Full Screen"), itemtool3Bitmap, itemtool3BitmapDisabled, wxITEM_NORMAL, _("Show full screen"), wxEmptyString);
    m_ToolbarCtrl->AddSeparator();
    wxBitmap itemtool5Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_go_first.png")));
    wxBitmap itemtool5BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_FIRST, _("First"), itemtool5Bitmap, itemtool5BitmapDisabled, wxITEM_NORMAL, _("Show the first image"), wxEmptyString);
    wxBitmap itemtool6Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_go_previous.png")));
    wxBitmap itemtool6BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_PREVIOUS, _("Previous"), itemtool6Bitmap, itemtool6BitmapDisabled, wxITEM_NORMAL, _("Show the previous image"), wxEmptyString);
    wxBitmap itemtool7Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_go_next.png")));
    wxBitmap itemtool7BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_NEXT, _("Next"), itemtool7Bitmap, itemtool7BitmapDisabled, wxITEM_NORMAL, _("Show the next image"), wxEmptyString);
    wxBitmap itemtool8Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_go_last.png")));
    wxBitmap itemtool8BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_LAST, _("Last"), itemtool8Bitmap, itemtool8BitmapDisabled, wxITEM_NORMAL, _("Show the last image"), wxEmptyString);
    m_ToolbarCtrl->AddSeparator();
    wxBitmap itemtool10Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_fit.png")));
    wxBitmap itemtool10BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_FIT, _("Fit"), itemtool10Bitmap, itemtool10BitmapDisabled, wxITEM_NORMAL, _("Fit image to window"), wxEmptyString);
    wxBitmap itemtool11Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_fit_horizontal.png")));
    wxBitmap itemtool11BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_FIT_H, _("Fit Width"), itemtool11Bitmap, itemtool11BitmapDisabled, wxITEM_NORMAL, _("Fit image width to window"), wxEmptyString);
    wxBitmap itemtool12Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_fit_vertical.png")));
    wxBitmap itemtool12BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_FIT_V, _("Fit Height"), itemtool12Bitmap, itemtool12BitmapDisabled, wxITEM_NORMAL, _("Fit image height to window"), wxEmptyString);
    wxBitmap itemtool13Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_original.png")));
    wxBitmap itemtool13BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_REAL_SIZE, _("Real Size"), itemtool13Bitmap, itemtool13BitmapDisabled, wxITEM_NORMAL, _("Show image at real size"), wxEmptyString);
    wxBitmap itemtool14Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_in.png")));
    wxBitmap itemtool14BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_ZOOM_IN, _("Zoom In"), itemtool14Bitmap, itemtool14BitmapDisabled, wxITEM_NORMAL, wxEmptyString, wxEmptyString);
    wxBitmap itemtool15Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_out.png")));
    wxBitmap itemtool15BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_ZOOM_OUT, _("Zoom Out"), itemtool15Bitmap, itemtool15BitmapDisabled, wxITEM_NORMAL, wxEmptyString, wxEmptyString);
    m_ToolbarCtrl->Realize();
    itemFrame1->SetToolBar(m_ToolbarCtrl);

    m_ViewerPanel = new wxPanel( itemFrame1, ID_PANEL_VIEWER, wxDefaultPosition, wxDefaultSize, wxNO_BORDER );

    wxBoxSizer* itemBoxSizer36 = new wxBoxSizer(wxVERTICAL);
    m_ViewerPanel->SetSizer(itemBoxSizer36);

    m_ViewerSizer = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer36->Add(m_ViewerSizer, 1, wxGROW|wxALL, 0);

    m_ImageViewerCtrl = new FS_ImageViewer( m_ViewerPanel, ID_IMAGE_VIEWER_CTRL, wxDefaultPosition, wxSize(100, 100), wxDOUBLE_BORDER|wxHSCROLL|wxVSCROLL );
    m_ViewerSizer->Add(m_ImageViewerCtrl, 1, wxGROW|wxALL, 0);
    m_ImageViewerCtrl->SetScrollbars(1, 1, 0, 0);

    m_StatusBar = new wxStatusBar( itemFrame1, ID_IMGVIEWER_STATUSBAR, wxST_SIZEGRIP|wxNO_BORDER );
    m_StatusBar->SetFieldsCount(6);
    int m_StatusBarWidths[6];
    m_StatusBarWidths[0] = 50;
    m_StatusBarWidths[1] = 100;
    m_StatusBarWidths[2] = 60;
    m_StatusBarWidths[3] = 60;
    m_StatusBarWidths[4] = 160;
    m_StatusBarWidths[5] = -1;
    m_StatusBar->SetStatusWidths(6, m_StatusBarWidths);
    itemFrame1->SetStatusBar(m_StatusBar);

////@end CImageViewerFrame content construction

    // create the viewer control for full screen
    m_ImageViewerInvisibleCtrl = new FS_ImageViewer( m_ViewerPanel, wxID_ANY, wxDefaultPosition, wxSize(100, 100), wxNO_BORDER|wxHSCROLL|wxVSCROLL );
    m_ViewerSizer->Add(m_ImageViewerInvisibleCtrl, 1, wxGROW|wxALL, 0);
    m_ImageViewerInvisibleCtrl->SetScrollbars(1, 1, 0, 0);
    m_ViewerSizer->Hide( m_ImageViewerInvisibleCtrl );

#if defined(__WXMAC__)
	// recreate the toolbar with larger bitmaps to make it look better in OS X
	delete m_ToolbarCtrl;
	SetToolBar( NULL );
	CreateMacToolbar();
#endif

}


/*!
 * Should we show tooltips?
 */

bool CImageViewerFrame::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CImageViewerFrame::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CImageViewerFrame bitmap retrieval
    wxUnusedVar(name);
    if (name == _T("graphics/tlb_view_fullscreen.png"))
    {
        wxMemoryInputStream memStream(tlb_view_fullscreen_png, sizeof(tlb_view_fullscreen_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_go_first.png"))
    {
        wxMemoryInputStream memStream(tlb_go_first_png, sizeof(tlb_go_first_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_go_previous.png"))
    {
        wxMemoryInputStream memStream(tlb_go_previous_png, sizeof(tlb_go_previous_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_go_next.png"))
    {
        wxMemoryInputStream memStream(tlb_go_next_png, sizeof(tlb_go_next_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_go_last.png"))
    {
        wxMemoryInputStream memStream(tlb_go_last_png, sizeof(tlb_go_last_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_zoom_fit.png"))
    {
        wxMemoryInputStream memStream(tlb_zoom_fit_png, sizeof(tlb_zoom_fit_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_zoom_fit_horizontal.png"))
    {
        wxMemoryInputStream memStream(tlb_zoom_fit_horizontal_png, sizeof(tlb_zoom_fit_horizontal_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_zoom_fit_vertical.png"))
    {
        wxMemoryInputStream memStream(tlb_zoom_fit_vertical_png, sizeof(tlb_zoom_fit_vertical_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_zoom_original.png"))
    {
        wxMemoryInputStream memStream(tlb_zoom_original_png, sizeof(tlb_zoom_original_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_zoom_in.png"))
    {
        wxMemoryInputStream memStream(tlb_zoom_in_png, sizeof(tlb_zoom_in_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_zoom_out.png"))
    {
        wxMemoryInputStream memStream(tlb_zoom_out_png, sizeof(tlb_zoom_out_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        return bitmap;
    }
    return wxNullBitmap;
////@end CImageViewerFrame bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CImageViewerFrame::GetIconResource( const wxString& name )
{
    // Icon retrieval
    wxUnusedVar(name);
    if (name == _T("graphics/vvvPicon32.png"))
    {
#ifdef __WXMSW__
		wxIcon icon( wxT("VVVP_ICON") );
#else
        wxMemoryInputStream memStream(vvvPicon32_png, sizeof(vvvPicon32_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        wxIcon icon;
        icon.CopyFromBitmap(bitmap);
#endif
		return icon;
    }
    return wxNullIcon;
}

void CImageViewerFrame::SetFiledataArray( CFileDataArray *fda )
{
	m_FileDataArray = fda;
}

void CImageViewerFrame::SetFiledataArrayIndex( int index )
{
	m_FileDataIndex = index;
}



void CImageViewerFrame::ShowCurrentImage()
{
	wxASSERT( m_FileDataArray != NULL );
	wxASSERT( m_FileDataIndex >= 0 );

	wxBusyCursor bc;
	
	CFileData &fd = (*m_FileDataArray)[m_FileDataIndex];
	int id = fd.GetID();
	wxString fileName = fd.GetName();

	// get the path to the fixed location
	wxString fullPath;
	CDbUtils::LocationsStatus st = CDbUtils::FixedLocationPathFromID( id, fileName, fullPath );

	// set the window caption
	SetTitle( fileName + wxT(" - ") + CUtils::GetApplicationName() );

	// select the new image in the main frame
	CMainFrame* mf = dynamic_cast<CMainFrame*>(GetParent());
	wxASSERT( mf != NULL );
	CRightPaneList* lp = mf->GetCurrentRightPaneListCtrl();
	lp->SelectNone();
	lp->Select( m_FileDataIndex );
	lp->EnsureVisible( m_FileDataIndex );

	// checks
	if( st == CDbUtils::lsMissingFixedLocation ) {
		m_ImageViewerCtrl->ShowMessage( _("This image does not have a fixed location") );
		return;
	}
	if( st == CDbUtils::lsUndefinedFixedLocation ) {
		m_ImageViewerCtrl->ShowMessage( _("This image has a fixed location, but the location is not defined in this computer") );
		return;
	}
	if( !wxFileExists(fullPath) ) {
		m_ImageViewerCtrl->ShowMessage( _("This image has not been found in the fixed location") );
		return;
	}

	// show the image
	fsImage img;
	CMemoryChunk colorProfile;
	bool usePreview = false;
	CMetadata::ColorSpace exifColorSpace = CMetadata::csUnknown;
	if( !CCatalogImagesFunctions::IsHandledImageType( fd.GetExt() ) ) {
		// do not use preview for jpeg and other standard formats: we read the full image file
		CImagePreview::ReturnValue r = CImagePreview::ExtractLargestPreview( fullPath, img, colorProfile, exifColorSpace );
		usePreview = (r == CImagePreview::rtOk );
	}
	if( !usePreview ) {
		// no preview, read the image file
		wxFFile fin( fullPath, wxT("rb") );
		wxUint8 *fileBuffer = new wxUint8[fin.Length()];
		fin.Read( fileBuffer, fin.Length() );
		wxMemoryInputStream mis( fileBuffer, fin.Length() );
		{
			wxLogNull ln;
			fsImage imgtmp( mis );
			colorProfile.SetData( imgtmp.GetICCProfileAddress(), imgtmp.GetICCProfileSize() );
			imgtmp.DeleteICCProfile();
			img = imgtmp;
		}
		// if no profile try to read color space from exif metadata
		if( colorProfile.GetAddress() == NULL ) {
			CUtils::ImageRotation imageRotation;
			wxDateTime imageDateTime;
			int imageWidth = -1, imageHeight = -1;
			CMetadata::ReadBasicMetadata( fileBuffer, fin.Length(), imageRotation, imageDateTime, imageWidth, imageHeight, exifColorSpace );
		}
		fin.Close();
		delete fileBuffer;
	}

	// convert the image to the output profile
	CAppSettings& as = wxGetApp().GetAppSettings();
	if( as.m_EnableColorManagement ) {
		if( exifColorSpace == CMetadata::csAdobeRGB ) {
			// the image does not contain a profile but Exif data say that it uses Adobe RGB
			m_ColorProfilesHandler.TransformImageFromAdobeRGB( img );
		}
		else {
			m_ColorProfilesHandler.TransformImage( img, colorProfile );
		}
	}

	if( img.IsOk() ) {
		wxImage rotImage = CUtils::RotateImage( img, fd.GetRotation() );
		m_ImageViewerCtrl->SetImage( rotImage );
		m_ImageViewerCtrl->Fit();
	}
	else {
		m_ImageViewerCtrl->ShowMessage( wxT("Unable to load the image, it is probably corrupted") );
	}

	UpdateStatusBar();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_FIT
 */

void CImageViewerFrame::OnImgvViewFitClick( wxCommandEvent& WXUNUSED(event) )
{
	m_ImageViewerCtrl->Fit();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_FIT_H
 */

void CImageViewerFrame::OnImgvViewFitHClick( wxCommandEvent& WXUNUSED(event) )
{
	m_ImageViewerCtrl->FitHorizontal();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_FIT_V
 */

void CImageViewerFrame::OnImgvViewFitVClick( wxCommandEvent& WXUNUSED(event) )
{
	m_ImageViewerCtrl->FitVertical();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_FIRST
 */

void CImageViewerFrame::OnImgvViewFirstClick( wxCommandEvent& WXUNUSED(event) )
{
	if( m_FileDataIndex > 0 ) {
		m_FileDataIndex = 0;
		ShowCurrentImage();
	}
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_FIRST
 */

void CImageViewerFrame::OnImgvViewFirstUpdate( wxUpdateUIEvent& event )
{
	event.Enable( m_FileDataIndex > 0 );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_PREVIOUS
 */

void CImageViewerFrame::OnImgvViewPreviousClick( wxCommandEvent& WXUNUSED(event) )
{
	MovePrevious();
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_PREVIOUS
 */

void CImageViewerFrame::OnImgvViewPreviousUpdate( wxUpdateUIEvent& event )
{
	event.Enable( m_FileDataIndex > 0 );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_NEXT
 */

void CImageViewerFrame::OnImgvViewNextClick( wxCommandEvent& WXUNUSED(event) )
{
	MoveNext();
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_NEXT
 */

void CImageViewerFrame::OnImgvViewNextUpdate( wxUpdateUIEvent& event )
{
	if( m_FileDataArray != NULL )
		event.Enable( m_FileDataIndex < m_FileDataArray->size() - 1 );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_LAST
 */

void CImageViewerFrame::OnImgvViewLastClick( wxCommandEvent& WXUNUSED(event) )
{
	if( m_FileDataIndex < m_FileDataArray->size() - 1 ) {
		m_FileDataIndex = m_FileDataArray->size() - 1;
		ShowCurrentImage();
	}
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_LAST
 */

void CImageViewerFrame::OnImgvViewLastUpdate( wxUpdateUIEvent& event )
{
	if( m_FileDataArray != NULL )
		event.Enable( m_FileDataIndex < m_FileDataArray->size() - 1 );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_REAL_SIZE
 */

void CImageViewerFrame::OnImgvViewRealSizeClick( wxCommandEvent& event )
{
	m_ImageViewerCtrl->SetZoomLevel( 100 );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_REAL_SIZE
 */

void CImageViewerFrame::OnImgvViewRealSizeUpdate( wxUpdateUIEvent& event )
{
	event.Enable( m_ImageViewerCtrl->GetZoomLevel() != 100 );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_ZOOM_IN
 */

void CImageViewerFrame::OnImgvViewZoomInClick( wxCommandEvent& event )
{
	m_ImageViewerCtrl->ZoomIn();
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_ZOOM_IN
 */

void CImageViewerFrame::OnImgvViewZoomInUpdate( wxUpdateUIEvent& event )
{
////@begin wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_ZOOM_IN in CImageViewerFrame.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_ZOOM_IN in CImageViewerFrame. 
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_ZOOM_OUT
 */

void CImageViewerFrame::OnImgvViewZoomOutClick( wxCommandEvent& event )
{
	m_ImageViewerCtrl->ZoomOut();
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_ZOOM_OUT
 */

void CImageViewerFrame::OnImgvViewZoomOutUpdate( wxUpdateUIEvent& event )
{
////@begin wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_ZOOM_OUT in CImageViewerFrame.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_UPDATE_UI event handler for ID_IMGV_VIEW_ZOOM_OUT in CImageViewerFrame. 
}


void CImageViewerFrame::MoveNext()
{
	if( m_FileDataIndex < m_FileDataArray->size() - 1 ) {
		m_FileDataIndex++;
		ShowCurrentImage();
	}
}

void CImageViewerFrame::MovePrevious()
{
	if( m_FileDataIndex > 0 ) {
		m_FileDataIndex--;
		ShowCurrentImage();
	}
}

void CImageViewerFrame::UpdateStatusBar()
{
	CFileData &fd = (*m_FileDataArray)[m_FileDataIndex];

	wxString s;
	if( fd.GetRotation() == CUtils::irNoRotation || fd.GetRotation() == CUtils::irUpsideDown )
		s = CUtils::long2string(fd.GetWidth()) + wxT("x") + CUtils::long2string(fd.GetHeight());
	else
		s = CUtils::long2string(fd.GetHeight()) + wxT("x") + CUtils::long2string(fd.GetWidth());
	m_StatusBar->SetStatusText( s , sbeDimensions );

	m_StatusBar->SetStatusText( CUtils::long2string(m_FileDataIndex + 1) + wxT("/") + CUtils::long2string(m_FileDataArray->size()), sbePosition );
	m_StatusBar->SetStatusText( CUtils::long2string( static_cast<int>(m_ImageViewerCtrl->GetZoomLevel() + 0.5)) + wxT("%"), sbeZoom );
	wxDateTime dt = fd.GetImageDateTime();
	if( dt.IsValid() )
		m_StatusBar->SetStatusText( dt.FormatDate() + wxT(" ") + dt.FormatTime() , sbeDateTime );
	m_StatusBar->SetStatusText( CUtils::NewLine2Space(fd.GetDescription()), sbeDescription );
}

void CImageViewerFrame::OnViewerZoomChanged( fsImageViewerEvent& event )
{
	UpdateStatusBar();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_VIEW_FULLSCREEN
 */

void CImageViewerFrame::OnImgvViewFullscreenClick( wxCommandEvent& WXUNUSED(event) )
{
	if( IsFullScreen() )
		EndFullScreen();
	else
		StartFullScreen();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_IMGV_EDIT_IMAGE_INFORMATION
 */

void CImageViewerFrame::OnImgvEditImageInformationClick( wxCommandEvent& WXUNUSED(event) )
{
	CFileData &fd = (*m_FileDataArray)[m_FileDataIndex];
	CImageInformationDialog dialog;
	dialog.SetFileData( &fd );
	dialog.Create( this, ID_CIMAGEINFORMATIONDIALOG, _("Image information") );
	dialog.ShowModal();
}

void CImageViewerFrame::StartFullScreen()
{
	// switch to the viewer control without border because the border looks bad in full screen
	wxImage img = m_ImageViewerCtrl->GetImage();
	m_ImageViewerInvisibleCtrl->SetImage( img );
	m_ImageViewerInvisibleCtrl->SetAutoFit( m_ImageViewerCtrl->GetAutoFit() );
    m_ViewerSizer->Hide( m_ImageViewerCtrl );
    m_ViewerSizer->Show( m_ImageViewerInvisibleCtrl );
	m_ImageViewerInvisibleCtrl->SetFocus();
    // switch pointers
    FS_ImageViewer* ptTmp = m_ImageViewerInvisibleCtrl;
    m_ImageViewerInvisibleCtrl = m_ImageViewerCtrl;
    m_ImageViewerCtrl = ptTmp;

#ifdef __WXMAC__
    // the Mac version does not hide these controls so we must do it
    m_ToolbarCtrl->Show( false );
    m_StatusBar->Show( false );
#endif

	ShowFullScreen( true );
	m_ImageViewerCtrl->Fit();
}

void CImageViewerFrame::EndFullScreen()
{
	ShowFullScreen( false );

#ifdef __WXMAC__
    m_ToolbarCtrl->Show( true );
    m_StatusBar->Show( true );
#endif

	wxImage img = m_ImageViewerCtrl->GetImage();
	m_ImageViewerInvisibleCtrl->SetImage( img );
	m_ImageViewerInvisibleCtrl->SetAutoFit( m_ImageViewerCtrl->GetAutoFit() );
    m_ViewerSizer->Hide( m_ImageViewerCtrl );
    m_ViewerSizer->Show( m_ImageViewerInvisibleCtrl );
	m_ImageViewerInvisibleCtrl->SetFocus();
    // switch pointers
    FS_ImageViewer* ptTmp = m_ImageViewerInvisibleCtrl;
    m_ImageViewerInvisibleCtrl = m_ImageViewerCtrl;
    m_ImageViewerCtrl = ptTmp;

	m_ImageViewerCtrl->Fit();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for wxID_HELP
 */

void CImageViewerFrame::OnHelpClick( wxCommandEvent& WXUNUSED(event) )
{
	wxGetApp().GetHelpController()->DisplaySection( wxT("image_viewer.htm") );
}




void CImageViewerFrame::OnChar( wxKeyEvent& event )
{
	bool keyProcessed = false;

	int kc = event.GetKeyCode();
	switch( kc ) {
		case WXK_RIGHT:
		case WXK_NUMPAD_RIGHT:
			keyProcessed = true;
			MoveNext();
			break;
		case WXK_LEFT:
		case WXK_NUMPAD_LEFT:
			keyProcessed = true;
			MovePrevious();
			break;
	}

	if( IsFullScreen() && !keyProcessed ) {
		// any key stops fullscreen
		EndFullScreen();
	}
	else {
		if( kc == WXK_ESCAPE )
			Close();
	}
}


/*!
 * wxEVT_CLOSE_WINDOW event handler for ID_CIMAGEVIEWERFRAME
 */

void CImageViewerFrame::OnCloseWindow( wxCloseEvent& event )
{
	if( event.CanVeto() ) {
		// hide the window instead of closing it
		m_WasMaximized = IsMaximized();	// remember this because in wxGTK the windows is never maximized after showing it again
		Show( false );
		event.Veto();
	}
	else {
		event.Skip();
	}
}

void CImageViewerFrame::CreateMacToolbar()
{
	const int bmpSize = 32;	// size of the toolbar bitmaps

    CImageViewerFrame* itemFrame1 = this;

	// this code is copied from the CreateControls() code. The only difference is that it uses larger toolbar
	// to make them look better in the Mac version
    m_ToolbarCtrl = CreateToolBar( wxTB_FLAT|wxTB_HORIZONTAL|wxTB_TEXT, ID_IMGVIEWER_TOOLBAR );
    m_ToolbarCtrl->SetToolBitmapSize(wxSize(bmpSize, bmpSize));
//    wxBitmap itemtool3Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_view_fullscreen.png")));
    wxBitmap itemtool3Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_view_fullscreen.png")), bmpSize, bmpSize));
    wxBitmap itemtool3BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_FULLSCREEN, _("Full Screen"), itemtool3Bitmap, itemtool3BitmapDisabled, wxITEM_NORMAL, _("Show full screen"), wxEmptyString);
    m_ToolbarCtrl->AddSeparator();
//    wxBitmap itemtool5Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_go_first.png")));
    wxBitmap itemtool5Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_go_first.png")), bmpSize, bmpSize));
    wxBitmap itemtool5BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_FIRST, _("First"), itemtool5Bitmap, itemtool5BitmapDisabled, wxITEM_NORMAL, _("Show the first image"), wxEmptyString);
//    wxBitmap itemtool6Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_go_previous.png")));
    wxBitmap itemtool6Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_go_previous.png")), bmpSize, bmpSize));
    wxBitmap itemtool6BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_PREVIOUS, _("Previous"), itemtool6Bitmap, itemtool6BitmapDisabled, wxITEM_NORMAL, _("Show the previous image"), wxEmptyString);
//    wxBitmap itemtool7Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_go_next.png")));
    wxBitmap itemtool7Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_go_next.png")), bmpSize, bmpSize));
    wxBitmap itemtool7BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_NEXT, _("Next"), itemtool7Bitmap, itemtool7BitmapDisabled, wxITEM_NORMAL, _("Show the next image"), wxEmptyString);
//    wxBitmap itemtool8Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_go_last.png")));
    wxBitmap itemtool8Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_go_last.png")), bmpSize, bmpSize));
    wxBitmap itemtool8BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_LAST, _("Last"), itemtool8Bitmap, itemtool8BitmapDisabled, wxITEM_NORMAL, _("Show the last image"), wxEmptyString);
    m_ToolbarCtrl->AddSeparator();
//    wxBitmap itemtool10Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_fit.png")));
    wxBitmap itemtool10Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_fit.png")), bmpSize, bmpSize));
    wxBitmap itemtool10BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_FIT, _("Fit"), itemtool10Bitmap, itemtool10BitmapDisabled, wxITEM_NORMAL, _("Fit image to window"), wxEmptyString);
//    wxBitmap itemtool11Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_fit_horizontal.png")));
    wxBitmap itemtool11Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_fit_horizontal.png")), bmpSize, bmpSize));
    wxBitmap itemtool11BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_FIT_H, _("Fit Width"), itemtool11Bitmap, itemtool11BitmapDisabled, wxITEM_NORMAL, _("Fit image width to window"), wxEmptyString);
//    wxBitmap itemtool12Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_fit_vertical.png")));
    wxBitmap itemtool12Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_fit_vertical.png")), bmpSize, bmpSize));
    wxBitmap itemtool12BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_FIT_V, _("Fit Height"), itemtool12Bitmap, itemtool12BitmapDisabled, wxITEM_NORMAL, _("Fit image height to window"), wxEmptyString);
//    wxBitmap itemtool13Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_original.png")));
    wxBitmap itemtool13Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_original.png")), bmpSize, bmpSize));
    wxBitmap itemtool13BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_REAL_SIZE, _("Real Size"), itemtool13Bitmap, itemtool13BitmapDisabled, wxITEM_NORMAL, _("Show image at real size"), wxEmptyString);
//    wxBitmap itemtool14Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_in.png")));
    wxBitmap itemtool14Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_in.png")), bmpSize, bmpSize));
    wxBitmap itemtool14BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_ZOOM_IN, _("Zoom In"), itemtool14Bitmap, itemtool14BitmapDisabled, wxITEM_NORMAL, wxEmptyString, wxEmptyString);
//    wxBitmap itemtool15Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_out.png")));
    wxBitmap itemtool15Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_zoom_out.png")), bmpSize, bmpSize));
    wxBitmap itemtool15BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_IMGV_VIEW_ZOOM_OUT, _("Zoom Out"), itemtool15Bitmap, itemtool15BitmapDisabled, wxITEM_NORMAL, wxEmptyString, wxEmptyString);
    m_ToolbarCtrl->Realize();
    itemFrame1->SetToolBar(m_ToolbarCtrl);
}


/*!
 * wxEVT_ERASE_BACKGROUND event handler for ID_CIMAGEVIEWERFRAME
 */

// to remove flicker
void CImageViewerFrame::OnEraseBackground( wxEraseEvent& event )
{
}

void CImageViewerFrame::SetupColorManagement()
{
	CAppSettings& as = wxGetApp().GetAppSettings();
	if( as.m_EnableColorManagement ) {
		switch( as.m_MonitorProfile ) {
			case CAppSettings::mp_sRGB:
				m_ColorProfilesHandler.Assign_sRGBOutputProfile();
				break;
			case CAppSettings::mp_FileProfile:
				{
					CColorProfile::ReturnValue r = m_ColorProfilesHandler.AssignFileOutputProfile( as.m_FileProfileName );
					if( r == CColorProfile::rvFileMissing ) {
						CUtils::MsgInfo( _("The monitor color profile file is missing: switching to sRGB") );
					}
					if( r == CColorProfile::rvFileInvalid ) {
						CUtils::MsgInfo( _("The monitor color profile file is invalid: switching to sRGB") );
					}
					if( r != CColorProfile::rvOk ) {
						as.m_MonitorProfile = CAppSettings::mp_sRGB;
					}
				}
				break;
			default:
				wxFAIL;
		}
	}
}


