/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2009, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef _LEFT_RIGHT_PANES_EVENTS_H_
#define _LEFT_RIGHT_PANES_EVENTS_H_

/*
Events used to notify that the rigt or left panes have received focus

This should be a solution to the problem of knowing in the main frame
if the left or right pane has focus
*/

// this event is sent when a control in the left pane receives focus
class fsLeftPaneFocusedEvent : public wxNotifyEvent
{
public:
	fsLeftPaneFocusedEvent( wxEventType commandType = wxEVT_NULL, int id = 0 ) :
	  wxNotifyEvent( commandType, id ) {}

	fsLeftPaneFocusedEvent( const fsLeftPaneFocusedEvent& event ) :
	  wxNotifyEvent( event ) {}

	virtual wxEvent *Clone() const { return new fsLeftPaneFocusedEvent( *this ); }

	DECLARE_DYNAMIC_CLASS( fsLeftPaneFocusedEvent );
};

typedef void (wxEvtHandler::*fsLeftPaneFocusedEventFunction) (fsLeftPaneFocusedEvent&);

BEGIN_DECLARE_EVENT_TYPES()
	DECLARE_EVENT_TYPE( fsEVT_LEFT_PANE_FOCUSED, -1 )
END_DECLARE_EVENT_TYPES()

#define EVT_LEFT_PANE_FOCUSED( id, fn) \
	DECLARE_EVENT_TABLE_ENTRY( \
		fsEVT_LEFT_PANE_FOCUSED, id, wxID_ANY, \
        (wxObjectEventFunction)(wxEventFunction) wxStaticCastEvent( fsLeftPaneFocusedEventFunction, &fn ), \
        (wxObject *) NULL \
    ),



// this event is sent when a control in the right pane receives focus
class fsRightPaneFocusedEvent : public wxNotifyEvent
{
public:
	fsRightPaneFocusedEvent( wxEventType commandType = wxEVT_NULL, int id = 0 ) :
	  wxNotifyEvent( commandType, id ) {}

	fsRightPaneFocusedEvent( const fsRightPaneFocusedEvent& event ) :
	  wxNotifyEvent( event ) {}

	virtual wxEvent *Clone() const { return new fsRightPaneFocusedEvent( *this ); }

	DECLARE_DYNAMIC_CLASS( fsRightPaneFocusedEvent );
};

typedef void (wxEvtHandler::*fsRightPaneFocusedEventFunction) (fsRightPaneFocusedEvent&);

BEGIN_DECLARE_EVENT_TYPES()
	DECLARE_EVENT_TYPE( fsEVT_RIGHT_PANE_FOCUSED, -1 )
END_DECLARE_EVENT_TYPES()

#define EVT_RIGHT_PANE_FOCUSED( id, fn) \
	DECLARE_EVENT_TABLE_ENTRY( \
		fsEVT_RIGHT_PANE_FOCUSED, id, wxID_ANY, \
        (wxObjectEventFunction)(wxEventFunction) wxStaticCastEvent( fsRightPaneFocusedEventFunction, &fn ), \
        (wxObject *) NULL \
    ),



#endif
