/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        mainframe.cpp
// Purpose:     
// Author:      
// Modified by: 
// Created:     30/07/2008 21:53:32
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
#include "wx/mstream.h"
#include "treectrl_physical.h"
#include "rightpanelist.h"
#include "status_bar_with_gauge.h"
////@end includes

#include "choose_virtual_folder.h"
#include "treectrl_virtual.h"
#include "treectrl_virtual_itemdata.h"
#include "fsdb.h"
#include "utils.h"
#include "db_utils.h"
#include "mainframe.h"
#include "catalog_images.h"
#include "catalog_images_functions.h"
#include "vvvpapp.h"
#include "long_task_beep.h"
#include "long_text_info.h"
#include "settings.h"
#include "serverfiledialog.h"
#include "sqlstrings.h"
#include "fixed_locations_dialog.h"
#include "object_description_dialog.h"
#include "search_panel.h"
#include "timeline_panel.h"
#include "copy_files_to_location_dialog.h"
#include "image_viewer_frame.h"
#include "about_dialog.h"
#include "resize_thumbnails_dialog.h"
#include "restore_dialog.h"
#include "choose_folder.h"
#include "db_file_list.h"
#include "sha1wrapper.h"

#include <wx/filename.h>
#include <wx/file.h>
#include <wx/image.h>
#include <wx/mstream.h>
#include <wx/stdpaths.h>
#include <wx/dirdlg.h>
#include <wx/progdlg.h>
#include <wx/utils.h>


using namespace FSDB;

////@begin XPM images
#include "graphics/vvvPicon32.inc"
#include "graphics/tlb_new.xpm"
#include "graphics/tlb_open.xpm"
#include "graphics/tlb_catalog.inc"
#include "graphics/tlb_info.inc"
#include "graphics/tlb_fixed.xpm"
#include "graphics/tlb_removable.xpm"
#include "graphics/tlb_virtual.xpm"
#include "graphics/tlb_timeline.xpm"
#include "graphics/tlb_search.xpm"
#include "graphics/tlb_large_thumbnails.xpm"
#include "graphics/tlb_medium_thumbnails.xpm"
#include "graphics/tlb_small_thumbnails.xpm"
#include "graphics/tlb_no_thumbnails.xpm"
////@end XPM images
#include "graphics/vvvPicon16.xpm"


/*!
 * CMainFrame type definition
 */

IMPLEMENT_CLASS( CMainFrame, wxFrame )


/*!
 * CMainFrame event table definition
 */

BEGIN_EVENT_TABLE( CMainFrame, wxFrame )

////@begin CMainFrame event table entries
    EVT_CLOSE( CMainFrame::OnCloseWindow )
    EVT_IDLE( CMainFrame::OnIdle )

    EVT_MENU( wxID_NEW, CMainFrame::OnNewClick )

    EVT_MENU( wxID_OPEN, CMainFrame::OnOpenClick )

    EVT_MENU( ID_FILE_CHECK_INTEGRITY, CMainFrame::OnFileCheckIntegrityClick )
    EVT_UPDATE_UI( ID_FILE_CHECK_INTEGRITY, CMainFrame::OnFileCheckIntegrityUpdate )

    EVT_MENU( ID_FILE_BACKUP, CMainFrame::OnFileBackupClick )
    EVT_UPDATE_UI( ID_FILE_BACKUP, CMainFrame::OnFileBackupUpdate )

    EVT_MENU( ID_FILE_RESTORE, CMainFrame::OnFileRestoreClick )

    EVT_MENU( ID_COPY_FILES, CMainFrame::OnCopyFilesClick )
    EVT_UPDATE_UI( ID_COPY_FILES, CMainFrame::OnCopyFilesUpdate )

    EVT_MENU( ID_ADD_VIRTUAL_FOLDER, CMainFrame::OnAddVirtualFolderClick )
    EVT_UPDATE_UI( ID_ADD_VIRTUAL_FOLDER, CMainFrame::OnAddVirtualFolderUpdate )

    EVT_MENU( wxID_SELECTALL, CMainFrame::OnSelectallClick )
    EVT_UPDATE_UI( wxID_SELECTALL, CMainFrame::OnSelectallUpdate )

    EVT_UPDATE_UI( ID_EDIT_ROTATE, CMainFrame::OnEditRotateUpdate )

    EVT_MENU( ID_EDIT_ROTATE_RIGHT, CMainFrame::OnEditRotateRightClick )
    EVT_UPDATE_UI( ID_EDIT_ROTATE_RIGHT, CMainFrame::OnEditRotateRightUpdate )

    EVT_MENU( ID_EDIT_ROTATE_UPSIDE_DOWN, CMainFrame::OnEditRotateUpsideDownClick )
    EVT_UPDATE_UI( ID_EDIT_ROTATE_UPSIDE_DOWN, CMainFrame::OnEditRotateUpsideDownUpdate )

    EVT_MENU( ID_EDIT_ROTATE_LEFT, CMainFrame::OnEditRotateLeftClick )
    EVT_UPDATE_UI( ID_EDIT_ROTATE_LEFT, CMainFrame::OnEditRotateLeftUpdate )

    EVT_MENU( ID_EDIT_RENAME, CMainFrame::OnEditRenameClick )
    EVT_UPDATE_UI( ID_EDIT_RENAME, CMainFrame::OnEditRenameUpdate )

    EVT_MENU( ID_EDIT_DELETE, CMainFrame::OnEditDeleteClick )
    EVT_UPDATE_UI( ID_EDIT_DELETE, CMainFrame::OnEditDeleteUpdate )

    EVT_UPDATE_UI( ID_EDIT_THUMBNAILS, CMainFrame::OnEditThumbnailsUpdate )

    EVT_MENU( ID_EDIT_THUMBNAILS_RESIZE, CMainFrame::OnEditThumbnailsResizeClick )
    EVT_UPDATE_UI( ID_EDIT_THUMBNAILS_RESIZE, CMainFrame::OnEditThumbnailsResizeUpdate )

    EVT_MENU( ID_EDIT_THUMBNAILS_DELETE, CMainFrame::OnEditThumbnailsDeleteClick )
    EVT_UPDATE_UI( ID_EDIT_THUMBNAILS_DELETE, CMainFrame::OnEditThumbnailsDeleteUpdate )

    EVT_MENU( ID_EDIT_OBJECT_INFORMATION, CMainFrame::OnEditObjectInformationClick )
    EVT_UPDATE_UI( ID_EDIT_OBJECT_INFORMATION, CMainFrame::OnEditObjectInformationUpdate )

    EVT_MENU( ID_NEW_VIRTUAL_ROOT_FOLDER, CMainFrame::OnNewVirtualRootFolderClick )
    EVT_UPDATE_UI( ID_NEW_VIRTUAL_ROOT_FOLDER, CMainFrame::OnNewVirtualRootFolderUpdate )

    EVT_MENU( ID_NEW_VIRTUAL_SUBFOLDER, CMainFrame::OnNewVirtualSubfolderClick )
    EVT_UPDATE_UI( ID_NEW_VIRTUAL_SUBFOLDER, CMainFrame::OnNewVirtualSubfolderUpdate )

#if defined(__WXMAC__) || defined(__WXGTK__)
    EVT_MENU( wxID_PREFERENCES, CMainFrame::OnPreferencesClick )
#endif

    EVT_UPDATE_UI( ID_EDIT_EP, CMainFrame::OnEditEpUpdate )

    EVT_MENU( ID_EDIT_EP0, CMainFrame::OnEditExternalProgramsClick )

    EVT_MENU( ID_EDIT_EP1, CMainFrame::OnEditExternalProgramsClick )

    EVT_MENU( ID_EDIT_EP2, CMainFrame::OnEditExternalProgramsClick )

    EVT_MENU( ID_EDIT_EP3, CMainFrame::OnEditExternalProgramsClick )

    EVT_MENU( ID_EDIT_EP4, CMainFrame::OnEditExternalProgramsClick )

    EVT_MENU( ID_EDIT_EP5, CMainFrame::OnEditExternalProgramsClick )

    EVT_MENU( ID_EDIT_EP6, CMainFrame::OnEditExternalProgramsClick )

    EVT_MENU( ID_EDIT_EP7, CMainFrame::OnEditExternalProgramsClick )

    EVT_MENU( ID_EDIT_EP8, CMainFrame::OnEditExternalProgramsClick )

    EVT_MENU( ID_EDIT_EP9, CMainFrame::OnEditExternalProgramsClick )

    EVT_MENU( ID_CATALOG_VOLUME, CMainFrame::OnCatalogVolumeClick )
    EVT_UPDATE_UI( ID_CATALOG_VOLUME, CMainFrame::OnCatalogVolumeUpdate )

    EVT_MENU( ID_FIXED_LOCATIONS, CMainFrame::OnFixedLocationsClick )
    EVT_UPDATE_UI( ID_FIXED_LOCATIONS, CMainFrame::OnFixedLocationsUpdate )

    EVT_MENU( ID_VIEW_FIXED_LOCATIONS, CMainFrame::OnViewFixedLocationsClick )
    EVT_UPDATE_UI( ID_VIEW_FIXED_LOCATIONS, CMainFrame::OnViewFixedLocationsUpdate )

    EVT_MENU( ID_VIEW_REMOVABLE_LOCATIONS, CMainFrame::OnViewRemovableLocationsClick )
    EVT_UPDATE_UI( ID_VIEW_REMOVABLE_LOCATIONS, CMainFrame::OnViewRemovableLocationsUpdate )

    EVT_MENU( ID_VIEW_VIRTUAL_LOCATIONS, CMainFrame::OnViewVirtualLocationsClick )
    EVT_UPDATE_UI( ID_VIEW_VIRTUAL_LOCATIONS, CMainFrame::OnViewVirtualLocationsUpdate )

    EVT_MENU( ID_VIEW_TIMELINE_VIEW, CMainFrame::OnViewTimelineViewClick )
    EVT_UPDATE_UI( ID_VIEW_TIMELINE_VIEW, CMainFrame::OnViewTimelineViewUpdate )

    EVT_MENU( ID_VIEW_SEARCH_VIEW, CMainFrame::OnViewSearchViewClick )
    EVT_UPDATE_UI( ID_VIEW_SEARCH_VIEW, CMainFrame::OnViewSearchViewUpdate )

    EVT_MENU( ID_VIEW_NO_THUMBNAILS, CMainFrame::OnViewNoThumbnailsClick )
    EVT_UPDATE_UI( ID_VIEW_NO_THUMBNAILS, CMainFrame::OnViewNoThumbnailsUpdate )

    EVT_MENU( ID_VIEW_SMALL_THUMBNAILS, CMainFrame::OnViewSmallThumbnailsClick )
    EVT_UPDATE_UI( ID_VIEW_SMALL_THUMBNAILS, CMainFrame::OnViewSmallThumbnailsUpdate )

    EVT_MENU( ID_VIEW_MEDIUM_THUMBNAILS, CMainFrame::OnViewMediumThumbnailsClick )
    EVT_UPDATE_UI( ID_VIEW_MEDIUM_THUMBNAILS, CMainFrame::OnViewMediumThumbnailsUpdate )

    EVT_MENU( ID_VIEW_LARGE_THUMBNAILS, CMainFrame::OnViewLargeThumbnailsClick )
    EVT_UPDATE_UI( ID_VIEW_LARGE_THUMBNAILS, CMainFrame::OnViewLargeThumbnailsUpdate )

    EVT_MENU( ID_VIEW_TOOLBAR, CMainFrame::OnViewToolbarClick )

    EVT_MENU( ID_VIEW_STATUS_BAR, CMainFrame::OnViewStatusBarClick )

    EVT_MENU( ID_TOOLS_SHOW_IMAGE, CMainFrame::OnToolsShowImageClick )
    EVT_UPDATE_UI( ID_TOOLS_SHOW_IMAGE, CMainFrame::OnToolsShowImageUpdate )

#if defined(__WXMSW__)
    EVT_MENU( wxID_PREFERENCES, CMainFrame::OnPreferencesClick )
#endif

    EVT_MENU( wxID_HELP_CONTENTS, CMainFrame::OnHelpContentsClick )

    EVT_MENU( wxID_ABOUT, CMainFrame::OnAboutClick )

    EVT_SPLITTER_DCLICK( ID_SPLITTERWINDOW, CMainFrame::OnSplitterwindowSashDClick )

////@end CMainFrame event table entries

	EVT_MENU_RANGE( wxID_FILE1, wxID_FILE9, CMainFrame::OnMRUFile)

	EVT_THUMBNAIL_LEFT_DCLICK( wxID_ANY, CMainFrame::OnThumbnailCtrlLeftDClick )
	EVT_LIST_ITEM_ACTIVATED( wxID_ANY, CMainFrame::OnListviewItemActivated )

	EVT_THUMBNAIL_ITEM_SELECTED( wxID_ANY, CMainFrame::OnThumbnailCtrlSelected )
	EVT_LIST_ITEM_SELECTED( wxID_ANY, CMainFrame::OnListviewItemSelected )

	EVT_THUMBNAIL_ITEM_DESELECTED( wxID_ANY, CMainFrame::OnThumbnailCtrlDeselected )
	EVT_LIST_ITEM_DESELECTED( wxID_ANY, CMainFrame::OnListviewItemDeselected )

	EVT_LEFT_PANE_FOCUSED( wxID_ANY, CMainFrame::OnLeftPaneFocused )
	EVT_RIGHT_PANE_FOCUSED( wxID_ANY, CMainFrame::OnRightPaneFocused )

END_EVENT_TABLE()


/*!
 * CMainFrame constructors
 */

CMainFrame::CMainFrame()
{
    Init();
}

CMainFrame::CMainFrame( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create( parent, id, caption, pos, size, style );
}


/*!
 * CMainFrame creator
 */

bool CMainFrame::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CMainFrame creation
    wxFrame::Create( parent, id, caption, pos, size, style );

    CreateControls();
    SetIcon(GetIconResource(wxT("graphics/vvvPicon32.png")));
    Centre();
////@end CMainFrame creation

#ifdef __WXGTK__
	// add more than one icon to the Linux version
	wxIconBundle ib( GetIcon() );
	ib.AddIcon( vvvPicon16_xpm );
	SetIcons( ib );
#endif
    
	// restore the frame position
	CAppSettings& as = wxGetApp().GetAppSettings();
	if( CWindowPosition::CheckCoordinates(as.mf_XPosition, as.mf_YPosition, as.mf_Width, as.mf_Height) ) {
		Move(as.mf_XPosition, as.mf_YPosition);
		SetClientSize(as.mf_Width, as.mf_Height);
		SendSizeEvent();
	}
	if( as.mf_Maximized ) Maximize(true);
	m_SplitterWindow->SetSashPosition( as.mf_SashPosition );

	// check to see if a catalog name has been passed in the command line
	// or if the program must reload the last used catalog
	wxString catalogName = wxGetApp().GetParameterCatalog();
	if( catalogName.empty() && as.m_ReopenLastUsedCatalog && (as.m_fileHistory->GetCount() > 0) ) {
		// no parameter passed, looks for auto load of last used catalog
		catalogName = as.m_fileHistory->GetHistoryFile( 0 );
	}
	if( !catalogName.empty() ) {
		if( !as.dbConnectToServer && !wxFileName::FileExists(catalogName) ) {
			CUtils::MsgErr( _("This catalog file does not exist:\n\n") + catalogName );
			catalogName = wxEmptyString;
		}
	}
	if( !catalogName.empty() )
		OpenDatabase( catalogName, CUtils::GetExpectedDatabaseVersion() );

	// set the "long task beep" time
	CLongTaskBeep::SetMinSecondsForBell( as.m_BeepTime );
	
	// reset the last used view
	switch( as.m_CurrentView ) {
		case CAppSettings::cvVirtual:
			m_CurrentView = CAppSettings::cvVirtual;
			m_ViewMenu->Check( ID_VIEW_VIRTUAL_LOCATIONS, true );
			m_ToolbarCtrl->ToggleTool( ID_VIEW_VIRTUAL_LOCATIONS, true );
			SetViewVirtual();
			break;
		case CAppSettings::cvFixed:
			m_CurrentView = CAppSettings::cvFixed;
			m_ViewMenu->Check( ID_VIEW_FIXED_LOCATIONS, true );
			m_ToolbarCtrl->ToggleTool( ID_VIEW_FIXED_LOCATIONS, true );
			SetViewFixedLocations();
			break;
		case CAppSettings::cvSearch:
			m_CurrentView = CAppSettings::cvSearch;
			m_ViewMenu->Check( ID_VIEW_SEARCH_VIEW, true );
			m_ToolbarCtrl->ToggleTool( ID_VIEW_SEARCH_VIEW, true );
			SetViewSearch();
			break;
		case CAppSettings::cvTimeline:
			m_CurrentView = CAppSettings::cvTimeline;
			m_ViewMenu->Check( ID_VIEW_TIMELINE_VIEW, true );
			m_ToolbarCtrl->ToggleTool( ID_VIEW_TIMELINE_VIEW, true );
			SetViewTimeline();
			break;
		default:
			m_CurrentView = CAppSettings::cvRemovable;
			m_ViewMenu->Check( ID_VIEW_REMOVABLE_LOCATIONS, true );
			m_ToolbarCtrl->ToggleTool( ID_VIEW_REMOVABLE_LOCATIONS, true );
			SetViewRemovableLocations();
			break;
	}
	
	// reset the last thumbnail size
	switch( as.m_CurrentThumbnailViewSize ) {
		case CAppSettings::tsSmallThumbnails:
			m_ViewMenu->Check( ID_VIEW_SMALL_THUMBNAILS, true );
			m_ToolbarCtrl->ToggleTool( ID_VIEW_SMALL_THUMBNAILS, true );
			SetViewSmallThumbnails();
			break;
		case CAppSettings::tsMediumThumbnails:
			m_ViewMenu->Check( ID_VIEW_MEDIUM_THUMBNAILS, true );
			m_ToolbarCtrl->ToggleTool( ID_VIEW_MEDIUM_THUMBNAILS, true );
			SetViewMediumThumbnails();
			break;
		case CAppSettings::tsLargeThumbnails:
			m_ViewMenu->Check( ID_VIEW_LARGE_THUMBNAILS, true );
			m_ToolbarCtrl->ToggleTool( ID_VIEW_LARGE_THUMBNAILS, true );
			SetViewLargeThumbnails();
			break;
		default:
			SetViewNoThumbnails();
			m_ViewMenu->Check( ID_VIEW_NO_THUMBNAILS, true );
			m_ToolbarCtrl->ToggleTool( ID_VIEW_NO_THUMBNAILS, true );
			break;
	}

	// create a hidden "image viewer" frame for later use
	// it is much simpler than continuously creating and destroying it when it is needed
	m_ImgViewerFrame = new CImageViewerFrame( this );
	m_ImgViewerFrame->Lower();
	m_ImgViewerFrame->Show( true );
	m_ImgViewerFrame->Show( false );


	return true;
}


/*!
 * CMainFrame destructor
 */

CMainFrame::~CMainFrame()
{
////@begin CMainFrame destruction
////@end CMainFrame destruction

	CAppSettings& as = wxGetApp().GetAppSettings();
	int x, y, w, h;
	GetClientSize(&w, &h);
	GetPosition(&x, &y);
	if( !IsMaximized() && CWindowPosition::CheckCoordinates(x, y, w, h) ) {
		as.mf_XPosition = x;
		as.mf_YPosition = y;
		as.mf_Width = w;
		as.mf_Height = h;
	}
	as.mf_Maximized = IsMaximized();
	as.mf_SashPosition = m_SplitterWindow->GetSashPosition();

	as.m_CurrentThumbnailViewSize = m_CurrentThumbnailSize;
	as.m_CurrentView = m_CurrentView;

	if( m_ImgViewerFrame != NULL )
		m_ImgViewerFrame->Destroy();
		
	// fix to avoid a crash in wxThumbnailCtrl under OS X (wxWidgets 2.9)
	Hide();
}


/*!
 * Member initialisation
 */

void CMainFrame::Init()
{
////@begin CMainFrame member initialisation
    m_ToolbarCtrl = NULL;
    m_FileMenu = NULL;
    m_EditMenu = NULL;
    m_ViewMenu = NULL;
    m_SplitterWindow = NULL;
    m_TreeCtrlRemovable = NULL;
    m_RightPaneListRemovable = NULL;
    m_StatusBar = NULL;
////@end CMainFrame member initialisation
	m_ImgViewerFrame = NULL;
	m_HandlingSelectionEvent = false;
	m_LoadingFixedView = false;
	m_LoadingRemovableView = false;
	m_LoadingVirtualView = false;
	m_LoadingTimelineView = false;
	m_NumRowsToLoadFixed = 0;
	m_NumRowsToLoadRemovable = 0;
	m_NumRowsToLoadVirtual = 0;
	m_NumRowsToLoadTimeline = 0;
	m_RightPaneHasFocus = false;
}


/*!
 * Control creation for CMainFrame
 */

void CMainFrame::CreateControls()
{    
////@begin CMainFrame content construction
    CMainFrame* itemFrame1 = this;

    wxMenuBar* menuBar = new wxMenuBar;
    m_FileMenu = new wxMenu;
    m_FileMenu->Append(wxID_NEW, _("&New..."), wxEmptyString, wxITEM_NORMAL);
    m_FileMenu->Append(wxID_OPEN, _("&Open..."), wxEmptyString, wxITEM_NORMAL);
    m_FileMenu->AppendSeparator();
    m_FileMenu->Append(ID_FILE_CHECK_INTEGRITY, _("Check integrity"), wxEmptyString, wxITEM_NORMAL);
    m_FileMenu->AppendSeparator();
    m_FileMenu->Append(ID_FILE_BACKUP, _("Backup..."), wxEmptyString, wxITEM_NORMAL);
    m_FileMenu->Append(ID_FILE_RESTORE, _("Restore..."), wxEmptyString, wxITEM_NORMAL);
    m_FileMenu->AppendSeparator();
    m_FileMenu->Append(ID_COPY_FILES, _("Copy selected files...\tAlt-C"), wxEmptyString, wxITEM_NORMAL);
    menuBar->Append(m_FileMenu, _("&File"));
    m_EditMenu = new wxMenu;
    m_EditMenu->Append(ID_ADD_VIRTUAL_FOLDER, _("Add To Virtual Folder..."), wxEmptyString, wxITEM_NORMAL);
    m_EditMenu->AppendSeparator();
    m_EditMenu->Append(wxID_SELECTALL, _("Select All\tCtrl-A"), wxEmptyString, wxITEM_NORMAL);
    m_EditMenu->AppendSeparator();
    wxMenu* itemMenu34 = new wxMenu;
    itemMenu34->Append(ID_EDIT_ROTATE_RIGHT, _("Rotate right\tCtrl-R"), wxEmptyString, wxITEM_NORMAL);
    itemMenu34->Append(ID_EDIT_ROTATE_UPSIDE_DOWN, _("Upside down"), wxEmptyString, wxITEM_NORMAL);
    itemMenu34->Append(ID_EDIT_ROTATE_LEFT, _("Rotate left\tCtrl-L"), wxEmptyString, wxITEM_NORMAL);
    m_EditMenu->Append(ID_EDIT_ROTATE, _("Rotate"), itemMenu34);
    m_EditMenu->Append(ID_EDIT_RENAME, _("Rename"), wxEmptyString, wxITEM_NORMAL);
    m_EditMenu->Append(ID_EDIT_DELETE, _("Delete"), wxEmptyString, wxITEM_NORMAL);
    wxMenu* itemMenu40 = new wxMenu;
    itemMenu40->Append(ID_EDIT_THUMBNAILS_RESIZE, _("Resize selected thumbnails"), wxEmptyString, wxITEM_NORMAL);
    itemMenu40->Append(ID_EDIT_THUMBNAILS_DELETE, _("Delete selected thumbnails"), wxEmptyString, wxITEM_NORMAL);
    m_EditMenu->Append(ID_EDIT_THUMBNAILS, _("Thumbnails"), itemMenu40);
    m_EditMenu->AppendSeparator();
    m_EditMenu->Append(ID_EDIT_OBJECT_INFORMATION, _("Object Information...\tCtrl-I"), wxEmptyString, wxITEM_NORMAL);
    m_EditMenu->AppendSeparator();
    m_EditMenu->Append(ID_NEW_VIRTUAL_ROOT_FOLDER, _("New Virtual Root Folder..."), wxEmptyString, wxITEM_NORMAL);
    m_EditMenu->Append(ID_NEW_VIRTUAL_SUBFOLDER, _("New Virtual Subfolder..."), wxEmptyString, wxITEM_NORMAL);
#if defined(__WXMAC__) || defined(__WXGTK__)
    m_EditMenu->Append(wxID_PREFERENCES, _("Preferences..."), wxEmptyString, wxITEM_NORMAL);
#endif
    wxMenu* itemMenu49 = new wxMenu;
    itemMenu49->Append(ID_EDIT_EP0, _("External program 0"), wxEmptyString, wxITEM_NORMAL);
    itemMenu49->Append(ID_EDIT_EP1, _("External program 1"), wxEmptyString, wxITEM_NORMAL);
    itemMenu49->Append(ID_EDIT_EP2, _("External program 2"), wxEmptyString, wxITEM_NORMAL);
    itemMenu49->Append(ID_EDIT_EP3, _("External program 3"), wxEmptyString, wxITEM_NORMAL);
    itemMenu49->Append(ID_EDIT_EP4, _("External program 4"), wxEmptyString, wxITEM_NORMAL);
    itemMenu49->Append(ID_EDIT_EP5, _("External program 5"), wxEmptyString, wxITEM_NORMAL);
    itemMenu49->Append(ID_EDIT_EP6, _("External program 6"), wxEmptyString, wxITEM_NORMAL);
    itemMenu49->Append(ID_EDIT_EP7, _("External program 7"), wxEmptyString, wxITEM_NORMAL);
    itemMenu49->Append(ID_EDIT_EP8, _("External program 8"), wxEmptyString, wxITEM_NORMAL);
    itemMenu49->Append(ID_EDIT_EP9, _("External program 9"), wxEmptyString, wxITEM_NORMAL);
    m_EditMenu->Append(ID_EDIT_EP, _("External Programs"), itemMenu49);
    menuBar->Append(m_EditMenu, _("&Edit"));
    wxMenu* itemMenu60 = new wxMenu;
    itemMenu60->Append(ID_CATALOG_VOLUME, _("Catalog Images..."), wxEmptyString, wxITEM_NORMAL);
    itemMenu60->Append(ID_FIXED_LOCATIONS, _("Fixed Locations..."), wxEmptyString, wxITEM_NORMAL);
    menuBar->Append(itemMenu60, _("Catalog"));
    m_ViewMenu = new wxMenu;
    m_ViewMenu->Append(ID_VIEW_FIXED_LOCATIONS, _("Fixed locations"), wxEmptyString, wxITEM_RADIO);
    m_ViewMenu->Append(ID_VIEW_REMOVABLE_LOCATIONS, _("Removable locations"), wxEmptyString, wxITEM_RADIO);
    m_ViewMenu->Append(ID_VIEW_VIRTUAL_LOCATIONS, _("Virtual locations"), wxEmptyString, wxITEM_RADIO);
    m_ViewMenu->Append(ID_VIEW_TIMELINE_VIEW, _("Timeline view"), wxEmptyString, wxITEM_RADIO);
    m_ViewMenu->Append(ID_VIEW_SEARCH_VIEW, _("Search view"), wxEmptyString, wxITEM_RADIO);
    m_ViewMenu->AppendSeparator();
    m_ViewMenu->Append(ID_VIEW_NO_THUMBNAILS, _("No thumbnails"), wxEmptyString, wxITEM_RADIO);
    m_ViewMenu->Append(ID_VIEW_SMALL_THUMBNAILS, _("Small thumbnails"), wxEmptyString, wxITEM_RADIO);
    m_ViewMenu->Append(ID_VIEW_MEDIUM_THUMBNAILS, _("Medium thumbnails"), wxEmptyString, wxITEM_RADIO);
    m_ViewMenu->Append(ID_VIEW_LARGE_THUMBNAILS, _("Large thumbnails"), wxEmptyString, wxITEM_RADIO);
    m_ViewMenu->AppendSeparator();
    m_ViewMenu->Append(ID_VIEW_TOOLBAR, _("Toolbar"), wxEmptyString, wxITEM_CHECK);
    m_ViewMenu->Check(ID_VIEW_TOOLBAR, true);
    m_ViewMenu->Append(ID_VIEW_STATUS_BAR, _("Status Bar"), wxEmptyString, wxITEM_CHECK);
    m_ViewMenu->Check(ID_VIEW_STATUS_BAR, true);
    menuBar->Append(m_ViewMenu, _("&View"));
    wxMenu* itemMenu77 = new wxMenu;
    itemMenu77->Append(ID_TOOLS_SHOW_IMAGE, _("Show selected image"), wxEmptyString, wxITEM_NORMAL);
#if defined(__WXMSW__)
    itemMenu77->AppendSeparator();
#endif
#if defined(__WXMSW__)
    itemMenu77->Append(wxID_PREFERENCES, _("&Options..."), wxEmptyString, wxITEM_NORMAL);
#endif
    menuBar->Append(itemMenu77, _("&Tools"));
    wxMenu* itemMenu81 = new wxMenu;
    itemMenu81->Append(wxID_HELP_CONTENTS, _("Help Contents"), wxEmptyString, wxITEM_NORMAL);
#if defined(__WXMSW__) || defined(__WXGTK__)
    itemMenu81->AppendSeparator();
#endif
    itemMenu81->Append(wxID_ABOUT, _("About vvvP..."), wxEmptyString, wxITEM_NORMAL);
    menuBar->Append(itemMenu81, _("Help"));
    itemFrame1->SetMenuBar(menuBar);

    m_ToolbarCtrl = CreateToolBar( wxTB_FLAT|wxTB_HORIZONTAL|wxTB_TEXT, ID_TOOLBAR );
    m_ToolbarCtrl->SetToolBitmapSize(wxSize(16, 16));
    wxBitmap itemtool3Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_new.xpm")));
    wxBitmap itemtool3BitmapDisabled;
    m_ToolbarCtrl->AddTool(wxID_NEW, _("New"), itemtool3Bitmap, itemtool3BitmapDisabled, wxITEM_NORMAL, _("Create a new catalog"), wxEmptyString);
    wxBitmap itemtool4Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_open.xpm")));
    wxBitmap itemtool4BitmapDisabled;
    m_ToolbarCtrl->AddTool(wxID_OPEN, _("Open"), itemtool4Bitmap, itemtool4BitmapDisabled, wxITEM_NORMAL, _("Open an existing catalog"), wxEmptyString);
    wxBitmap itemtool5Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_catalog.png")));
    wxBitmap itemtool5BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_CATALOG_VOLUME, _("Catalog"), itemtool5Bitmap, itemtool5BitmapDisabled, wxITEM_NORMAL, _("Catalog images"), wxEmptyString);
    wxBitmap itemtool6Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_info.png")));
    wxBitmap itemtool6BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_EDIT_OBJECT_INFORMATION, _(" Info"), itemtool6Bitmap, itemtool6BitmapDisabled, wxITEM_NORMAL, wxEmptyString, wxEmptyString);
    m_ToolbarCtrl->AddSeparator();
    wxBitmap itemtool8Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_fixed.xpm")));
    wxBitmap itemtool8BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_FIXED_LOCATIONS, _("Fixed"), itemtool8Bitmap, itemtool8BitmapDisabled, wxITEM_RADIO, _("View fixed locations"), wxEmptyString);
    wxBitmap itemtool9Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_removable.xpm")));
    wxBitmap itemtool9BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_REMOVABLE_LOCATIONS, _("Remov."), itemtool9Bitmap, itemtool9BitmapDisabled, wxITEM_RADIO, _("View removable locations"), wxEmptyString);
    wxBitmap itemtool10Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_virtual.xpm")));
    wxBitmap itemtool10BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_VIRTUAL_LOCATIONS, _("Virtual"), itemtool10Bitmap, itemtool10BitmapDisabled, wxITEM_RADIO, _("Virtual view"), wxEmptyString);
    wxBitmap itemtool11Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_timeline.xpm")));
    wxBitmap itemtool11BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_TIMELINE_VIEW, _("Timeline"), itemtool11Bitmap, itemtool11BitmapDisabled, wxITEM_RADIO, _("Timeline view"), wxEmptyString);
    wxBitmap itemtool12Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_search.xpm")));
    wxBitmap itemtool12BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_SEARCH_VIEW, _("Search"), itemtool12Bitmap, itemtool12BitmapDisabled, wxITEM_RADIO, wxEmptyString, wxEmptyString);
    m_ToolbarCtrl->AddSeparator();
    wxBitmap itemtool14Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_large_thumbnails.xpm")));
    wxBitmap itemtool14BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_LARGE_THUMBNAILS, _("Large"), itemtool14Bitmap, itemtool14BitmapDisabled, wxITEM_RADIO, _("Large thumbnails"), wxEmptyString);
    wxBitmap itemtool15Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_medium_thumbnails.xpm")));
    wxBitmap itemtool15BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_MEDIUM_THUMBNAILS, _("Medium"), itemtool15Bitmap, itemtool15BitmapDisabled, wxITEM_RADIO, _("Medium thumbnails"), wxEmptyString);
    wxBitmap itemtool16Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_small_thumbnails.xpm")));
    wxBitmap itemtool16BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_SMALL_THUMBNAILS, _("Small"), itemtool16Bitmap, itemtool16BitmapDisabled, wxITEM_RADIO, _("Small thumbnails"), wxEmptyString);
    wxBitmap itemtool17Bitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_no_thumbnails.xpm")));
    wxBitmap itemtool17BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_NO_THUMBNAILS, _("No"), itemtool17Bitmap, itemtool17BitmapDisabled, wxITEM_RADIO, _("No thumbnails"), wxEmptyString);
    m_ToolbarCtrl->Realize();
    itemFrame1->SetToolBar(m_ToolbarCtrl);

    m_SplitterWindow = new wxSplitterWindow( itemFrame1, ID_SPLITTERWINDOW, wxDefaultPosition, wxSize(100, 100), wxSP_3DBORDER|wxSP_3DSASH|wxSP_NO_XP_THEME|wxNO_BORDER );
    m_SplitterWindow->SetMinimumPaneSize(20);

    m_TreeCtrlRemovable = new CTreeCtrlPhysical( m_SplitterWindow, wxID_ANY, wxDefaultPosition, wxSize(100, 100), wxTR_HAS_BUTTONS |wxTR_HIDE_ROOT|wxTR_SINGLE|wxNO_BORDER|wxTR_DEFAULT_STYLE );
    m_RightPaneListRemovable = new CRightPaneList( m_SplitterWindow, ID_RIGHT_LISTCTRL, wxDefaultPosition, wxSize(100, 100), wxLC_REPORT|wxLC_NO_SORT_HEADER|wxNO_BORDER );
    m_SplitterWindow->SplitVertically(m_TreeCtrlRemovable, m_RightPaneListRemovable, 50);

    m_StatusBar = new CStatusBarWithGauge( itemFrame1, ID_STATUSBAR, wxST_SIZEGRIP );
    m_StatusBar->SetFieldsCount(4);
    int m_StatusBarWidths[4];
    m_StatusBarWidths[0] = 50;
    m_StatusBarWidths[1] = 120;
    m_StatusBarWidths[2] = 200;
    m_StatusBarWidths[3] = -1;
    m_StatusBar->SetStatusWidths(4, m_StatusBarWidths);
    itemFrame1->SetStatusBar(m_StatusBar);

////@end CMainFrame content construction

	// set the "edit/external programs" submenu
	CreateExternalProgramsMenu();

#if defined(__WXMAC__)
	// recreate the toolbar with larger bitmaps to make it look better in OS X
	delete m_ToolbarCtrl;
	SetToolBar( NULL );
	CreateMacToolbar();
#endif

	// set the volume type for the tree control
	m_TreeCtrlRemovable->SetVolumeType( wxT("R") );

	// create the thumbnails control for removable locations
	m_ThumbnailsCtrlRemovable = new vvvPThumbnailCtrl( m_SplitterWindow, ID_THUMBNAILS_CTRL, wxDefaultPosition, wxSize(100, 100), wxNO_BORDER|wxHSCROLL|wxVSCROLL|wxTH_TEXT_LABEL|wxTH_IMAGE_LABEL|wxTH_EXTENSION_OR_DATETIME_LABEL|wxTH_MULTIPLE_SELECT );

	// create the tree control for fixed locations
    m_TreeCtrlFixed = new CTreeCtrlPhysical( m_SplitterWindow, wxID_ANY, wxDefaultPosition, wxSize(100, 100), wxTR_HAS_BUTTONS |wxTR_HIDE_ROOT|wxTR_SINGLE|wxNO_BORDER|wxTR_DEFAULT_STYLE );
	m_TreeCtrlFixed->SetVolumeType( wxT("F") );
	m_TreeCtrlFixed->Show( false );

	// create the thumbnails controls for fixed locations
	m_ThumbnailsCtrlFixed = new vvvPThumbnailCtrl( m_SplitterWindow, wxID_ANY, wxDefaultPosition, wxSize(100, 100), wxNO_BORDER|wxHSCROLL|wxVSCROLL|wxTH_TEXT_LABEL|wxTH_IMAGE_LABEL|wxTH_EXTENSION_OR_DATETIME_LABEL|wxTH_MULTIPLE_SELECT );
	m_ThumbnailsCtrlFixed->Show( false );
	
	// create the listview control for the fixed locations
    m_RightPaneListFixed = new CRightPaneList( m_SplitterWindow, wxID_ANY, wxDefaultPosition, wxSize(100, 100), wxLC_REPORT|wxLC_NO_SORT_HEADER|wxNO_BORDER );
	m_RightPaneListFixed->Show( false );

	// create the tree control for the virtual view
    m_TreeCtrlVirtual = new CTreeCtrlVirtual( m_SplitterWindow, wxID_ANY, wxDefaultPosition, wxSize(100, 100), wxTR_HAS_BUTTONS |wxTR_HIDE_ROOT|wxTR_SINGLE|wxNO_BORDER|wxTR_DEFAULT_STYLE );
	m_TreeCtrlVirtual->SetUpdateRightPane( true );
	m_TreeCtrlVirtual->Show( false );

	// create the thumbnails controls for the virtual view
	m_ThumbnailsCtrlVirtual = new vvvPThumbnailCtrl( m_SplitterWindow, wxID_ANY, wxDefaultPosition, wxSize(100, 100), wxNO_BORDER|wxHSCROLL|wxVSCROLL|wxTH_TEXT_LABEL|wxTH_IMAGE_LABEL|wxTH_EXTENSION_OR_DATETIME_LABEL|wxTH_MULTIPLE_SELECT );
	m_ThumbnailsCtrlVirtual->Show( false );

	// create the listview control for the virtual view
    m_RightPaneListVirtual = new CRightPaneList( m_SplitterWindow, wxID_ANY, wxDefaultPosition, wxSize(100, 100), wxLC_REPORT|wxLC_NO_SORT_HEADER|wxNO_BORDER );
	m_RightPaneListVirtual->Show( false );

	// create the search panel
    m_SearchPanel = new CSearchPanel( m_SplitterWindow, ID_SEARCH_PANEL, wxDefaultPosition, wxDefaultSize, wxNO_BORDER|wxTAB_TRAVERSAL );
	m_SearchPanel->Show( false );

	// create the thumbnails controls for the search view
	m_ThumbnailsCtrlSearch = new vvvPThumbnailCtrl( m_SplitterWindow, wxID_ANY, wxDefaultPosition, wxSize(100, 100), wxNO_BORDER|wxHSCROLL|wxVSCROLL|wxTH_TEXT_LABEL|wxTH_IMAGE_LABEL|wxTH_EXTENSION_OR_DATETIME_LABEL|wxTH_MULTIPLE_SELECT );
	m_ThumbnailsCtrlSearch->Show( false );

	// create the listview control for the search view
    m_RightPaneListSearch = new CRightPaneList( m_SplitterWindow, wxID_ANY, wxDefaultPosition, wxSize(100, 100), wxLC_REPORT|wxLC_NO_SORT_HEADER|wxNO_BORDER );
	m_RightPaneListSearch->Show( false );

    // create the timeline panel
    m_TimelinePanel = new CTimelinePanel( m_SplitterWindow, ID_TIMELINE_PANEL, wxDefaultPosition, wxDefaultSize, wxNO_BORDER|wxTAB_TRAVERSAL );
	m_TimelinePanel->Show( false );

	// create the thumbnails controls for the timeline view
	m_ThumbnailsCtrlTimeline = new vvvPThumbnailCtrl( m_SplitterWindow, wxID_ANY, wxDefaultPosition, wxSize(100, 100), wxNO_BORDER|wxHSCROLL|wxVSCROLL|wxTH_TEXT_LABEL|wxTH_IMAGE_LABEL|wxTH_EXTENSION_OR_DATETIME_LABEL|wxTH_MULTIPLE_SELECT );
	m_ThumbnailsCtrlTimeline->Show( false );

	// create the listview control for the timeline view
    m_RightPaneListTimeline = new CRightPaneList( m_SplitterWindow, wxID_ANY, wxDefaultPosition, wxSize(100, 100), wxLC_REPORT|wxLC_NO_SORT_HEADER|wxNO_BORDER );
	m_RightPaneListTimeline->Show( false );

	// set a fixed initial state
	m_ThumbnailsCtrlRemovable->Show( false );
	m_ViewMenu->Check( ID_VIEW_NO_THUMBNAILS, true );
	m_CurrentThumbnailSize = CAppSettings::tsNoThumbnails;

	// associate the MRU files list
	CAppSettings& as = wxGetApp().GetAppSettings();
	as.m_fileHistory->UseMenu( m_FileMenu );
	as.m_fileHistory->AddFilesToMenu();
}


/*!
 * Should we show tooltips?
 */

bool CMainFrame::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CMainFrame::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CMainFrame bitmap retrieval
    wxUnusedVar(name);
    if (name == _T("graphics/tlb_new.xpm"))
    {
        wxBitmap bitmap(tlb_new_xpm);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_open.xpm"))
    {
        wxBitmap bitmap(tlb_open_xpm);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_catalog.png"))
    {
        wxMemoryInputStream memStream(tlb_catalog_png, sizeof(tlb_catalog_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_info.png"))
    {
        wxMemoryInputStream memStream(tlb_info_png, sizeof(tlb_info_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_fixed.xpm"))
    {
        wxBitmap bitmap(tlb_fixed_xpm);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_removable.xpm"))
    {
        wxBitmap bitmap(tlb_removable_xpm);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_virtual.xpm"))
    {
        wxBitmap bitmap(tlb_virtual_xpm);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_timeline.xpm"))
    {
        wxBitmap bitmap(tlb_timeline_xpm);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_search.xpm"))
    {
        wxBitmap bitmap(tlb_search_xpm);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_large_thumbnails.xpm"))
    {
        wxBitmap bitmap(tlb_large_thumbnails_xpm);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_medium_thumbnails.xpm"))
    {
        wxBitmap bitmap(tlb_medium_thumbnails_xpm);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_small_thumbnails.xpm"))
    {
        wxBitmap bitmap(tlb_small_thumbnails_xpm);
        return bitmap;
    }
    else if (name == _T("graphics/tlb_no_thumbnails.xpm"))
    {
        wxBitmap bitmap(tlb_no_thumbnails_xpm);
        return bitmap;
    }
    return wxNullBitmap;
////@end CMainFrame bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CMainFrame::GetIconResource( const wxString& name )
{
    // Icon retrieval
    wxUnusedVar(name);
    if (name == _T("graphics/vvvPicon32.png"))
    {
#ifdef __WXMSW__
		wxIcon icon( wxT("VVVP_ICON") );
#else
        wxMemoryInputStream memStream(vvvPicon32_png, sizeof(vvvPicon32_png));
        wxBitmap bitmap(wxImage(memStream, wxBITMAP_TYPE_ANY, -1), -1);
        wxIcon icon;
        icon.CopyFromBitmap(bitmap);
#endif
		return icon;
    }
    return wxNullIcon;
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_CATALOG_VOLUME
 */

void CMainFrame::OnCatalogVolumeClick( wxCommandEvent& WXUNUSED(event) )
{
    CDialogCatalogImages dialog(this, ID_DIALOG_CATALOG_IMAGES, _("Catalog Images"));
    dialog.ShowModal();

	if( dialog.IsDatabaseChanged() ) {
		wxBusyCursor wait;
		ClearAllRightPanes();
		wxYield();
		// update the database statistics
		wxGetApp().UpdateDBStatisticsPhysical();
		m_TreeCtrlRemovable->LoadData();
		m_TreeCtrlFixed->LoadData();
	}
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for wxID_PREFERENCES
 */

void CMainFrame::OnPreferencesClick( wxCommandEvent& event )
{
	CAppSettings& as = wxGetApp().GetAppSettings();
	CDialogSettings dlg( this, ID_DIALOG_SETTINGS, _("Settings") );
	dlg.SetData( as );
	if( dlg.ShowModal() == wxID_OK ) {
		as = dlg.GetData();
		CLongTaskBeep::SetMinSecondsForBell( as.m_BeepTime );
		CreateExternalProgramsMenu();
		m_ImgViewerFrame->SetupColorManagement();
	}
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for wxID_OPEN
 */

void CMainFrame::OnOpenClick( wxCommandEvent& WXUNUSED(event) )
{
	CAppSettings& as = wxGetApp().GetAppSettings();
	wxString caption = _("Open catalog");
	wxString databaseName = wxEmptyString;
	if( !as.dbConnectToServer ) {
		// open a local file
		wxString wildcard = _("vvvP  files (*.vvp)|*.vvp|All files (*.*)|*.*");
		wxFileDialog fd( this, caption, wxGetApp().GetDefaultDataFolder(), wxEmptyString, wildcard, wxFD_OPEN );
		if( fd.ShowModal() == wxID_OK )
			databaseName = fd.GetPath();
	} else {
		CServerFileDialog dlg( this );
		dlg.SetAction( wxT("O") );
		dlg.SetShowBrowseButton( as.IsLocalhost() );
		if( dlg.ShowModal() == wxID_OK )
			databaseName = dlg.GetCatalogName();

	}

	if( !databaseName.empty() )
		OpenDatabase( databaseName, CUtils::GetExpectedDatabaseVersion() );
}

bool CMainFrame::OpenDatabase( wxString fileName, int expectedVersion ) {

	bool opened = wxGetApp().OpenDatabase( fileName, expectedVersion );

	if( opened ) {
		// store the file in the MRU list
		CAppSettings& as = wxGetApp().GetAppSettings();
		as.m_fileHistory->AddFileToHistory( fileName );

		// set the main window caption
		SetTitle( fileName + wxT(" - ") + CUtils::GetApplicationName() );

		// load data into the physical tree view
		FSDB::Database db = wxGetApp().GetDatabase();
		// the following code will start a lot of database queries
		// executing them in a single transaction makes things faster, especially in an ext4 Linux partition
		db->TransactionStart();
		m_TreeCtrlRemovable->LoadData();
		m_TreeCtrlFixed->LoadData();
		m_TreeCtrlVirtual->LoadData();
		db->TransactionCommit();
		ClearAllRightPanes();
	}

	return opened;
}

void CMainFrame::OnMRUFile( wxCommandEvent& event ) {
	
	CAppSettings& as = wxGetApp().GetAppSettings();
	int i = event.GetId() - wxID_FILE1;
	wxString fileName = as.m_fileHistory->GetHistoryFile( i );
	if( fileName.empty() ) return;
	if( !as.dbConnectToServer && !wxFileName::FileExists(fileName) ) {
		CUtils::MsgErr( _("This catalog file does not exist any more") );
		as.m_fileHistory->RemoveFileFromHistory( i );
		return;
	}
	OpenDatabase( fileName, CUtils::GetExpectedDatabaseVersion() );

}

void CMainFrame::ShowFolderDataPhysical( long pathID ) {

	CFileDataArray* fda;
	vvvPThumbnailCtrl* tc;
	CRightPaneList* lc;

	Database db = wxGetApp().GetDatabase();
	std::string sql = CSqlStrings::SelectFilesInAPath( pathID );
	
	// count the number of rows to be loaded and set the progress bar
	std::string sqlCount = CSqlStrings::SelectCountFilesInAPath( pathID );
	FSDB_CREATE_RS_READONLY_FWDONLY( db, rsCount );
	rsCount.Open( sqlCount );
	int nRows = rsCount["NROWS"];
	rsCount.Close();
	m_StatusBar->ProgressBarSetRange( nRows );
	m_StatusBar->ProgressBarSetValue( 0 );

	fda = GetCurrentFileDataArray();
	switch( m_CurrentView ) {
		case CAppSettings::cvRemovable:
			m_NumRowsToLoadRemovable = nRows;
			tc = m_ThumbnailsCtrlRemovable;
			lc = m_RightPaneListRemovable;
			if( m_LoadingRemovableView ) {
				// we are still loading a previous folder, close the recordset
				rsPtRemovable->Close();
			}
			m_LoadingRemovableView = true;
			rsPtRemovable = db->CreateRecordset( FSDB::rtReadOnlyForwardOnly );
			rsPtRemovable->Open( sql );
			break;
		case CAppSettings::cvFixed:
			m_NumRowsToLoadFixed = nRows;
			tc = m_ThumbnailsCtrlFixed;
			lc = m_RightPaneListFixed;
			if( m_LoadingFixedView ) {
				// we are still loading a previous folder, close the recordset
				rsPtFixed->Close();
			}
			m_LoadingFixedView = true;
			rsPtFixed = db->CreateRecordset( FSDB::rtReadOnlyForwardOnly );
			rsPtFixed->Open( sql );
			break;
		default:
			wxASSERT( false );
	}

	fda->Clear();
	tc->PrepareForLoading();
	lc->PrepareForLoading();

	m_StatusBar->ProgressBarShow();
}


void CMainFrame::ShowFolderDataVirtual( long pathID ) {

	CFileDataArray* fda = &m_DataArrayVirtual;
	vvvPThumbnailCtrl* tc = m_ThumbnailsCtrlVirtual;
	CRightPaneList* lc = m_RightPaneListVirtual;

	Database db = wxGetApp().GetDatabase();

	if( m_LoadingVirtualView ) {
		// we are still loading a previous folder, close the recordset
		rsPtVirtual->Close();
	}

	// count the number of rows to be loaded and set the progress bar
	{
		std::string sqlCount = CSqlStrings::SelectCountVirtualFilesInAPath( pathID );
		FSDB_CREATE_RS_READONLY_FWDONLY( db, rs );
		rs.Open( sqlCount );
		m_NumRowsToLoadVirtual = rs["NROWS"];
		rs.Close();
		m_StatusBar->ProgressBarSetRange( m_NumRowsToLoadVirtual );
		m_StatusBar->ProgressBarSetValue( 0 );
	}

	rsPtVirtual = db->CreateRecordset( FSDB::rtReadOnlyForwardOnly );
	FSDB::Recordset rs = *rsPtVirtual;

	std::string sql = CSqlStrings::SelectVirtualFilesInAPath( pathID );
	rs.Open( sql );
	m_LoadingVirtualView = true;

	fda->Clear();
	tc->PrepareForLoading();
	lc->PrepareForLoading();

	m_StatusBar->ProgressBarShow();
}


void CMainFrame::ShowSearchData( std::string sql ) {

	StoreSearchData( sql, false, false );
	ShowSearchData( false );
}

void CMainFrame::StoreSearchData( std::string sql, bool isVirtual, bool append )
{
	CFileDataArray* fda = &m_DataArraySearch;
	Database db = wxGetApp().GetDatabase();
	FSDB_CREATE_RS_READONLY_FWDONLY( db, rs );
	rs.Open( sql );
	fda->FillFromRecordset( db, rs, isVirtual, append );
	rs.Close();
}

void CMainFrame::ShowSearchData( bool sort )
{
	CFileDataArray* fda = &m_DataArraySearch;
	vvvPThumbnailCtrl* tc = m_ThumbnailsCtrlSearch;
	CRightPaneList* lc = m_RightPaneListSearch;

	if( sort ) fda->Sort();

	tc->ShowContainerData( *fda );
	lc->ShowContainerData( *fda );

	UpdateStatusBar();
}

void CMainFrame::ClearSearchData()
{
	CFileDataArray* fda = &m_DataArraySearch;
	fda->Clear();
}

void CMainFrame::ShowTimelineData( const wxDateTime &dt )
{
    wxBusyCursor bc;

    Database db = wxGetApp().GetDatabase();
    std::string sql = CSqlStrings::SelectCountFilesByDate( dt );
    int nRows = db->ExecuteReturnInt( sql );
	m_StatusBar->ProgressBarSetRange( nRows );
	m_StatusBar->ProgressBarSetValue( 0 );

    m_NumRowsToLoadTimeline = nRows;
	vvvPThumbnailCtrl* tc = m_ThumbnailsCtrlTimeline;
	CRightPaneList* lc = m_RightPaneListTimeline;
    if( m_LoadingTimelineView ) {
        rsPtTimeline->Close();
    }
    m_LoadingTimelineView = true;
    sql = CSqlStrings::SelectFilesByDate( dt );
	rsPtTimeline = db->CreateRecordset( FSDB::rtReadOnlyForwardOnly );
	rsPtTimeline->Open( sql );
	CFileDataArray* fda = &m_DataArrayTimeline;

	fda->Clear();
	tc->PrepareForLoading();
	lc->PrepareForLoading();

	m_StatusBar->ProgressBarShow();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_NO_THUMBNAILS
 */

void CMainFrame::OnViewNoThumbnailsClick( wxCommandEvent& event )
{
	if( !event.IsChecked() ) return;
	SetViewNoThumbnails();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_SMALL_THUMBNAILS
 */

void CMainFrame::OnViewSmallThumbnailsClick( wxCommandEvent& event )
{
	if( !event.IsChecked() ) return;
	SetViewSmallThumbnails();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_MEDIUM_THUMBNAILS
 */

void CMainFrame::OnViewMediumThumbnailsClick( wxCommandEvent& event )
{
	if( !event.IsChecked() ) return;
	SetViewMediumThumbnails();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_LARGE_THUMBNAILS
 */

void CMainFrame::OnViewLargeThumbnailsClick( wxCommandEvent& event )
{
	if( !event.IsChecked() ) return;
	SetViewLargeThumbnails();
}

void CMainFrame::ShowWindowInRightPane( wxWindow* newWindow ) {
	wxWindow* oldWindow = m_SplitterWindow->GetWindow2();
	if( newWindow != oldWindow ) {
		m_SplitterWindow->ReplaceWindow( oldWindow, newWindow );
		oldWindow->Show( false );
		newWindow->Show( true );
		newWindow->SetFocus();
	}
}

void CMainFrame::SetViewNoThumbnails() {
	m_CurrentThumbnailSize = CAppSettings::tsNoThumbnails;
	ShowCorrectWindowInRightPane();
}

void CMainFrame::SetViewSmallThumbnails() {
	m_CurrentThumbnailSize = CAppSettings::tsSmallThumbnails;
	ShowCorrectWindowInRightPane();
}

void CMainFrame::SetViewMediumThumbnails() {
	m_CurrentThumbnailSize = CAppSettings::tsMediumThumbnails;
	ShowCorrectWindowInRightPane();
}

void CMainFrame::SetViewLargeThumbnails() {
	m_CurrentThumbnailSize = CAppSettings::tsLargeThumbnails;
	ShowCorrectWindowInRightPane();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_FIXED_LOCATIONS
 */

void CMainFrame::OnFixedLocationsClick( wxCommandEvent& event )
{
	CFixedLocationsDialog dlg( this, ID_CFIXEDLOCATIONSDIALOG, _("Fixed locations") );
	if( dlg.ShowModal() == wxID_OK ) {
	}
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_FIXED_LOCATIONS
 */

void CMainFrame::OnViewFixedLocationsClick( wxCommandEvent& event )
{
	if( !event.IsChecked() ) return;
	SetViewFixedLocations();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_REMOVABLE_LOCATIONS
 */

void CMainFrame::OnViewRemovableLocationsClick( wxCommandEvent& event )
{
	if( !event.IsChecked() ) return;
	SetViewRemovableLocations();
}

void CMainFrame::ShowWindowInLeftPane( wxWindow* newWindow ) {
	wxWindow* oldWindow = m_SplitterWindow->GetWindow1();
	if( newWindow != oldWindow ) {
		m_SplitterWindow->ReplaceWindow( oldWindow, newWindow );
		oldWindow->Show( false );
		newWindow->Show( true );
	}
}

void CMainFrame::SetViewFixedLocations() {
	m_CurrentView = CAppSettings::cvFixed;
	ShowWindowInLeftPane( m_TreeCtrlFixed );
	ShowCorrectWindowInRightPane();
	m_StatusBar->ProgressBarSetRange( m_NumRowsToLoadFixed);
}

void CMainFrame::SetViewRemovableLocations() {
	m_CurrentView = CAppSettings::cvRemovable;
	ShowWindowInLeftPane( m_TreeCtrlRemovable );
	ShowCorrectWindowInRightPane();
	m_StatusBar->ProgressBarSetRange( m_NumRowsToLoadRemovable );
}

void CMainFrame::SetViewVirtual() {
	m_CurrentView = CAppSettings::cvVirtual;
	ShowWindowInLeftPane( m_TreeCtrlVirtual );
	ShowCorrectWindowInRightPane();
	m_StatusBar->ProgressBarSetRange( m_NumRowsToLoadVirtual );
}

void CMainFrame::SetViewSearch() {
	// enable/disable scope choices
	int ID = m_TreeCtrlFixed->GetSelectedPathID();
	m_SearchPanel->EnableScopeFixedLocation( ID >= 0 );
	ID = m_TreeCtrlRemovable->GetSelectedPathID();
	m_SearchPanel->EnableScopeRemovableLocation( ID >= 0 );
	ID = m_TreeCtrlVirtual->GetSelectedPathID();
	m_SearchPanel->EnableScopeVirtualLocation( ID >= 0 );

	m_CurrentView = CAppSettings::cvSearch;
	ShowWindowInLeftPane( m_SearchPanel );
	ShowCorrectWindowInRightPane();
}

void CMainFrame::SetViewTimeline() {
	m_CurrentView = CAppSettings::cvTimeline;
	ShowWindowInLeftPane( m_TimelinePanel );
	ShowCorrectWindowInRightPane();
	m_StatusBar->ProgressBarSetRange( m_NumRowsToLoadTimeline );
}

wxWindow* CMainFrame::GetCorrectWindowInRightPane() {
	wxWindow *wp;

	// determine the right window
	switch( m_CurrentThumbnailSize ) {
		case CAppSettings::tsNoThumbnails:
			switch( m_CurrentView ) {
				case CAppSettings::cvFixed:
					wp = m_RightPaneListFixed;
					break;
				case CAppSettings::cvRemovable:
					wp = m_RightPaneListRemovable;
					break;
				case CAppSettings::cvVirtual:
					wp = m_RightPaneListVirtual;
					break;
				case CAppSettings::cvTimeline:
					wp = m_RightPaneListTimeline;
					break;
				case CAppSettings::cvSearch:
					wp = m_RightPaneListSearch;
					break;
				default:
					wxASSERT( false );
			}
			break;
		case CAppSettings::tsSmallThumbnails:
		case CAppSettings::tsMediumThumbnails:
		case CAppSettings::tsLargeThumbnails:
			switch( m_CurrentView ) {
				case CAppSettings::cvFixed:
					wp = m_ThumbnailsCtrlFixed;
					break;
				case CAppSettings::cvRemovable:
					wp = m_ThumbnailsCtrlRemovable;
					break;
				case CAppSettings::cvVirtual:
					wp = m_ThumbnailsCtrlVirtual;
					break;
				case CAppSettings::cvTimeline:
					wp = m_ThumbnailsCtrlTimeline;
					break;
				case CAppSettings::cvSearch:
					wp = m_ThumbnailsCtrlSearch;
					break;
				default:
					wxASSERT( false );
			}
			break;
		default:
			wxASSERT( false );
	}

	return wp;
}

wxWindow* CMainFrame::GetCorrectWindowInLeftPane() {
	wxWindow *wp;

	switch( m_CurrentView ) {
		case CAppSettings::cvFixed:
			wp = m_TreeCtrlFixed;
			break;
		case CAppSettings::cvRemovable:
			wp = m_TreeCtrlRemovable;
			break;
		case CAppSettings::cvVirtual:
			wp = m_TreeCtrlVirtual;
			break;
		case CAppSettings::cvTimeline:
			wp = m_TimelinePanel;
			break;
		case CAppSettings::cvSearch:
			wp = m_SearchPanel;
			break;
		default:
			wxASSERT( false );
	}

	return wp;
}

void CMainFrame::ShowCorrectWindowInRightPane() {
	wxWindow *wp = GetCorrectWindowInRightPane();

	// in case of a thumbnails control set the correct thumbnail size
	vvvPThumbnailCtrl* tp = (vvvPThumbnailCtrl*) wp;	// NOTE: this pointer is only valid if the window is really a thumbnails control
	CRightPaneList* lp = (CRightPaneList*) wp;	// NOTE: this pointer is only valid if the window is really a list control control
	CAppSettings& as = wxGetApp().GetAppSettings();
	switch( m_CurrentThumbnailSize ) {
		case CAppSettings::tsSmallThumbnails:
			tp->SetThumbnailImageSize( wxSize(as.m_ThumbnailViewSmallSize, as.m_ThumbnailViewSmallSize) );
			UpdateStatusBar();
			break;
		case CAppSettings::tsMediumThumbnails:
			tp->SetThumbnailImageSize( wxSize(as.m_ThumbnailViewMediumSize, as.m_ThumbnailViewMediumSize) );
			UpdateStatusBar();
			break;
		case CAppSettings::tsLargeThumbnails:
			tp->SetThumbnailImageSize( wxSize(as.m_ThumbnailViewLargeSize, as.m_ThumbnailViewLargeSize) );
			UpdateStatusBar();
			break;
		case CAppSettings::tsNoThumbnails:
			UpdateStatusBar();
			break;
	}		

	ShowWindowInRightPane( wp );
}

void CMainFrame::ClearAllRightPanes() {
	m_ThumbnailsCtrlRemovable->Clear();
	m_ThumbnailsCtrlFixed->Clear();
	m_ThumbnailsCtrlVirtual->Clear();
	m_ThumbnailsCtrlTimeline->Clear();
	m_ThumbnailsCtrlSearch->Clear();
	m_RightPaneListRemovable->DeleteAllItems();
	m_RightPaneListFixed->DeleteAllItems();
	m_RightPaneListVirtual->DeleteAllItems();
	m_RightPaneListTimeline->DeleteAllItems();
	m_RightPaneListSearch->DeleteAllItems();
}



/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_DELETE
 */

void CMainFrame::OnEditDeleteClick( wxCommandEvent& event )
{
	if( !IsEditDeleteEnabled() ) return;

	if( RightPaneHasFocus() && (m_CurrentView == CAppSettings::cvFixed || m_CurrentView == CAppSettings::cvRemovable) ) {
		DeleteSelectedPhysicalFiles();
	}

	if( RightPaneHasFocus() && m_CurrentView == CAppSettings::cvVirtual ) {
		DeleteSelectedVirtualFiles();
	}

	if( !RightPaneHasFocus() && m_CurrentView == CAppSettings::cvRemovable ) {
		if( m_TreeCtrlRemovable->GetSelectedVolumeID() == -1 )
			DeleteSelectedPhysicalFolder();
		else
			DeleteSelectedPhysicalVolume();
	}

	if( !RightPaneHasFocus() && m_CurrentView == CAppSettings::cvFixed ) {
		if( m_TreeCtrlFixed->GetSelectedVolumeID() == -1 )
			DeleteSelectedPhysicalFolder();
		else
			DeleteSelectedPhysicalVolume();
	}

	if( !RightPaneHasFocus() && m_CurrentView == CAppSettings::cvVirtual ) {
		DeleteSelectedVirtualFolder();
	}
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_EDIT_DELETE
 */

void CMainFrame::OnEditDeleteUpdate( wxUpdateUIEvent& event )
{
	event.Enable( IsEditDeleteEnabled() );
}

bool CMainFrame::IsEditDeleteEnabled() {
	bool retVal = false;

	if( !wxGetApp().IsDatabaseConnected() ) return false;

	if( m_CurrentView == CAppSettings::cvSearch ) return false;
	if( m_CurrentView == CAppSettings::cvTimeline ) return false;

	if( RightPaneHasFocus() ) {
		CRightPaneList *lp = GetCurrentRightPaneListCtrl();
		vvvPThumbnailCtrl* tp = GetCurrentThumbnailCtrl();
		if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
			retVal = lp->IsAnythingSelected();
		}
		else {
			retVal = tp->IsAnythingSelected();
		}
	}

	if( !RightPaneHasFocus() && m_CurrentView == CAppSettings::cvRemovable ) {
		retVal = (m_TreeCtrlRemovable->GetSelectedPathID() != -1);
	}
	if( !RightPaneHasFocus() && m_CurrentView == CAppSettings::cvFixed ) {
		retVal = (m_TreeCtrlFixed->GetSelectedPathID() != -1);
	}

	if( !RightPaneHasFocus() && m_CurrentView == CAppSettings::cvVirtual ) {
		retVal = (m_TreeCtrlVirtual->GetSelectedPathID() != -1);
	}

	return retVal;
}

void CMainFrame::DeleteSelectedPhysicalVolume() {

	int volumeID = -1;
	wxString volumeName;
	wxString msg;
	if( m_CurrentView == CAppSettings::cvRemovable ) {
		volumeID = m_TreeCtrlRemovable->GetSelectedVolumeID();
		volumeName = m_TreeCtrlRemovable->GetSelectedItemName();
		msg = _("This command will delete this volume:\n\n") + volumeName + 
			  _("\n\nfrom the vvvP catalog. It will not delete any file from your disk.\n\nDo you really want to delete this volume?");
	}
	if( m_CurrentView == CAppSettings::cvFixed ) {
		volumeID = m_TreeCtrlFixed->GetSelectedVolumeID();
		volumeName = m_TreeCtrlFixed->GetSelectedItemName();
		msg = _("This command will delete this fixed location:\n\n") + volumeName + 
			  _("\n\nfrom the vvvP catalog. It will not delete any file from your disk.\n\nDo you really want to delete this fixed location?");
	}
	if( volumeID == -1 ) return;

	if( !CUtils::MsgAskNo(msg) ) return;

	// delete the volume
	wxBusyCursor wait;
	FSDB::Parameters params = ParametersFactory();
	params->Append( volumeID );
	std::string sql = CSqlStrings::DeleteVolume();
	wxGetApp().GetDatabase()->ExecuteNoReturn( sql, params );

	wxGetApp().UpdateDBStatisticsPhysical();

	ClearCurrentRightPanes();
	wxYield();

	// remove the item from the tree control
	if( m_CurrentView == CAppSettings::cvRemovable ) {
		m_TreeCtrlRemovable->RemoveSelectedItem();
	}
	if( m_CurrentView == CAppSettings::cvFixed ) {
		m_TreeCtrlFixed->RemoveSelectedItem();
	}
}

void CMainFrame::ClearCurrentRightPanes() {
	switch( m_CurrentView ) {
		case CAppSettings::cvFixed:
			m_ThumbnailsCtrlFixed->Clear();
			m_RightPaneListFixed->DeleteAllItems();
			break;
		case CAppSettings::cvRemovable:
			m_ThumbnailsCtrlRemovable->Clear();
			m_RightPaneListRemovable->DeleteAllItems();
			break;
		case CAppSettings::cvVirtual:
			m_ThumbnailsCtrlVirtual->Clear();
			m_RightPaneListVirtual->DeleteAllItems();
			break;
		case CAppSettings::cvTimeline:
			m_ThumbnailsCtrlTimeline->Clear();
			m_RightPaneListTimeline->DeleteAllItems();
			break;
		case CAppSettings::cvSearch:
			m_ThumbnailsCtrlSearch->Clear();
			m_RightPaneListSearch->DeleteAllItems();
			break;
		default:
			wxASSERT( false );
	}

}

vvvPThumbnailCtrl* CMainFrame::GetCurrentThumbnailCtrl() {

	vvvPThumbnailCtrl* tp;
	switch( m_CurrentView ) {
		case CAppSettings::cvFixed:
			tp = m_ThumbnailsCtrlFixed;
			break;
		case CAppSettings::cvRemovable:
			tp = m_ThumbnailsCtrlRemovable;
			break;
		case CAppSettings::cvVirtual:
			tp = m_ThumbnailsCtrlVirtual;
			break;
		case CAppSettings::cvTimeline:
			tp = m_ThumbnailsCtrlTimeline;
			break;
		case CAppSettings::cvSearch:
			tp = m_ThumbnailsCtrlSearch;
			break;
		default:
			wxASSERT( false );
	}
	return tp;
}

CRightPaneList* CMainFrame::GetCurrentRightPaneListCtrl()
{
	CRightPaneList* lp;
	switch( m_CurrentView ) {
		case CAppSettings::cvFixed:
			lp = m_RightPaneListFixed;
			break;
		case CAppSettings::cvRemovable:
			lp = m_RightPaneListRemovable;
			break;
		case CAppSettings::cvVirtual:
			lp = m_RightPaneListVirtual;
			break;
		case CAppSettings::cvTimeline:
			lp = m_RightPaneListTimeline;
			break;
		case CAppSettings::cvSearch:
			lp = m_RightPaneListSearch;
			break;
		default:
			wxASSERT( false );
	}
	return lp;
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_EDIT_ROTATE
 */

void CMainFrame::OnEditRotateUpdate( wxUpdateUIEvent& event )
{
	event.Enable( EnableRotateMenus() );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_ROTATE_RIGHT
 */

void CMainFrame::OnEditRotateRightClick( wxCommandEvent& event )
{
	vvvPThumbnailCtrl* tp = GetCurrentThumbnailCtrl();
	tp->RotateSelectedThumbnails( CUtils::irRotateRight );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_EDIT_ROTATE_RIGHT
 */

void CMainFrame::OnEditRotateRightUpdate( wxUpdateUIEvent& event )
{
	event.Enable( EnableRotateMenus() );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_ROTATE_UPSIDE_DOWN
 */

void CMainFrame::OnEditRotateUpsideDownClick( wxCommandEvent& event )
{
	vvvPThumbnailCtrl* tp = GetCurrentThumbnailCtrl();
	tp->RotateSelectedThumbnails( CUtils::irUpsideDown );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_EDIT_ROTATE_UPSIDE_DOWN
 */

void CMainFrame::OnEditRotateUpsideDownUpdate( wxUpdateUIEvent& event )
{
	event.Enable( EnableRotateMenus() );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_ROTATE_LEFT
 */

void CMainFrame::OnEditRotateLeftClick( wxCommandEvent& event )
{
	vvvPThumbnailCtrl* tp = GetCurrentThumbnailCtrl();
	tp->RotateSelectedThumbnails( CUtils::irRotateLeft );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_EDIT_ROTATE_LEFT
 */

void CMainFrame::OnEditRotateLeftUpdate( wxUpdateUIEvent& event )
{
	event.Enable( EnableRotateMenus() );
}

bool CMainFrame::EnableRotateMenus() {

	// only if we are showing thumbnails
	if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) return false;

	// disable if we are in the left pane
	if( !RightPaneHasFocus() ) return false;

	// see if any thumbnails are selected
	vvvPThumbnailCtrl* tp = GetCurrentThumbnailCtrl();
	return !tp->GetSelections().IsEmpty();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_VIRTUAL_LOCATIONS
 */

void CMainFrame::OnViewVirtualLocationsClick( wxCommandEvent& event )
{
	if( !event.IsChecked() ) return;
	SetViewVirtual();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_ADD_VIRTUAL_FOLDER
 */

void CMainFrame::OnAddVirtualFolderClick( wxCommandEvent& event )
{
	// see if the focus was on the right pane
	bool rightPaneHadFocus = RightPaneHasFocus();

	CDialogChooseVirtualFolder cvf(this, ID_DIALOG_CHOOSE_VIRTUAL_FOLDER, _("Choose virtual folder"));
	cvf.SetCurrentPathArray( m_SelectedVirtualFolderForAdd );
	if( cvf.ShowModal() != wxID_OK ) {
		return;
	}
	int virtualFolderId = cvf.GetVirtualFolderID();
	cvf.GetCurrentPathArray( m_SelectedVirtualFolderForAdd );
	if( virtualFolderId < 0 ) return;

	wxBusyCursor wait;
	CLongTaskBeep ltb;
	Database db = wxGetApp().GetDatabase();
	db->TransactionStart();

	if( RightPaneHasFocus() ) {
		// loop over the selected items
		wxWindow *wp = GetCorrectWindowInRightPane();
		CFileDataArray *fda = GetCurrentFileDataArray();
		if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
			// we are showing the list control
			CRightPaneList *lp = dynamic_cast<CRightPaneList*>( wp );
			wxASSERT( lp != NULL );
			long item = -1;
			for( ;; ) {
				item = lp->GetNextItem( item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED );
				if( item == -1 ) break;
				int fileID = ((*fda)[item]).GetID();
				FSDB::Parameters params = ParametersFactory();
				params->Append( fileID );
				params->Append( virtualFolderId );
				std::string sql = CSqlStrings::AddPhysicalFileToVirtualPath();
				db->ExecuteNoReturn( sql, params );
			}
		}
		else {
			// we are showing the thumbnail control
			vvvPThumbnailCtrl *tp = dynamic_cast<vvvPThumbnailCtrl*>( wp );
			wxASSERT( tp != NULL );
			const wxArrayInt& thumbnails = tp->GetSelections();
			int nSel = (int) thumbnails.GetCount();
			for( int k = 0; k < nSel; k++ ) {
				int selectedIndex = thumbnails[k];
				int fileID = ((*fda)[selectedIndex]).GetID();
				FSDB::Parameters params = ParametersFactory();
				params->Append( fileID );
				params->Append( virtualFolderId );
				std::string sql = CSqlStrings::AddPhysicalFileToVirtualPath();
				db->ExecuteNoReturn( sql, params );
			}
		}
	}
	else {
		CTreeCtrlPhysical *tp = dynamic_cast<CTreeCtrlPhysical*>( GetCorrectWindowInLeftPane() );
		int volumeID = tp->GetSelectedVolumeID();
		int physicalFolderID = tp->GetSelectedPathID();
		if( volumeID >= 0 ) {
			// this is a volume
			FSDB::Parameters params = ParametersFactory();
			params->Append( volumeID );
			params->Append( physicalFolderID );
			params->Append( virtualFolderId );
			std::string sql = CSqlStrings::AppendVolumeToVirtualPath();
			db->ExecuteNoReturn( sql, params );
		}
		else {
			// this is a folder
			FSDB::Parameters params = ParametersFactory();
			params->Append( physicalFolderID );
			params->Append( virtualFolderId );
			std::string sql = CSqlStrings::AppendPhysicalPathToVirtualPath();
			db->ExecuteNoReturn( sql, params );
		}
	}

	db->TransactionCommit();

	// update the database statistics
	wxGetApp().UpdateDBStatisticsVirtual();

	// update the virtual tree control
	m_TreeCtrlVirtual->LoadData();
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_ADD_VIRTUAL_FOLDER
 */

void CMainFrame::OnAddVirtualFolderUpdate( wxUpdateUIEvent& event )
{
	if( !wxGetApp().IsDatabaseConnected() ) {
		event.Enable(false);
		return;
	}

	bool isEnabled = (m_CurrentView == CAppSettings::cvFixed || m_CurrentView == CAppSettings::cvRemovable);
	if( isEnabled ) {
        if( m_CurrentView == CAppSettings::cvFixed ) {
		    isEnabled = (m_TreeCtrlFixed->GetCount() > 0);
        }
        if( m_CurrentView == CAppSettings::cvRemovable ) {
		    isEnabled = (m_TreeCtrlRemovable->GetCount() > 0);
        }
	}

	event.Enable( isEnabled );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_NEW_VIRTUAL_SUBFOLDER
 */

void CMainFrame::OnNewVirtualSubfolderClick( wxCommandEvent& WXUNUSED(event) )
{
	wxTreeItemId item = m_TreeCtrlVirtual->GetSelection();
    CTreeVirtualItemData *itemData = (CTreeVirtualItemData *) m_TreeCtrlVirtual->GetItemData(item);
	long fatherID = itemData->GetPathID();

	CreateNewVirtualFolder( fatherID, _("New virtual subfolder") );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_NEW_VIRTUAL_SUBFOLDER
 */

void CMainFrame::OnNewVirtualSubfolderUpdate( wxUpdateUIEvent& event )
{
	if( !wxGetApp().IsDatabaseConnected() ) {
		event.Enable(false);
		return;
	}

	bool isEnabled = (m_CurrentView == CAppSettings::cvVirtual);
	if( isEnabled ) {
		isEnabled = m_TreeCtrlVirtual->IsNodeSelected();
	}

	event.Enable( isEnabled );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_NEW_VIRTUAL_ROOT_FOLDER
 */

void CMainFrame::OnNewVirtualRootFolderClick( wxCommandEvent& WXUNUSED(event) )
{
	CreateNewVirtualFolder( -1, _("New virtual root folder") );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_NEW_VIRTUAL_ROOT_FOLDER
 */

void CMainFrame::OnNewVirtualRootFolderUpdate( wxUpdateUIEvent& event )
{
	if( !wxGetApp().IsDatabaseConnected() ) {
		event.Enable(false);
		return;
	}

	event.Enable( m_CurrentView == CAppSettings::cvVirtual );
}

bool CMainFrame::RightPaneHasFocus() {

	return m_RightPaneHasFocus;

	//// this is a fix for a probable bug in wxGTK. When popping up a context menu FindFocus() returns NULL in the event handler but not in the
	//// UI update event so I can save the returned value for the subsequent call if it returns NULL
	//static wxWindow *lastWpFocus;
	//
	//wxWindow *wp = GetCorrectWindowInRightPane();
	//wxWindow *wpFocus = wxWindow::FindFocus();

	//if( wpFocus == NULL )
	//	wpFocus = lastWpFocus;
	//else
	//	lastWpFocus = wpFocus;

	//return (wp == wpFocus);
}

void CMainFrame::CreateNewVirtualFolder( int fatherID, wxString windowTitle ) {
	wxTreeItemId fatherItem;
	wxString folderName = wxEmptyString;

	if( fatherID == -1 ) {
		// this is a root folder
		fatherItem = m_TreeCtrlVirtual->GetRootItem();
	}
	else {
		// this is a subfolder
		fatherItem = m_TreeCtrlVirtual->GetSelection();
	}

	// asks for the new folder name
	wxTextEntryDialog ted( this, _("Enter the new folder name"), windowTitle, folderName, wxOK | wxCANCEL );
	if( ted.ShowModal() != wxID_OK ) return;
	folderName = ted.GetValue();
	if( folderName == wxEmptyString ) return;

	// add the folder to the database
	FSDB::Parameters params = ParametersFactory();
	params->Append( CUtils::DBwx2std(folderName) );
	if( fatherID == -1 )
		params->AppendNull();
	else
		params->Append( fatherID );
	params->AppendNull();	// physical path id
	std::string sql = CSqlStrings::CreateVirtualFolder();
	int pathID = wxGetApp().GetDatabase()->ExecuteReturnInt( sql, params );

	if( pathID < 0 ) {
		CUtils::MsgErr( _("The new folder name is already present in the catalog") );
		return;
	}

	// add the folder to the tree control
	wxTreeItemId itemID = m_TreeCtrlVirtual->AppendItem( fatherItem, folderName, 1, 2, 
						new CTreeVirtualItemData(folderName, pathID, wxEmptyString, -1 ) );
	m_TreeCtrlVirtual->SetItemTextColour( itemID, *wxBLUE );
	// set the expanded images
	m_TreeCtrlVirtual->SetItemImage( itemID, 1, wxTreeItemIcon_Expanded );
	m_TreeCtrlVirtual->SetItemImage( itemID, 2, wxTreeItemIcon_SelectedExpanded );
}

CFileDataArray *CMainFrame::GetCurrentFileDataArray() {
	CFileDataArray* fda = NULL;
	switch( m_CurrentView ) {
		case CAppSettings::cvRemovable:
			fda = &m_DataArrayRemovable;
			break;
		case CAppSettings::cvFixed:
			fda = &m_DataArrayFixed;
			break;
		case CAppSettings::cvVirtual:
			fda = &m_DataArrayVirtual;
			break;
		case CAppSettings::cvTimeline:
			fda = &m_DataArrayTimeline;
			break;
		case CAppSettings::cvSearch:
			fda = &m_DataArraySearch;
			break;
		default:
			wxASSERT( false );
	}
	return fda;
}

void CMainFrame::DeleteSelectedPhysicalFiles()
{
	wxASSERT( RightPaneHasFocus() );
	wxASSERT( m_CurrentView == CAppSettings::cvFixed || m_CurrentView == CAppSettings::cvRemovable );

	if( !CUtils::MsgAskNo( _("This command will delete the selected files from the vvvP catalog. It will not delete any file from your disk.\n\nDo you really want to delete the selected files?") ) )
		return;

	wxBusyCursor wait;
	CLongTaskBeep ltb;
	Database db = wxGetApp().GetDatabase();
	db->TransactionStart();

	CTreeCtrlPhysical *trp = dynamic_cast<CTreeCtrlPhysical*>( GetCorrectWindowInLeftPane() );
	int physicalFolderID = trp->GetSelectedPathID();

	CRightPaneList *lp = GetCurrentRightPaneListCtrl();
	vvvPThumbnailCtrl* tp = GetCurrentThumbnailCtrl();
	CFileDataArray *fda = GetCurrentFileDataArray();
	std::vector<int> selectedIndexes;
	if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
		// we are showing the list control
		selectedIndexes = lp->GetSelectedItemsIndexes();
	}
	else {
		// we are showing the thumbnail control
		selectedIndexes = tp->GetSelectedItemsIndexes();
	}
	// remove from the database
	for( int k = 0; k < (int) selectedIndexes.size(); k++ ) {
		int index = selectedIndexes[k];
		int fileID = ((*fda)[index]).GetID();
		FSDB::Parameters params = ParametersFactory();
		params->Append( physicalFolderID );
		params->Append( fileID );
		std::string sql = CSqlStrings::RemovePhysicalFileFromPath();
		db->ExecuteNoReturn( sql, params );
	}
	// remove the files from the right pane moving backwards to avoid problems
	m_HandlingSelectionEvent = true;	// to avoid problems automatically moving selection between list and thumbnails
	for( int k = (int) selectedIndexes.size() - 1; k >= 0; k-- ) {
		int index = selectedIndexes[k];
		lp->DeleteItem( index );
		tp->Delete( index );
	}
	lp->SelectNone();
	tp->SelectNone();
	m_HandlingSelectionEvent = false;

	db->TransactionCommit();

	wxGetApp().UpdateDBStatisticsPhysical();
}

void CMainFrame::DeleteSelectedPhysicalFolder()
{
	CTreeCtrlPhysical *trp = dynamic_cast<CTreeCtrlPhysical*>( GetCorrectWindowInLeftPane() );
	int pathID = trp->GetSelectedPathID();
	wxASSERT( pathID != -1 );

	wxString folderName = trp->GetSelectedItemName();
	wxString msg = _("This command will delete this folder:\n\n") + folderName + 
			  _("\n\nfrom the vvvP catalog. It will not delete any file from your disk.\n\nDo you really want to delete this folder?");
	if( !CUtils::MsgAskNo( msg ) )
		return;

	wxBusyCursor wait;
	FSDB::Parameters params = ParametersFactory();
	params->Append( pathID );
	std::string sql = CSqlStrings::DeletePhysicalFolder();
	wxGetApp().GetDatabase()->ExecuteNoReturn( sql, params );

	wxGetApp().UpdateDBStatisticsPhysical();

	ClearCurrentRightPanes();
	wxYield();

	trp->RemoveSelectedItem();
}

void CMainFrame::DeleteSelectedVirtualFiles() {

	wxASSERT( RightPaneHasFocus() );
	wxASSERT( m_CurrentView == CAppSettings::cvVirtual );

	if( !CUtils::MsgAskNo( _("This command will delete the selected virtual files, but it will not change the files in the other views.\n\nDo you really want to delete the selected virtual files?") ) )
		return;

	wxBusyCursor wait;
	CLongTaskBeep ltb;
	Database db = wxGetApp().GetDatabase();
	db->TransactionStart();

	std::vector<int> selectedIndexes;
	if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
		// we are showing the list control
		selectedIndexes = m_RightPaneListVirtual->GetSelectedItemsIndexes();
	}
	else {
		// we are showing the thumbnail control
		selectedIndexes = m_ThumbnailsCtrlVirtual->GetSelectedItemsIndexes();
	}
	// remove from the database
	for( int k = 0; k < (int) selectedIndexes.size(); k++ ) {
		int index = selectedIndexes[k];
		int fileID = m_DataArrayVirtual[index].GetVirtualID();
		std::string sql = CSqlStrings::DeleteVirtualFile( fileID );
		db->ExecuteNoReturn( sql );
	}
	// remove the files from the right pane moving backwards to avoid problems
	m_HandlingSelectionEvent = true;	// to avoid problems automatically moving selection between list and thumbnails
	for( int k = (int) selectedIndexes.size() - 1; k >= 0; k-- ) {
		int index = selectedIndexes[k];
		m_RightPaneListVirtual->DeleteItem( index );
		m_ThumbnailsCtrlVirtual->Delete( index );
	}
	m_RightPaneListVirtual->SelectNone();
	m_ThumbnailsCtrlVirtual->SelectNone();
	m_HandlingSelectionEvent = false;

	db->TransactionCommit();

	wxGetApp().UpdateDBStatisticsVirtual();
}

void CMainFrame::DeleteSelectedVirtualFolder() {
	if( !CUtils::MsgAskNo( _("This command will delete the selected virtual folder, but it will not change the folders in the other views.\n\nDo you really want to delete the selected virtual folder?") ) )
		return;

	int pathID = m_TreeCtrlVirtual->GetSelectedPathID();
	wxASSERT( pathID != -1 );

	wxBusyCursor wait;
	FSDB::Parameters params = ParametersFactory();
	params->Append( pathID );
	std::string sql = CSqlStrings::DeleteVirtualFolder();
	wxGetApp().GetDatabase()->ExecuteNoReturn( sql, params );

	wxGetApp().UpdateDBStatisticsVirtual();

	ClearCurrentRightPanes();
	wxYield();

	m_TreeCtrlVirtual->RemoveSelectedItem();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for wxID_NEW
 */

void CMainFrame::OnNewClick( wxCommandEvent& WXUNUSED(event) )
{
	CAppSettings& as = wxGetApp().GetAppSettings();
	if( as.dbConnectToServer && !as.IsLocalhost() ) {
		CUtils::MsgErr( _("Unable to create a new catalog.\n\nYou are connecting to a database server on another computer. You must execute this program on the server to be able to create a new catalog.") );
		return;
	}

	wxString caption = _("New catalog");

	// create the name of the database backup file to restore
	wxString appPath = wxStandardPaths::Get().GetExecutablePath();
	wxFileName fn(appPath);
	wxString path = fn.GetPath( wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR );
	wxString backupName = path + _T("vvvP.fbk");
	if( !wxFileExists(backupName) ) {
		CUtils::MsgErr( _("Unable to find the database backup to restore.\n\nYou can try reinstalling the program to solve this problem.") );
		return;
	}

	// ask for the name of the database file to be created
	wxString databaseFile = wxEmptyString;
	if( !as.dbConnectToServer ) {
		// create a local file
		wxString wildcard = _("vvvP  files (*.vvp)|*.vvp|All files (*.*)|*.*");
		wxFileDialog fd( this, caption, wxGetApp().GetDefaultDataFolder(), wxEmptyString, wildcard, wxFD_SAVE );
		if( fd.ShowModal() == wxID_OK )
			databaseFile = fd.GetPath();
	} else {
		CServerFileDialog dlg( this );
		dlg.SetAction( wxT("N") );
		dlg.SetShowBrowseButton( as.IsLocalhost() );
		if( dlg.ShowModal() == wxID_OK )
			databaseFile = dlg.GetCatalogName();
	}

	if ( databaseFile.empty() ) return;

	if( !RestoreDatabase(caption, backupName, databaseFile) ) return;	

	// open the database
	OpenDatabase( databaseFile, CUtils::GetExpectedDatabaseVersion() );

}

bool CMainFrame::RestoreDatabase( wxString caption, wxString backupName, wxString& databaseFile ) {

	// add an extension if needed
	{
		wxFileName fn(databaseFile);
		if( fn.GetExt().empty() )
			databaseFile += wxT(".vvp");
	}

	// check if the backup exists
	if( !wxFileExists(backupName) ) {
		CUtils::MsgErr( _("Unable to continue. The backup file:\n\n") + databaseFile + _("\n\ndoes not exist.") );
		return false;
	}

	// check if the database already exists
	if( wxFileExists(databaseFile) ) {
		CUtils::MsgErr( _("Unable to continue. You have choosen an existing file:\n\n") + databaseFile + _("\n\nYou must type the name of a non-existing file.") );
		return false;
	}

	// check if we can create the database
	wxFile f;
	{
		wxLogNull ln;
		if( !f.Create(databaseFile) ) {
			CUtils::MsgErr( _("Unable to continue: you have choosen a folder where you do not have the right to create a file.") );
			return false;
		}
	}
	f.Close();
	wxRemoveFile(databaseFile);

	wxBusyCursor wait;
	
	// restore the database
	{
		wxString serverName, userName, password;
		CAppSettings& as = wxGetApp().GetAppSettings();
		as.GetConnectionData( serverName, userName, password );
		DatabaseFirebird::Restore( CUtils::wx2std(backupName), CUtils::wx2std(databaseFile), CUtils::wx2std(serverName), CUtils::wx2std(userName), CUtils::wx2std(password) );
	}
	
	// since under Windows the restore process creates an all-uppercase file name, rename it to the original name
#ifdef __WXMSW__
	{
		wxFileName fn(databaseFile);
		wxString name = fn.GetFullName();
		wxString path = fn.GetPath( wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR );
		wxString ucDatabaseFile = path + name.Upper();
		wxString tmpDatabaseFile = ucDatabaseFile + wxT("$$tmp$$");
		if( wxFileExists(tmpDatabaseFile) )
			wxRemoveFile( tmpDatabaseFile );
		wxRenameFile( ucDatabaseFile, tmpDatabaseFile, false );
		wxRenameFile( tmpDatabaseFile, databaseFile, false );
	}
#endif

	return true;
}




/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_OBJECT_INFORMATION
 */

void CMainFrame::OnEditObjectInformationClick( wxCommandEvent& WXUNUSED(event) )
{
	if( !wxGetApp().IsDatabaseConnected() ) return;

	if( RightPaneHasFocus() ) {
		// show the "image info" dialog

		wxWindow *wp = GetCorrectWindowInRightPane();
		switch( m_CurrentThumbnailSize ) {
			case CAppSettings::tsNoThumbnails:
				// list control
				{
					CRightPaneList *lp = dynamic_cast<CRightPaneList*>( wp );
					wxASSERT( lp != NULL );
					lp->ShowInfoDialog();
				}
				break;
			case CAppSettings::tsSmallThumbnails:
			case CAppSettings::tsMediumThumbnails:
			case CAppSettings::tsLargeThumbnails:
				// thumbnails control
				{
				vvvPThumbnailCtrl *tp = dynamic_cast<vvvPThumbnailCtrl*>( wp );
				wxASSERT( tp != NULL );
					tp->ShowInfoDialog();
				}
				break;
			default:
				wxASSERT( false );
		}
		UpdateStatusBar();
	}
	else {
		// show the "object description" dialog
		wxString objectName, objectDescription;
		switch( m_CurrentView ) {
			case CAppSettings::cvRemovable:
				objectName = m_TreeCtrlRemovable->GetSelectedItemName();
				objectDescription = m_TreeCtrlRemovable->GetSelectedItemDescription();
				break;
			case CAppSettings::cvFixed:
				objectName = m_TreeCtrlFixed->GetSelectedItemName();
				objectDescription = m_TreeCtrlFixed->GetSelectedItemDescription();
				break;
			case CAppSettings::cvVirtual:
				objectName = m_TreeCtrlVirtual->GetSelectedItemName();
				objectDescription = m_TreeCtrlVirtual->GetSelectedItemDescription();
				break;
			default:
				wxASSERT( false );
		}
		CDialogObjectDescription dialog( this, ID_CDIALOGOBJECTDESCRIPTION, _("Object description") );
		dialog.SetObjectName( objectName );
		dialog.SetDescription( objectDescription );

		if( dialog.ShowModal() != wxID_OK ) return;

		wxString newDescr = dialog.GetDescription();
		if( newDescr == objectDescription) return;

		switch( m_CurrentView ) {
			case CAppSettings::cvRemovable:
				m_TreeCtrlRemovable->SetSelectedItemDescription( newDescr );
				break;
			case CAppSettings::cvFixed:
				m_TreeCtrlFixed->SetSelectedItemDescription( newDescr );
				break;
			case CAppSettings::cvVirtual:
				m_TreeCtrlVirtual->SetSelectedItemDescription( newDescr );
				break;
		}
	}
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_EDIT_OBJECT_INFORMATION
 */

void CMainFrame::OnEditObjectInformationUpdate( wxUpdateUIEvent& event )
{
	if( !wxGetApp().IsDatabaseConnected() ) {
		event.Enable(false);
		return;
	}

	bool menuEnabled = false;
	if( RightPaneHasFocus() ) {
		// check if anything is selected
		wxWindow *wp = GetCorrectWindowInRightPane();
		if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
			CRightPaneList *lp = dynamic_cast<CRightPaneList*>( wp );
			wxASSERT( lp != NULL );
			menuEnabled = lp->IsAnythingSelected();
		}
		else {
			vvvPThumbnailCtrl *tp = dynamic_cast<vvvPThumbnailCtrl*>( wp );
			wxASSERT( tp != NULL );
			menuEnabled = tp->IsAnythingSelected();
		}
	}
	else {
		if( m_CurrentView == CAppSettings::cvRemovable ) {
			menuEnabled = (!m_TreeCtrlRemovable->GetSelectedItemName().empty());
		}
		if( m_CurrentView == CAppSettings::cvFixed ) {
			menuEnabled = (!m_TreeCtrlFixed->GetSelectedItemName().empty());
		}
		if( m_CurrentView == CAppSettings::cvVirtual ) {
			menuEnabled = (!m_TreeCtrlVirtual->GetSelectedItemName().empty());
		}
	}		

	event.Enable( menuEnabled );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_VIEW_FIXED_LOCATIONS
 */

void CMainFrame::OnViewFixedLocationsUpdate( wxUpdateUIEvent& event )
{
	event.Enable( wxGetApp().IsDatabaseConnected() );
	event.Check( m_CurrentView == CAppSettings::cvFixed );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_VIEW_REMOVABLE_LOCATIONS
 */

void CMainFrame::OnViewRemovableLocationsUpdate( wxUpdateUIEvent& event )
{
	event.Enable( wxGetApp().IsDatabaseConnected() );
	event.Check( m_CurrentView == CAppSettings::cvRemovable );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_VIEW_VIRTUAL_LOCATIONS
 */

void CMainFrame::OnViewVirtualLocationsUpdate( wxUpdateUIEvent& event )
{
	event.Enable( wxGetApp().IsDatabaseConnected() );
	event.Check( m_CurrentView == CAppSettings::cvVirtual );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_VIEW_NO_THUMBNAILS
 */

void CMainFrame::OnViewNoThumbnailsUpdate( wxUpdateUIEvent& event )
{
	event.Enable( wxGetApp().IsDatabaseConnected() );
	event.Check( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_VIEW_SMALL_THUMBNAILS
 */

void CMainFrame::OnViewSmallThumbnailsUpdate( wxUpdateUIEvent& event )
{
	event.Enable( wxGetApp().IsDatabaseConnected() );
	event.Check( m_CurrentThumbnailSize == CAppSettings::tsSmallThumbnails );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_VIEW_MEDIUM_THUMBNAILS
 */

void CMainFrame::OnViewMediumThumbnailsUpdate( wxUpdateUIEvent& event )
{
	event.Enable( wxGetApp().IsDatabaseConnected() );
	event.Check( m_CurrentThumbnailSize == CAppSettings::tsMediumThumbnails );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_VIEW_LARGE_THUMBNAILS
 */

void CMainFrame::OnViewLargeThumbnailsUpdate( wxUpdateUIEvent& event )
{
	event.Enable( wxGetApp().IsDatabaseConnected() );
	event.Check( m_CurrentThumbnailSize == CAppSettings::tsLargeThumbnails );
}

void CMainFrame::UpdateStatusBar() {

	CRightPaneList* lc = GetCurrentRightPaneListCtrl();
	long nObjects = lc->GetNumFilesTot();
	wxULongLong sizeObjects = lc->GetSizeFilesTot();
	// get data about the selected items
	std::vector<CFileData*> selList = lc->GetSelectedItemsCFileData();
	long nSelObjects = 0;
	wxULongLong sizeSelObjects = 0;
	for( size_t k = 0; k < selList.size(); k++ ) {
		nSelObjects++;
		sizeSelObjects += selList[k]->GetSize();
	}

	if( nObjects == 0 )
		m_StatusBar->SetStatusText( wxEmptyString, sbeObjectsNumber );
	else {
		wxString s = _("Objects: ") + CUtils::long2string( nObjects );
		if( nSelObjects > 0 ) {
			s += wxT(" (") + CUtils::long2string( nSelObjects ) + wxT(")");
		}
		m_StatusBar->SetStatusText( s, sbeObjectsNumber );
	}

	if( sizeObjects == 0 )
		m_StatusBar->SetStatusText( wxEmptyString, sbeObjectsSize );
	else {
		wxString s = _("Total size: ") + CUtils::HumanReadableFileSize( sizeObjects );
		if( sizeSelObjects > 0 ) {
			s += wxT(" (") + CUtils::HumanReadableFileSize( sizeSelObjects ) + wxT(")");
		}
		m_StatusBar->SetStatusText( s, sbeObjectsSize );
	}

	if( nSelObjects != 1 ) {
		m_StatusBar->SetStatusText( wxEmptyString, sbeObjectDescription );
	}
	else {
		// show the description in the status bar
		wxString s = CUtils::NewLine2Space( selList[0]->GetDescription() );
		m_StatusBar->SetStatusText( s, sbeObjectDescription );
	}


}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_TOOLBAR
 */

void CMainFrame::OnViewToolbarClick( wxCommandEvent& event )
{
	if( event.IsChecked() )
		m_ToolbarCtrl->Show(true);
	else {
		m_ToolbarCtrl->Show(false);
	}
#ifdef __WXMSW__
	SendSizeEvent();
#else
	wxSize s = GetSize();
	Fit();
	SetSize(s);
#endif
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_STATUS_BAR
 */

void CMainFrame::OnViewStatusBarClick( wxCommandEvent& event )
{
	if( event.IsChecked() )
		m_StatusBar->Show(true);
	else {
		m_StatusBar->Show(false);
	}

#ifdef __WXMSW__
	SendSizeEvent();
#else
	wxSize s = GetSize();
	Fit();
	SetSize(s);
#endif
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_SEARCH_VIEW
 */

void CMainFrame::OnViewSearchViewClick( wxCommandEvent& event )
{
	if( !event.IsChecked() ) return;
	SetViewSearch();
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_VIEW_SEARCH_VIEW
 */

void CMainFrame::OnViewSearchViewUpdate( wxUpdateUIEvent& event )
{
	event.Enable( wxGetApp().IsDatabaseConnected() );
	event.Check( m_CurrentView == CAppSettings::cvSearch );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_COPY_FILES
 */

void CMainFrame::OnCopyFilesClick( wxCommandEvent& WXUNUSED(event) )
{
	std::vector<CFileData*> selectedFileData;
	wxWindow *wp = GetCorrectWindowInRightPane();
	CFileDataArray *fda = GetCurrentFileDataArray();
	if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
		CRightPaneList *lp = dynamic_cast<CRightPaneList*>( wp );
		wxASSERT( lp != NULL );
		selectedFileData = lp->GetSelectedItemsCFileData();
	}
	else {
		vvvPThumbnailCtrl *tp = dynamic_cast<vvvPThumbnailCtrl*>( wp );
		wxASSERT( tp != NULL );
		selectedFileData = tp->GetSelectedItemsCFileData();
	}
	if( selectedFileData.size() == 0 ) return;

	// store the file IDs and file names in two vectors
	std::vector<int> selectedIDs;
	std::vector<wxString> selectedFileNames;
	for( size_t k = 0; k < selectedFileData.size(); k++ ) {
		selectedIDs.push_back( (*selectedFileData[k]).GetID() );
		selectedFileNames.push_back( (*selectedFileData[k]).GetName() );
	}

	// send the vectors to the dialog and show it
	CAppSettings& as = wxGetApp().GetAppSettings();
	CCopyFilesToLocationDialog dlg( this, ID_CCOPYFILESTOLOCATIONDIALOG, _("Copy files to a new location") );
	dlg.SetFilesData( selectedIDs, selectedFileNames );
	dlg.SetPath( as.m_CopyFilesDestinationPath );
	dlg.SetExtractJPEGPrevewFromRAW( as.m_ExtractPreviewWhenCopyingRawFiles );
	if( dlg.ShowModal() == wxID_OK ) {
		as.m_CopyFilesDestinationPath = dlg.GetPath();
		as.m_ExtractPreviewWhenCopyingRawFiles = dlg.GetExtractJPEGPrevewFromRAW();
	}
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_COPY_FILES
 */

void CMainFrame::OnCopyFilesUpdate( wxUpdateUIEvent& event )
{
	bool doEnable = wxGetApp().IsDatabaseConnected();
	if( doEnable ) {
		wxWindow *wp = GetCorrectWindowInRightPane();
		if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
			CRightPaneList *lp = dynamic_cast<CRightPaneList*>( wp );
			wxASSERT( lp != NULL );
			doEnable = lp->IsAnythingSelected();
		}
		else {
			vvvPThumbnailCtrl *tp = dynamic_cast<vvvPThumbnailCtrl*>( wp );
			wxASSERT( tp != NULL );
			doEnable = tp->IsAnythingSelected();
		}
	}

	event.Enable( doEnable );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for wxID_HELP_CONTENTS
 */

void CMainFrame::OnHelpContentsClick( wxCommandEvent& WXUNUSED(event) )
{
	wxGetApp().GetHelpController()->DisplayContents();
}

/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_TOOLS_SHOW_IMAGE
 */

void CMainFrame::OnToolsShowImageClick( wxCommandEvent& WXUNUSED(event) )
{
	ShowSelectedImage();
}

/*!
 * wxEVT_UPDATE_UI event handler for ID_TOOLS_SHOW_IMAGE
 */

void CMainFrame::OnToolsShowImageUpdate( wxUpdateUIEvent& event )
{
	bool doEnable = wxGetApp().IsDatabaseConnected();
	if( doEnable ) {
		wxWindow *wp = GetCorrectWindowInRightPane();
		if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
			CRightPaneList *lp = dynamic_cast<CRightPaneList*>( wp );
			wxASSERT( lp != NULL );
			doEnable = lp->IsAnythingSelected();
		}
		else {
			vvvPThumbnailCtrl *tp = dynamic_cast<vvvPThumbnailCtrl*>( wp );
			wxASSERT( tp != NULL );
			doEnable = tp->IsAnythingSelected();
		}
	}

	event.Enable( doEnable );
}

void CMainFrame::OnThumbnailCtrlLeftDClick( wxThumbnailEvent& event )
{
	ShowSelectedImage();
}

void CMainFrame::OnListviewItemActivated( wxListEvent& event )
{
	ShowSelectedImage();
}

void CMainFrame::ShowSelectedImage()
{
	std::vector<int> selectedIndexes;
	wxWindow *wp = GetCorrectWindowInRightPane();
	CFileDataArray *fda = GetCurrentFileDataArray();
	if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
		CRightPaneList *lp = dynamic_cast<CRightPaneList*>( wp );
		wxASSERT( lp != NULL );
		selectedIndexes = lp->GetSelectedItemsIndexes();
	}
	else {
		vvvPThumbnailCtrl *tp = dynamic_cast<vvvPThumbnailCtrl*>( wp );
		wxASSERT( tp != NULL );
		selectedIndexes = tp->GetSelectedItemsIndexes();
	}
	if( selectedIndexes.size() == 0 ) return;

	// show the first image in the vector
	int imageIndex = selectedIndexes[0];

	m_ImgViewerFrame->SetFiledataArray( fda );
	m_ImgViewerFrame->SetFiledataArrayIndex( imageIndex );
	if( m_ImgViewerFrame->IsIconized() ) {
		m_ImgViewerFrame->ShowCurrentImage();
		m_ImgViewerFrame->Iconize( false );
	}
	else {
		m_ImgViewerFrame->ShowCurrentImage();
		m_ImgViewerFrame->Show( true );
	}
	if( m_ImgViewerFrame->WasMaximized() && !m_ImgViewerFrame->IsMaximized() )
		m_ImgViewerFrame->Maximize( true );
	m_ImgViewerFrame->Raise();
}



void CMainFrame::OnListviewItemSelected( wxListEvent& event )
{
	// the user has selected an item in the listview:
	// select the same item in the thumbnail control to keep controls in sync

	if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
		CopySelectionFromListviewToThumbnails( event.GetIndex() );
		UpdateStatusBar();
	}
}


void CMainFrame::OnThumbnailCtrlSelected( wxThumbnailEvent& event )
{
	// the user has selected items in the thumbnail control:
	// select the same items in the listview to keep controls in sync

	if( m_CurrentThumbnailSize != CAppSettings::tsNoThumbnails ) {
		CopySelectionFromThumbnailsToListview( event.GetIndex() );
		UpdateStatusBar();
	}
}

void CMainFrame::OnListviewItemDeselected( wxListEvent& event )
{
	if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
		CopySelectionFromListviewToThumbnails( event.GetIndex() );
		UpdateStatusBar();
	}
}

void CMainFrame::OnThumbnailCtrlDeselected( wxThumbnailEvent& event )
{
	if( m_CurrentThumbnailSize != CAppSettings::tsNoThumbnails ) {
		CopySelectionFromThumbnailsToListview( event.GetIndex() );
		UpdateStatusBar();
	}
}

void CMainFrame::CopySelectionFromThumbnailsToListview( int eventIndex )
{
	if( m_HandlingSelectionEvent ) return;
	m_HandlingSelectionEvent = true;

	std::vector<int> selectedIndexes;
	vvvPThumbnailCtrl* tp = GetCurrentThumbnailCtrl();
	selectedIndexes = tp->GetSelectedItemsIndexes();

	CRightPaneList *lp = GetCurrentRightPaneListCtrl();
	lp->Freeze();
	lp->SelectNone();
	for( size_t k = 0; k < selectedIndexes.size(); k ++ )
		lp->Select( selectedIndexes[k] );
	if( selectedIndexes.size() > 0 )
		lp->EnsureVisible( eventIndex );
	lp->Thaw();

	m_HandlingSelectionEvent = false;
}

void CMainFrame::CopySelectionFromListviewToThumbnails( int eventIndex )
{
	if( m_HandlingSelectionEvent ) return;
	m_HandlingSelectionEvent = true;

	std::vector<int> selectedIndexes;
	CRightPaneList *lp = GetCurrentRightPaneListCtrl();
	selectedIndexes = lp->GetSelectedItemsIndexes();

	vvvPThumbnailCtrl* tp = GetCurrentThumbnailCtrl();
	tp->Freeze();
	tp->SelectNone();
	for( size_t k = 0; k < selectedIndexes.size(); k ++ )
		tp->Select( selectedIndexes[k] );
	if( selectedIndexes.size() > 0 )
		tp->EnsureVisible( eventIndex );
	tp->Thaw();

	m_HandlingSelectionEvent = false;
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for wxID_ABOUT
 */

void CMainFrame::OnAboutClick( wxCommandEvent& event )
{
	CAboutDialog dlg( this );
	dlg.SetVersion( CUtils::GetApplicationVersion() );
	dlg.ShowModal();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_RENAME
 */

void CMainFrame::OnEditRenameClick( wxCommandEvent& event )
{
	wxString oldName = m_TreeCtrlVirtual->GetSelectedItemName();

	// ask for the new volume name
	wxTextEntryDialog ted( this, _("Enter the new folder name"), _("Rename virtual folder"), oldName, wxOK | wxCANCEL );
	if( ted.ShowModal() != wxID_OK ) return;
	wxString newName = ted.GetValue();
	if( newName == oldName || newName.empty() ) return;

	m_TreeCtrlVirtual->SetSelectedItemName( newName );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_EDIT_RENAME
 */

void CMainFrame::OnEditRenameUpdate( wxUpdateUIEvent& event )
{
	bool doEnable = false;

	if( !RightPaneHasFocus() && m_CurrentView == CAppSettings::cvVirtual ) {
		doEnable = (m_TreeCtrlVirtual->GetSelectedPathID() != -1);
	}

	event.Enable( doEnable );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_THUMBNAILS_DELETE
 */

void CMainFrame::OnEditThumbnailsDeleteClick( wxCommandEvent& event )
{
	std::vector<CFileData*> cfd;
	CRightPaneList* lp = GetCurrentRightPaneListCtrl();
	vvvPThumbnailCtrl* tp = GetCurrentThumbnailCtrl();

	if( !CUtils::MsgAskNo( _("This command will delete the thumbnails of the selected images from the vvvP catalog.\n\nDo you really want to delete the thumbnails?")) )
		return;

	wxBusyCursor bc;
	if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
		cfd = lp->GetSelectedItemsCFileData();
	}
	else {
		cfd = tp->GetSelectedItemsCFileData();
	}

	// now cfd contains data about all the selected images
	Database db = wxGetApp().GetDatabase();
	db->TransactionStart();

	for( size_t k = 0; k < cfd.size(); k++ ) {
		int fileID = (cfd[k])->GetID();
		std::string sql = CSqlStrings::DeleteFileThumbnail( fileID );
		db->ExecuteNoReturn(sql);
		(cfd[k])->DeleteThumbnail();
	}

	db->TransactionCommit();
	tp->ReloadAll();
	tp->Refresh();
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_EDIT_THUMBNAILS_DELETE
 */

void CMainFrame::OnEditThumbnailsDeleteUpdate( wxUpdateUIEvent& event )
{
	event.Enable( EnableEditThumbnailsMenus() );
}

bool CMainFrame::EnableEditThumbnailsMenus() {

	// disable if we are in the left pane
	if( !RightPaneHasFocus() ) return false;

	bool retVal;

	// see if any images are selected
	if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
		CRightPaneList* lp = GetCurrentRightPaneListCtrl();
		retVal = lp->IsAnythingSelected();
	}
	else {
		vvvPThumbnailCtrl* tp = GetCurrentThumbnailCtrl();
		retVal = tp->IsAnythingSelected();
	}

	return retVal;
}



/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_THUMBNAILS_RESIZE
 */

void CMainFrame::OnEditThumbnailsResizeClick( wxCommandEvent& event )
{
	std::vector<CFileData*> cfd;
	CRightPaneList* lp = GetCurrentRightPaneListCtrl();
	vvvPThumbnailCtrl* tp = GetCurrentThumbnailCtrl();

	CAppSettings& as = wxGetApp().GetAppSettings();
	CDialogResizeThumbnails dlg( this );
	dlg.SetThumbnailSize( as.m_ThumbnailSize );
	if( dlg.ShowModal() != wxID_OK  ) return;

	int thNewSize = dlg.GetThumbnailsSize();
	wxBusyCursor bc;
	if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
		cfd = lp->GetSelectedItemsCFileData();
	}
	else {
		cfd = tp->GetSelectedItemsCFileData();
	}

	// now cfd contains data about all the selected images
	Database db = wxGetApp().GetDatabase();
	db->TransactionStart();

	for( size_t k = 0; k < cfd.size(); k++ ) {
		wxImage imgResized;
		int fileID = (cfd[k])->GetID();
		FSDB_CREATE_RS_READWRITE_FWDONLY( db, rs );
		std::string sql = CSqlStrings::SelectFileThumbnail( fileID );
		rs.Open( sql, "THUMBNAILS", "FILE_ID_TH" );
		if( !rs.IsEmpty() ) {
			wxImage thumbnail = wxNullImage;
			int thWidth = 0, thHeight = 0;
			if( !rs["THUMBNAIL"].IsNull() ) {
				// load thumbnail data
				const void *buf;
				int size;
				rs["THUMBNAIL"].GetAsBlobConstPointer( buf, size );
				wxMemoryInputStream mis( buf, size );
				thumbnail.LoadFile( mis, wxBITMAP_TYPE_JPEG );
				thWidth = rs["TH_WIDTH"];
				thHeight = rs["TH_HEIGHT"];
			}
			if( thumbnail.IsOk() && thWidth > 0 && thHeight > 0 ) {
				// we have a valid thumbnail
				int maxSize = thWidth;
				if( thHeight > maxSize ) maxSize = thHeight;
				if( thNewSize < maxSize ) {
					// the thumbnail must be reduced, ok (we do not enlarge thumbnails)
					int w, h;
					char *buf;
					int bufSize;
					imgResized = CCatalogImagesFunctions::ResizeImageToThumbnailSize( thumbnail, thNewSize, w, h );
					CCatalogImagesFunctions::ConvertImageToBlob( imgResized, buf, bufSize );
					rs.Edit();
					rs["THUMBNAIL"].SetAsBlobNoMemoryAllocation( buf, bufSize );
					rs["TH_WIDTH"] = w;
					rs["TH_HEIGHT"] = h;
					rs.Update();
					(cfd[k])->SetThumbnail( imgResized );
				}
			}
		}
		rs.Close();
	}

	db->TransactionCommit();

	tp->ReloadAll();
	tp->Refresh();
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_EDIT_THUMBNAILS_RESIZE
 */

void CMainFrame::OnEditThumbnailsResizeUpdate( wxUpdateUIEvent& event )
{
	event.Enable( EnableEditThumbnailsMenus() );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_EDIT_THUMBNAILS
 */

void CMainFrame::OnEditThumbnailsUpdate( wxUpdateUIEvent& event )
{
	event.Enable( EnableEditThumbnailsMenus() );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for wxID_SELECTALL
 */

void CMainFrame::OnSelectallClick( wxCommandEvent& event )
{
	wxBusyCursor bc;
	CRightPaneList* lp = GetCurrentRightPaneListCtrl();
	lp->SelectAll();	// the thumbnails control is selected automatically by the list control
}


/*!
 * wxEVT_UPDATE_UI event handler for wxID_SELECTALL
 */

void CMainFrame::OnSelectallUpdate( wxUpdateUIEvent& event )
{
	if( !wxGetApp().IsDatabaseConnected() ) {
		event.Enable(false);
		return;
	}

	bool menuEnabled = false;
	if( m_CurrentView == CAppSettings::cvRemovable ) {
		menuEnabled = (!m_TreeCtrlRemovable->GetSelectedItemName().empty());
	}
	if( m_CurrentView == CAppSettings::cvFixed ) {
		menuEnabled = (!m_TreeCtrlFixed->GetSelectedItemName().empty());
	}
	if( m_CurrentView == CAppSettings::cvVirtual ) {
		menuEnabled = (!m_TreeCtrlVirtual->GetSelectedItemName().empty());
	}

	event.Enable( menuEnabled );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_FILE_BACKUP
 */

void CMainFrame::OnFileBackupClick( wxCommandEvent& event )
{
	wxString backupFile = wxEmptyString;
	CAppSettings& as = wxGetApp().GetAppSettings();

	// get the backup file name
	if( as.dbConnectToServer ) {
		// file name relative to the server
		CServerFileDialog dlg( this );
		dlg.SetAction( wxT("B") );
		dlg.SetShowBrowseButton( as.IsLocalhost() );
		if( dlg.ShowModal() == wxID_OK )
			backupFile = dlg.GetCatalogName();
	}
	else {
		// create a local file
		wxString wildcard = _("vvvP backup files (*.vvvpbk)|*.vvvpbk|All files (*.*)|*.*");
		wxFileDialog fd( this, _("Backup catalog"), wxGetApp().GetDefaultDataFolder(), wxEmptyString, wildcard, wxFD_SAVE );
		if( fd.ShowModal() == wxID_OK )
			backupFile = fd.GetPath();
	}
	if ( backupFile.empty() ) return;

	// add an extension if needed
	{
		wxFileName fn(backupFile);
		if( fn.GetExt().empty() )
			backupFile += wxT(".vvvpbk");
	}

	// check if the file already exists
	if( wxFileExists(backupFile) ) {
		wxString msg = _("You have choosen an existing file:\n\n") + backupFile + _("\n\nThe existing backup file will be overwritten. Do you want to overwrite an existing backup?");
		if( !CUtils::MsgAskNo(msg) ) {
			return;
		}
		wxRemoveFile( backupFile );
	}

	// check if we can create the file
	wxFile f;
	{
		wxLogNull ln;
		if( !f.Create(backupFile) ) {
			CUtils::MsgErr( _("Unable to continue: you have choosen a folder where you do not have the right to create a file.") );
			return;
		}
	}
	f.Close();
	wxRemoveFile(backupFile);

	// now do the backup
	wxBusyCursor wait;
	wxYield();
	Database db = wxGetApp().GetDatabase();
	db->Backup( CUtils::wx2std(backupFile) );
	CUtils::MsgInfo( _("Backup completed") );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_FILE_BACKUP
 */

void CMainFrame::OnFileBackupUpdate( wxUpdateUIEvent& event )
{
	if( !wxGetApp().IsDatabaseConnected() ) {
		event.Enable(false);
	}
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_FILE_RESTORE
 */

void CMainFrame::OnFileRestoreClick( wxCommandEvent& event )
{
	CAppSettings& as = wxGetApp().GetAppSettings();
	if( as.dbConnectToServer && !as.IsLocalhost() ) {
		CUtils::MsgErr( _("Unable to restore.\n\nYou are connecting to a database server on another computer. You must execute this command on the server to be able to restore a catalog.") );
		return;
	}

	wxString backupFile, databaseFile;

	CDialogRestore dlg( this );
	if( dlg.ShowModal() != wxID_OK ) return;

	backupFile = dlg.GetBackupName();
	databaseFile = dlg.GetCatalogName();

	if( databaseFile.empty() ) {
		CUtils::MsgErr( _("The catalog name is missing") );
		return;
	}
	if( backupFile.empty() ) {
		CUtils::MsgErr( _("The backup name is missing") );
		return;
	}

	wxString caption = _("Restore catalog");

	if( !RestoreDatabase(caption, backupFile, databaseFile) ) return;	

	CUtils::MsgInfo( _("Restore completed") );
}

void CMainFrame::CreateMacToolbar()
{
	const int bmpSize = 32;	// size of the toolbar bitmaps

    CMainFrame* itemFrame1 = this;

	// this code is copied from the CreateControls() code. The only difference is that it uses larger toolbar
	// to make them look better in the Mac version
    m_ToolbarCtrl = CreateToolBar( wxTB_FLAT|wxTB_HORIZONTAL|wxTB_TEXT, ID_TOOLBAR );
    m_ToolbarCtrl->SetToolBitmapSize(wxSize(bmpSize, bmpSize));
	wxBitmap itemtool3Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_new.xpm")), bmpSize, bmpSize) );
    wxBitmap itemtool3BitmapDisabled;
    m_ToolbarCtrl->AddTool(wxID_NEW, _("New"), itemtool3Bitmap, itemtool3BitmapDisabled, wxITEM_NORMAL, _("Create a new catalog"), wxEmptyString);
	wxBitmap itemtool4Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_open.xpm")), bmpSize, bmpSize) );
    wxBitmap itemtool4BitmapDisabled;
    m_ToolbarCtrl->AddTool(wxID_OPEN, _("Open"), itemtool4Bitmap, itemtool4BitmapDisabled, wxITEM_NORMAL, _("Open an existing catalog"), wxEmptyString);
	wxBitmap itemtool5Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_catalog.png")), bmpSize, bmpSize) );
    wxBitmap itemtool5BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_CATALOG_VOLUME, _("Catalog"), itemtool5Bitmap, itemtool5BitmapDisabled, wxITEM_NORMAL, _("Catalog images"), wxEmptyString);
	wxBitmap itemtool6Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_info.png")), bmpSize, bmpSize) );
    wxBitmap itemtool6BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_EDIT_OBJECT_INFORMATION, _(" Info"), itemtool6Bitmap, itemtool6BitmapDisabled, wxITEM_NORMAL, wxEmptyString, wxEmptyString);
    m_ToolbarCtrl->AddSeparator();
	wxBitmap itemtool8Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_fixed.xpm")), bmpSize, bmpSize) );
    wxBitmap itemtool8BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_FIXED_LOCATIONS, _("Fixed"), itemtool8Bitmap, itemtool8BitmapDisabled, wxITEM_RADIO, _("View fixed locations"), wxEmptyString);
	wxBitmap itemtool9Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_removable.xpm")), bmpSize, bmpSize) );
    wxBitmap itemtool9BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_REMOVABLE_LOCATIONS, _("Remov."), itemtool9Bitmap, itemtool9BitmapDisabled, wxITEM_RADIO, _("View removable locations"), wxEmptyString);
	wxBitmap itemtool10Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_virtual.xpm")), bmpSize, bmpSize) );
    wxBitmap itemtool10BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_VIRTUAL_LOCATIONS, _("Virtual"), itemtool10Bitmap, itemtool10BitmapDisabled, wxITEM_RADIO, _("Virtual view"), wxEmptyString);
	wxBitmap itemtool11Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_search.xpm")), bmpSize, bmpSize) );
    wxBitmap itemtool11BitmapDisabled;
    wxBitmap itemtool11b_Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_timeline.xpm")), bmpSize, bmpSize) );
    wxBitmap itemtool11b_BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_TIMELINE_VIEW, _("Timeline"), itemtool11b_Bitmap, itemtool11b_BitmapDisabled, wxITEM_RADIO, _("Timeline view"), wxEmptyString);
    m_ToolbarCtrl->AddTool(ID_VIEW_SEARCH_VIEW, _("Search"), itemtool11Bitmap, itemtool11BitmapDisabled, wxITEM_RADIO, wxEmptyString, wxEmptyString);
    m_ToolbarCtrl->AddSeparator();
	wxBitmap itemtool13Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_large_thumbnails.xpm")), bmpSize, bmpSize) );
    wxBitmap itemtool13BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_LARGE_THUMBNAILS, _("Large"), itemtool13Bitmap, itemtool13BitmapDisabled, wxITEM_RADIO, _("Large thumbnails"), wxEmptyString);
	wxBitmap itemtool14Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_medium_thumbnails.xpm")), bmpSize, bmpSize) );
    wxBitmap itemtool14BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_MEDIUM_THUMBNAILS, _("Medium"), itemtool14Bitmap, itemtool14BitmapDisabled, wxITEM_RADIO, _("Medium thumbnails"), wxEmptyString);
	wxBitmap itemtool15Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_small_thumbnails.xpm")), bmpSize, bmpSize) );
    wxBitmap itemtool15BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_SMALL_THUMBNAILS, _("Small"), itemtool15Bitmap, itemtool15BitmapDisabled, wxITEM_RADIO, _("Small thumbnails"), wxEmptyString);
	wxBitmap itemtool16Bitmap( CUtils::EnlargeBitmap(itemFrame1->GetBitmapResource(wxT("graphics/tlb_no_thumbnails.xpm")), bmpSize, bmpSize) );
    wxBitmap itemtool16BitmapDisabled;
    m_ToolbarCtrl->AddTool(ID_VIEW_NO_THUMBNAILS, _("No"), itemtool16Bitmap, itemtool16BitmapDisabled, wxITEM_RADIO, _("No thumbnails"), wxEmptyString);
    m_ToolbarCtrl->Realize();

	itemFrame1->SetToolBar(m_ToolbarCtrl);
}



/*!
 * wxEVT_IDLE event handler for ID_MAINFRAME
 */

void CMainFrame::OnIdle( wxIdleEvent& event )
{
	bool doShowProgressBar = false;

	Database db = wxGetApp().GetDatabase();

	if( m_LoadingFixedView ) {
		CFileDataArray* fda = &m_DataArrayFixed;
		vvvPThumbnailCtrl* tc = m_ThumbnailsCtrlFixed;
		CRightPaneList* lc = m_RightPaneListFixed;

		FSDB::Recordset rs = *rsPtFixed;
		int nRead = fda->FillFromRecordsetPartial( db, rs, false, DB_ROWS_READ_CHUNK );
		tc->LoadNewData( *fda );
		lc->LoadNewData( *fda );
		if( m_CurrentView == CAppSettings::cvFixed ) m_StatusBar->ProgressBarSetValue( tc->GetCount() );
		if( nRead == DB_ROWS_READ_CHUNK ) {
			if( m_CurrentView == CAppSettings::cvFixed ) doShowProgressBar = true;
			event.RequestMore();
		}
		else {
			// end of recordset
			rs.Close();
			UpdateStatusBar();
			m_LoadingFixedView = false;
		}
	}

	if( m_LoadingRemovableView ) {
		CFileDataArray* fda = &m_DataArrayRemovable;
		vvvPThumbnailCtrl* tc = m_ThumbnailsCtrlRemovable;
		CRightPaneList* lc = m_RightPaneListRemovable;

		FSDB::Recordset rs = *rsPtRemovable;
		int nRead = fda->FillFromRecordsetPartial( db, rs, false, DB_ROWS_READ_CHUNK );
		tc->LoadNewData( *fda );
		lc->LoadNewData( *fda );
		if( m_CurrentView == CAppSettings::cvRemovable ) m_StatusBar->ProgressBarSetValue( tc->GetCount() );
		if( nRead == DB_ROWS_READ_CHUNK ) {
			if( m_CurrentView == CAppSettings::cvRemovable ) doShowProgressBar = true;
			event.RequestMore();
		}
		else {
			// end of recordset
			rs.Close();
			UpdateStatusBar();
			m_LoadingRemovableView = false;
		}
	}

	if( m_LoadingVirtualView ) {
		CFileDataArray* fda = &m_DataArrayVirtual;
		vvvPThumbnailCtrl* tc = m_ThumbnailsCtrlVirtual;
		CRightPaneList* lc = m_RightPaneListVirtual;

		FSDB::Recordset rs = *rsPtVirtual;
		int nRead = fda->FillFromRecordsetPartial( db, rs, true, DB_ROWS_READ_CHUNK );
		tc->LoadNewData( *fda );
		lc->LoadNewData( *fda );
		if( m_CurrentView == CAppSettings::cvVirtual ) m_StatusBar->ProgressBarSetValue( tc->GetCount() );
		if( nRead == DB_ROWS_READ_CHUNK ) {
			if( m_CurrentView == CAppSettings::cvVirtual ) doShowProgressBar = true;
			event.RequestMore();
		}
		else {
			// end of recordset
			rs.Close();
			UpdateStatusBar();
			m_LoadingVirtualView = false;
		}
	}

	if( m_LoadingTimelineView ) {
		CFileDataArray* fda = &m_DataArrayTimeline;
		vvvPThumbnailCtrl* tc = m_ThumbnailsCtrlTimeline;
		CRightPaneList* lc = m_RightPaneListTimeline;

		FSDB::Recordset rs = *rsPtTimeline;
		int nRead = fda->FillFromRecordsetPartial( db, rs, false, DB_ROWS_READ_CHUNK );
		tc->LoadNewData( *fda );
		lc->LoadNewData( *fda );
		if( m_CurrentView == CAppSettings::cvTimeline ) m_StatusBar->ProgressBarSetValue( tc->GetCount() );
		if( nRead == DB_ROWS_READ_CHUNK ) {
			if( m_CurrentView == CAppSettings::cvTimeline ) doShowProgressBar = true;
			event.RequestMore();
		}
		else {
			// end of recordset
			rs.Close();
			UpdateStatusBar();
			m_LoadingTimelineView = false;
		}
	}

	if( doShowProgressBar ) {
		m_StatusBar->ProgressBarShow();
	}
	else {
		m_StatusBar->ProgressBarHide();
	}
}


/*!
 * wxEVT_COMMAND_SPLITTER_DOUBLECLICKED event handler for ID_SPLITTERWINDOW
 */

void CMainFrame::OnSplitterwindowSashDClick( wxSplitterEvent& event )
{
	// void default processing of double clicks (unsplit window)
	event.Veto();
}

void CMainFrame::OnLeftPaneFocused( fsLeftPaneFocusedEvent& event )
{
	m_RightPaneHasFocus = false;
}

void CMainFrame::OnRightPaneFocused( fsRightPaneFocusedEvent& event )
{
	m_RightPaneHasFocus = true;
}


/*!
 * wxEVT_CLOSE_WINDOW event handler for ID_MAINFRAME
 */

void CMainFrame::OnCloseWindow( wxCloseEvent& event )
{
	// empty the tree control (the default code is very slow so I need to do this manually)
	m_TreeCtrlFixed->Freeze();
	m_TreeCtrlFixed->DeleteAllItems();
	m_TreeCtrlFixed->Thaw();
	m_TreeCtrlRemovable->Freeze();
	m_TreeCtrlRemovable->DeleteAllItems();
	m_TreeCtrlRemovable->Thaw();
	m_TreeCtrlVirtual->Freeze();
	m_TreeCtrlVirtual->DeleteAllItems();
	m_TreeCtrlVirtual->Thaw();

	event.Skip();
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_EDIT_EP0-ID_EDIT_EP9
 */

void CMainFrame::OnEditExternalProgramsClick( wxCommandEvent& event )
{
	// get the command line
	CExternalPrograms& ep = wxGetApp().GetAppSettings().m_ExternalPrograms;
	wxString commandLine;
	bool makeCopy = false;
	switch( event.GetId() ) {
		case ID_EDIT_EP0:
			commandLine = ep[0].GetCommandLine();
			makeCopy = ep[0].GetMakeCopy();
			break;
		case ID_EDIT_EP1:
			commandLine = ep[1].GetCommandLine();
			makeCopy = ep[1].GetMakeCopy();
			break;
		case ID_EDIT_EP2:
			commandLine = ep[2].GetCommandLine();
			makeCopy = ep[2].GetMakeCopy();
			break;
		case ID_EDIT_EP3:
			commandLine = ep[3].GetCommandLine();
			makeCopy = ep[3].GetMakeCopy();
			break;
		case ID_EDIT_EP4:
			commandLine = ep[4].GetCommandLine();
			makeCopy = ep[4].GetMakeCopy();
			break;
		case ID_EDIT_EP5:
			commandLine = ep[5].GetCommandLine();
			makeCopy = ep[5].GetMakeCopy();
			break;
		case ID_EDIT_EP6:
			commandLine = ep[6].GetCommandLine();
			makeCopy = ep[6].GetMakeCopy();
			break;
		case ID_EDIT_EP7:
			commandLine = ep[7].GetCommandLine();
			makeCopy = ep[7].GetMakeCopy();
			break;
		case ID_EDIT_EP8:
			commandLine = ep[8].GetCommandLine();
			makeCopy = ep[8].GetMakeCopy();
			break;
		case ID_EDIT_EP9:
			commandLine = ep[9].GetCommandLine();
			makeCopy = ep[9].GetMakeCopy();
			break;
		default:
			wxASSERT( true );
	}

	// if this command will copy the images to another folder ask for the folder
	wxString tempPath = wxEmptyString;
	if( makeCopy ) {
		tempPath = wxGetApp().GetAppSettings().m_ExternalProgramsTempPath;
		CDlgChoosefolder dlg( this );
		dlg.SetTitle( _("Select temp folder to copy images") );
		dlg.SetDescription( _("This program is about to copy images to a temporary folder before executing the external command.\n\nPlease enter the path to the temporary folder.") );
		dlg.SetPath( tempPath );
		if( dlg.ShowModal() == wxID_OK ) {
			tempPath = dlg.GetPath();
			wxGetApp().GetAppSettings().m_ExternalProgramsTempPath = tempPath;
		} else {
			return;
		}
	}

	// get the list of selected images
	std::vector<CFileData*> selectedFileData;
	wxWindow *wp = GetCorrectWindowInRightPane();
	CFileDataArray *fda = GetCurrentFileDataArray();
	if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
		CRightPaneList *lp = dynamic_cast<CRightPaneList*>( wp );
		wxASSERT( lp != NULL );
		selectedFileData = lp->GetSelectedItemsCFileData();
	}
	else {
		vvvPThumbnailCtrl *tp = dynamic_cast<vvvPThumbnailCtrl*>( wp );
		wxASSERT( tp != NULL );
		selectedFileData = tp->GetSelectedItemsCFileData();
	}
	if( selectedFileData.size() == 0 ) return;

	// add the file names to the command string and copy the files if needed
	wxString qm = wxT("\"");	// quotation mark
	commandLine = qm + commandLine + qm;
	for( size_t k = 0; k < selectedFileData.size(); k++ ) {
		wxString path, name;
		CDbUtils::LocationsStatus st = CDbUtils::FixedLocationPathFromID( (*selectedFileData[k]).GetID(), (*selectedFileData[k]).GetName(), path );
		if( st == CDbUtils::lsMissingFixedLocation ) {
			CUtils::MsgErr( _("One or more images do not have a fixed location defined") );
			return;
		}
		if( st == CDbUtils::lsUndefinedFixedLocation ) {
			CUtils::MsgErr( _("One or more images have a fixed location, but this location is not associated with a physical path in this computer") );
			return;
		}
		if( !wxFileName::IsFileReadable(path) ) {
			CUtils::MsgErr( _("One or more images are missing from the fixed location") );
			return;
		}
		if( ! tempPath.IsEmpty() ) {
			// copy the file to another folder
			wxString tmp = tempPath + wxFileName::GetPathSeparator() + (*selectedFileData[k]).GetName();
			if( wxFileExists(tmp) ) {
				wxString msg = wxString(_("The file:")) + wxT("\n") + tmp + wxT("\n") + _("already exists: do you want to overwrite it?");
				if( !CUtils::MsgAskNo(msg) )
					return;
			}
			wxCopyFile( path, tmp, true );
			commandLine += wxT(" ") + qm + tmp + qm;
		}
		else {
			commandLine += wxT(" ") + qm + path + qm;
		}
	}

	// execute the command
	if( wxExecute(commandLine) == 0 ) {
		CUtils::MsgErr( wxString(_("Unable to execute the command:")) + wxT("\n") + commandLine );
		return;
	}
}

void CMainFrame::CreateExternalProgramsMenu()
{
	// delete the submenu (if any)
	wxMenuItem *mp = m_EditMenu->FindItem( ID_EDIT_EP );
	if( mp != NULL ) {
		m_EditMenu->Destroy( mp );
	}

	// add the needed menu entries
	CExternalPrograms& ep = wxGetApp().GetAppSettings().m_ExternalPrograms;
	if( ep.size() == 0 ) return;

    wxMenu* itemMenu46 = new wxMenu;
	AppendExternalProgramsMenus( itemMenu46, ep );
    m_EditMenu->Append(ID_EDIT_EP, _("External programs"), itemMenu46);
}

void CMainFrame::AppendExternalProgramsMenus( wxMenu *menu, CExternalPrograms& ep )
{
    menu->Append(ID_EDIT_EP0, ep[0].GetDescription(), wxEmptyString, wxITEM_NORMAL);
	if( ep.size() > 1 ) menu->Append(ID_EDIT_EP1, ep[1].GetDescription(), wxEmptyString, wxITEM_NORMAL);
    if( ep.size() > 2 ) menu->Append(ID_EDIT_EP2, ep[2].GetDescription(), wxEmptyString, wxITEM_NORMAL);
    if( ep.size() > 3 ) menu->Append(ID_EDIT_EP3, ep[3].GetDescription(), wxEmptyString, wxITEM_NORMAL);
    if( ep.size() > 4 ) menu->Append(ID_EDIT_EP4, ep[4].GetDescription(), wxEmptyString, wxITEM_NORMAL);
    if( ep.size() > 5 ) menu->Append(ID_EDIT_EP5, ep[5].GetDescription(), wxEmptyString, wxITEM_NORMAL);
    if( ep.size() > 6 ) menu->Append(ID_EDIT_EP6, ep[6].GetDescription(), wxEmptyString, wxITEM_NORMAL);
    if( ep.size() > 7 ) menu->Append(ID_EDIT_EP7, ep[7].GetDescription(), wxEmptyString, wxITEM_NORMAL);
    if( ep.size() > 8 ) menu->Append(ID_EDIT_EP8, ep[8].GetDescription(), wxEmptyString, wxITEM_NORMAL);
    if( ep.size() > 9 ) menu->Append(ID_EDIT_EP9, ep[9].GetDescription(), wxEmptyString, wxITEM_NORMAL);
}

/*!
 * wxEVT_UPDATE_UI event handler for ID_EDIT_EP
 */

void CMainFrame::OnEditEpUpdate( wxUpdateUIEvent& event )
{
	event.Enable( EnableEditExternalProgramsMenus() );
}

bool CMainFrame::EnableEditExternalProgramsMenus() {

	// disable if we are in the left pane
	if( !RightPaneHasFocus() ) return false;

	bool retVal;

	// see if any images are selected
	if( m_CurrentThumbnailSize == CAppSettings::tsNoThumbnails ) {
		CRightPaneList* lp = GetCurrentRightPaneListCtrl();
		retVal = lp->IsAnythingSelected();
	}
	else {
		vvvPThumbnailCtrl* tp = GetCurrentThumbnailCtrl();
		retVal = tp->IsAnythingSelected();
	}

	return retVal;
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_CATALOG_VOLUME
 */

void CMainFrame::OnCatalogVolumeUpdate( wxUpdateUIEvent& event )
{
	event.Enable( wxGetApp().IsDatabaseConnected() );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_FIXED_LOCATIONS
 */

void CMainFrame::OnFixedLocationsUpdate( wxUpdateUIEvent& event )
{
	event.Enable( wxGetApp().IsDatabaseConnected() );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_FILE_CHECK_INTEGRITY
 */

void CMainFrame::OnFileCheckIntegrityClick( wxCommandEvent& event )
{
	wxASSERT( !RightPaneHasFocus() );
	wxASSERT( m_CurrentView == CAppSettings::cvFixed || m_CurrentView == CAppSettings::cvRemovable );

	CTreeCtrlPhysical *tp = dynamic_cast<CTreeCtrlPhysical*>( GetCorrectWindowInLeftPane() );
	int volumeID = tp->GetVolumeIDFromSelectedPath();	// return the volume ID for this path
	int physicalFolderID = tp->GetSelectedPathID();

	wxString rootFolderPath;
	CDbUtils::LocationsStatus ls = CDbUtils::FixedLocationFolderFromPathID( physicalFolderID, rootFolderPath );

	if( ls == CDbUtils::lsMissingFixedLocation ) {
		CUtils::MsgErr( _("No fixed location defined for this folder") );
		return;
	}
	if( ls == CDbUtils::lsUndefinedFixedLocation ) {
		CUtils::MsgErr( _("This folder has a fixed location, but the location is not defined in this computer") );
		return;
	}

	// show a progress dialog
	wxProgressDialog dlg( _("File integrity check"), _("Retrieving the file list..."), 100, this, wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_CAN_ABORT  );
	// enlarge the dialog, since the default is rather small
	//int w, h, x, y;
	//dlg.GetSize( &w, &h );
	//dlg.GetPosition( &x, &y );
	//dlg.SetSize( x - (400 - w)/2, y, 400, h );	// also keep the original position
	//dlg.GetSizer()->Layout();

	// get the files list from the database
	Database db = wxGetApp().GetDatabase();
	CDbFileList ofl( db );
	ofl.CreateFileListFolder( volumeID, physicalFolderID, rootFolderPath );
	std::vector<CDbFileListData> fl = ofl.GetFileList();

	// now check the integrity of the files
	std::vector<wxString> missingFilesList;
	std::vector<wxString> changedFilesList;
	size_t k;
	dlg.Update( 0, _("Checking file integrity...") );
	for( k = 0; k < fl.size(); k++ ) {
		if( wxFileExists(fl[k].path) ) {
			// check hash
			wxFileName fn( fl[k].path );
			wxULongLong fileSize = fn.GetSize();
			wxFile fin( fl[k].path );
			wxUint8 *fileBuffer = new wxUint8[fileSize.ToULong()];
			fin.Read( fileBuffer, fileSize.ToULong() );
			fin.Close();
			// compute the SHA1 hash
			wxString sha1 = CSHA1Wrapper::ComputeHash( fileBuffer, fileSize.ToULong() );
			if( sha1 != fl[k].SHA1 ) {
				changedFilesList.push_back( fl[k].path );
			}
			delete fileBuffer;
		}
		else {
			missingFilesList.push_back( fl[k].path );
		}

		int progressValue = k * 100 / fl.size();
		bool cancel = dlg.Update( progressValue );
		if( !cancel ) {
			if( CUtils::MsgAskNo( _("Are you sure that you want to cancel file integrity checking?") ) ) {
				return;
			}
			else {
				dlg.Resume();
			}
		}
	}

	if( missingFilesList.size() == 0 && changedFilesList.size() == 0 ) {
		CUtils::MsgInfo( _("Integrity checking has been successful: no file change has been detected.") );
	}
	else {
		// report changed files
		CLongTextInfo lti( this, wxID_ANY, _("File integrity check") );
		lti.SetUpperText( _("One or more files have changed.\nHere is a list of the files that failed integrity checking.") );
		wxString fileList = wxEmptyString;

		wxString newLine = wxT("\n");
		if( missingFilesList.size() > 0 ) {
			fileList += _("The following files are missing:") + newLine + newLine;
			for( k = 0; k < missingFilesList.size(); k++ ) {
				fileList += missingFilesList[k] + newLine;
			}
			if( changedFilesList.size() > 0 ) fileList += newLine;
		}
		if( changedFilesList.size() > 0 ) {
			fileList += _("The following files have changed:") + newLine + newLine;
			for( k = 0; k < changedFilesList.size(); k++ ) {
				fileList += changedFilesList[k] + newLine;
			}
		}
		lti.SetLongText( fileList );
		lti.ShowModal();
	}
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_FILE_CHECK_INTEGRITY
 */

void CMainFrame::OnFileCheckIntegrityUpdate( wxUpdateUIEvent& event )
{
	bool enable = wxGetApp().IsDatabaseConnected();

	if( enable ) {
		enable = !RightPaneHasFocus();
	}
	if( enable ) {
		enable = (m_CurrentView == CAppSettings::cvFixed);
	}

	event.Enable( enable );
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_VIEW_TIMELINE_VIEW
 */

void CMainFrame::OnViewTimelineViewClick( wxCommandEvent& event )
{
	if( !event.IsChecked() ) return;
	SetViewTimeline();
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_VIEW_TIMELINE_VIEW
 */

void CMainFrame::OnViewTimelineViewUpdate( wxUpdateUIEvent& event )
{
	event.Enable( wxGetApp().IsDatabaseConnected() );
	event.Check( m_CurrentView == CAppSettings::cvTimeline );
}

