/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        rightpanelist.h
// Purpose:     The list control than can be used to show image details in the right pane
// Author:      
// Modified by: 
// Created:     22/09/2008 11:54:01
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _RIGHTPANELIST_H_
#define _RIGHTPANELIST_H_


/*!
 * Includes
 */

////@begin includes
#include "wx/listctrl.h"
////@end includes

#include <vector>

/*!
 * Forward declarations
 */

////@begin forward declarations
class CRightPaneList;
////@end forward declarations

class CFileData;
class CFileDataArray;

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_RIGHT_LISTCTRL 10035
#define SYMBOL_CRIGHTPANELIST_STYLE wxLC_REPORT|wxLC_NO_SORT_HEADER|wxNO_BORDER
#define SYMBOL_CRIGHTPANELIST_IDNAME ID_RIGHT_LISTCTRL
#define SYMBOL_CRIGHTPANELIST_SIZE wxSize(100, 100)
#define SYMBOL_CRIGHTPANELIST_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CRightPaneList class declaration
 */

class CRightPaneList: public wxListCtrl
{    
    DECLARE_DYNAMIC_CLASS( CRightPaneList )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CRightPaneList();
    CRightPaneList(wxWindow* parent, wxWindowID id, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxLC_ICON, const wxValidator& validator = wxDefaultValidator);

    /// Creation
    bool Create(wxWindow* parent, wxWindowID id, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxLC_ICON, const wxValidator& validator = wxDefaultValidator);

    /// Destructor
    ~CRightPaneList();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

	/// Fill the listview with the data contained in the container
	void ShowContainerData( CFileDataArray &fileDataArray );

	/// Prepare the control for loading data (clears everything)
	void PrepareForLoading();

	/// Append new images to the control
	/// if fileDataArray contains more items than the control appends the new items
	/// the code is based only on the number of elements
	/// the caller MUST always pass the same fileDataArray afted adding some elements
	void LoadNewData( CFileDataArray &fileDataArray );

	/// Create the the listview's headers
	void CreateHeaders();

	/// Store the current list control columns widths in the app settings
	void StoreColumnsWidth();

	/// Return true if one or more items are selected
	bool IsAnythingSelected();

	/// return a vector contining the 0-based indexes of the selected items 
	/// or an empty vector if nothing is selected
	std::vector<int> GetSelectedItemsIndexes();

	/// clear all selections
	void SelectNone();

	/// select an item
	void Select( int index );

	/// select all items
	void SelectAll();

	/// return a vector containing the pointers to the CFileData of all the selected items
	/// or an empty vector if nothing is selected
	std::vector<CFileData*> GetSelectedItemsCFileData();

	/// show the "image information" dialog if a single image is selected,
	/// or the "object description" dialog if multiple images are selected
	void ShowInfoDialog();

	/// return the total number of files shown in the control
	int GetNumFilesTot() { return numFilesTot; }
	/// return the total size of the files shown in the control
	wxULongLong GetSizeFilesTot() { return sizeFilesTot; }

////@begin CRightPaneList event handler declarations

    /// wxEVT_CONTEXT_MENU event handler for ID_RIGHT_LISTCTRL
    void OnContextMenu( wxContextMenuEvent& event );

    /// wxEVT_SET_FOCUS event handler for ID_RIGHT_LISTCTRL
    void OnSetFocus( wxFocusEvent& event );

////@end CRightPaneList event handler declarations

////@begin CRightPaneList member function declarations

////@end CRightPaneList member function declarations

////@begin CRightPaneList member variables
////@end CRightPaneList member variables

private:

	/// Add one row to the list control
	/// row data are stored in the CFileData object
	/// idx is the index of the new row
	void AddRowToListControl( CFileData& fd, long idx );

	CFileDataArray *m_fileDataArray;

	// 0/based index of the column that shows the file description
	int descriptionColumn;

	// total number of files shown in the control
	int numFilesTot;
	// total size of the files shown in the control
	wxULongLong sizeFilesTot;

};

#endif
    // _RIGHTPANELIST_H_
