/////////////////////////////////////////////////////////////////////////////
// Name:        search_panel.h
// Purpose:     
// Author:      
// Modified by: 
// Created:     25/05/2009 11:04:49
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _SEARCH_PANEL_H_
#define _SEARCH_PANEL_H_


/*!
 * Includes
 */

////@begin includes
#include "wx/datectrl.h"
#include "wx/dateevt.h"
#include "fswx/date_picker.h"
////@end includes

#include "utils.h"


/*!
 * Forward declarations
 */

////@begin forward declarations
class CSearchPanel;
class fsDatePickerCtrl;
////@end forward declarations

class CMainFrame;

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_SEARCH_PANEL 10109
#define ID_CHOICE_FILENAME 10110
#define ID_SEARCH_FILE_NAME 10111
#define ID_SEARCH_EXTENSION 10112
#define ID_CHOICE_DESCRIPTION 10114
#define ID_SEARCH_DESCRIPTION 10113
#define ID_STARTDATECTRL 10117
#define ID_ENDDATECTRL 10118
#define ID_RADIOBOX_SEARCH 10115
#define ID_BUTTON_SEARCH 10116
#define SYMBOL_CSEARCHPANEL_STYLE wxSUNKEN_BORDER|wxTAB_TRAVERSAL
#define SYMBOL_CSEARCHPANEL_IDNAME ID_SEARCH_PANEL
#define SYMBOL_CSEARCHPANEL_SIZE wxDefaultSize
#define SYMBOL_CSEARCHPANEL_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CSearchPanel class declaration
 */

class CSearchPanel: public wxPanel
{    
    DECLARE_DYNAMIC_CLASS( CSearchPanel )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CSearchPanel();
    CSearchPanel(wxWindow* parent, wxWindowID id = ID_SEARCH_PANEL, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxSUNKEN_BORDER|wxTAB_TRAVERSAL);

    /// Creation
    bool Create(wxWindow* parent, wxWindowID id = ID_SEARCH_PANEL, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxSUNKEN_BORDER|wxTAB_TRAVERSAL);

    /// Destructor
    ~CSearchPanel();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CSearchPanel event handler declarations

    /// wxEVT_SET_FOCUS event handler for ID_SEARCH_PANEL
    void OnSetFocus( wxFocusEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_SEARCH
    void OnButtonSearchClick( wxCommandEvent& event );

////@end CSearchPanel event handler declarations

    void OnChildSetFocus( wxChildFocusEvent& event );

////@begin CSearchPanel member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CSearchPanel member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CSearchPanel member variables
    wxChoice* m_ComboFileName;
    wxTextCtrl* m_SearchFileName;
    wxTextCtrl* m_SearchExtension;
    wxChoice* m_ComboDescription;
    wxTextCtrl* m_SearchDescription;
    fsDatePickerCtrl* m_StartDate;
    fsDatePickerCtrl* m_EndDate;
    wxRadioBox* m_SearchRadioBox;
    wxButton* m_SearchButton;
////@end CSearchPanel member variables

	/// enable or disable the "selected fixed location folder" depending on the value of the "enable" parameter
	void EnableScopeFixedLocation( bool enable );

	/// enable or disable the "selected removable location folder" depending on the value of the "enable" parameter
	void EnableScopeRemovableLocation( bool enable );

	/// enable or disable the "selected virtual location folder" depending on the value of the "enable" parameter
	void EnableScopeVirtualLocation( bool enable );


private:

	/// return a pointer to the main frame
	CMainFrame* GetMainFrame() { return (CMainFrame*) GetParent()->GetParent(); }

	// recursively searches a physical folder and its subfolders, adding found images to the data list
	void SearchPhysicalFolder( wxString fileName, CUtils::TextSearchKind fnSearchKind, 
		                       wxString extension, wxString description, 
							   CUtils::TextSearchKind descrSearchKind, 
							   wxDateTime startDate, wxDateTime endDate, int folderID, int volumeID );

	// recursively searches a virtual folder and its subfolders, adding found images to the data list
	void SearchVirtualFolder( wxString fileName, CUtils::TextSearchKind fnSearchKind, 
		                       wxString extension, wxString description, 
							   CUtils::TextSearchKind descrSearchKind, 
							   wxDateTime startDate, wxDateTime endDate, int folderID );

};

#endif
    // _SEARCH_PANEL_H_
