/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        serverfiledialog.cpp
// Purpose:     
// Author:      
// Modified by: 
// Created:     07/08/2008 22:01:04
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
#include "wx/mstream.h"
////@end includes

#include "serverfiledialog.h"

#include <wx/filename.h>

////@begin XPM images
////@end XPM images


/*!
 * CServerFileDialog type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CServerFileDialog, wxDialog )


/*!
 * CServerFileDialog event table definition
 */

BEGIN_EVENT_TABLE( CServerFileDialog, wxDialog )

////@begin CServerFileDialog event table entries
    EVT_INIT_DIALOG( CServerFileDialog::OnInitDialog )

    EVT_BUTTON( ID_FILESERVER_BROWSE, CServerFileDialog::OnFileserverBrowseClick )

////@end CServerFileDialog event table entries

END_EVENT_TABLE()


/*!
 * CServerFileDialog constructors
 */

CServerFileDialog::CServerFileDialog()
{
    Init();
}

CServerFileDialog::CServerFileDialog( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * CServerFileDialog creator
 */

bool CServerFileDialog::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CServerFileDialog creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CServerFileDialog creation

	m_WindowPosition.SetWindow( this );
	m_WindowPosition.RestorePosition();

	return true;
}


/*!
 * CServerFileDialog destructor
 */

CServerFileDialog::~CServerFileDialog()
{
////@begin CServerFileDialog destruction
////@end CServerFileDialog destruction

	m_WindowPosition.SavePosition();

}


/*!
 * Member initialisation
 */

void CServerFileDialog::Init()
{
////@begin CServerFileDialog member initialisation
    m_FirstTextRowCtrl = NULL;
    m_CatalogNameCtrl = NULL;
    m_BrowseCtrl = NULL;
////@end CServerFileDialog member initialisation
}


/*!
 * Control creation for CServerFileDialog
 */

void CServerFileDialog::CreateControls()
{    
////@begin CServerFileDialog content construction
    CServerFileDialog* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer2->Add(itemBoxSizer3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    m_FirstTextRowCtrl = new wxStaticText( itemDialog1, wxID_STATIC, _("You are about to open a catalog located on the server."), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer3->Add(m_FirstTextRowCtrl, 0, wxALIGN_LEFT|wxALL, 5);

    wxStaticText* itemStaticText5 = new wxStaticText( itemDialog1, wxID_STATIC, _("Enter the alias or the complete path to the catalog file.\nRemember that the path must be relative to the server, not to the client."), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer3->Add(itemStaticText5, 0, wxALIGN_LEFT|wxALL, 5);

    itemBoxSizer3->Add(5, 5, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer7 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer3->Add(itemBoxSizer7, 0, wxGROW|wxTOP|wxBOTTOM, 5);

    m_CatalogNameCtrl = new wxTextCtrl( itemDialog1, ID_FILESERVER_CATALOG_NAME, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer7->Add(m_CatalogNameCtrl, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_BrowseCtrl = new wxButton( itemDialog1, ID_FILESERVER_BROWSE, _("..."), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT );
    itemBoxSizer7->Add(m_BrowseCtrl, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStdDialogButtonSizer* itemStdDialogButtonSizer10 = new wxStdDialogButtonSizer;

    itemBoxSizer2->Add(itemStdDialogButtonSizer10, 0, wxALIGN_RIGHT|wxALL, 5);
    wxButton* itemButton11 = new wxButton( itemDialog1, wxID_OK, _("&OK"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer10->AddButton(itemButton11);

    wxButton* itemButton12 = new wxButton( itemDialog1, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStdDialogButtonSizer10->AddButton(itemButton12);

    itemStdDialogButtonSizer10->Realize();

    // Set validators
    m_CatalogNameCtrl->SetValidator( wxGenericValidator(& m_CatalogName) );
////@end CServerFileDialog content construction
}


/*!
 * Should we show tooltips?
 */

bool CServerFileDialog::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CServerFileDialog::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CServerFileDialog bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CServerFileDialog bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CServerFileDialog::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CServerFileDialog icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CServerFileDialog icon retrieval
}


/*!
 * wxEVT_INIT_DIALOG event handler for ID_CSERVERFILEDIALOG
 */

void CServerFileDialog::OnInitDialog( wxInitDialogEvent& event )
{
	wxASSERT( m_Action == wxT("O") || m_Action == wxT("N") || m_Action == wxT("B") );

	if( m_Action == wxT("O") ) {
		// called as an open dialog
		SetTitle( _("Open Catalog") );
	}
	if( m_Action == wxT("N") ) {
		// called as a NEW dialog
		SetTitle( _("New Catalog") );
		m_FirstTextRowCtrl->SetLabel( _("You are about to create a new catalog located on the server.") );
	}
	if( m_Action == wxT("B") ) {
		// called as a BACKUP dialog
		SetTitle( _("Backup") );
		m_FirstTextRowCtrl->SetLabel( _("You are about to create a new backup file located on the server.") );
	}
	if( !m_ShowBrowseButton )
		m_BrowseCtrl->Show( false );

	wxDialog::OnInitDialog( event );
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_FILESERVER_BROWSE
 */

void CServerFileDialog::OnFileserverBrowseClick( wxCommandEvent& WXUNUSED(event) )
{
	wxString wildcard;
	wxString caption = this->GetTitle();
	if( m_Action == wxT("B") )
		wildcard = _("vvvP backup files (*.vvvpbk)|*.vvvpbk|All files (*.*)|*.*");
	else
		wildcard = _("vvvP files (*.vvp)|*.vvp|All files (*.*)|*.*");
	wxFileDialog fd( this, caption, wxEmptyString, wxEmptyString, wildcard, m_Action == wxT("O") ? wxFD_OPEN : wxFD_SAVE );
	if( fd.ShowModal() != wxID_OK ) return;
	wxString fileName = fd.GetPath();
	if ( fileName.empty() ) return;

	// add an extension if needed

	if( m_Action == wxT("N") ) {
		wxFileName fn(fileName);
		if( fn.GetExt().empty() )
			fileName += wxT(".vvp");
	}


	m_CatalogNameCtrl->SetValue( fileName );
}

