/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        settings.cpp
// Purpose:     Settings dialog
// Author:      
// Modified by: 
// Created:     06/08/2008 11:00:08
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
#include "wx/mstream.h"
#include "wx/bookctrl.h"
////@end includes

#include "settings.h"
#include "utils.h"
#include "vvvpapp.h"
#include "dlg_external_programs.h"

#ifdef __WXMSW__
	#include "windows_specific.h"
#endif

#include <wx/filedlg.h>

////@begin XPM images
////@end XPM images


/*!
 * CDialogSettings type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CDialogSettings, wxPropertySheetDialog )


/*!
 * CDialogSettings event table definition
 */

BEGIN_EVENT_TABLE( CDialogSettings, wxPropertySheetDialog )

////@begin CDialogSettings event table entries
    EVT_BUTTON( ID_BTN_EXTERNAL_PROGRAMS, CDialogSettings::OnBtnExternalProgramsClick )

    EVT_UPDATE_UI( ID_DS_USE_SRGB_PROFILE, CDialogSettings::OnDsUseSrgbProfileUpdate )

    EVT_UPDATE_UI( ID_DS_USE_FILE_PROFILE, CDialogSettings::OnDsUseFileProfileUpdate )

    EVT_UPDATE_UI( ID_DS_FILE_PROFILE, CDialogSettings::OnDsFileProfileUpdate )

    EVT_BUTTON( ID_DS_BROWSE_FILE_PROFILE, CDialogSettings::OnDsBrowseFileProfileClick )
    EVT_UPDATE_UI( ID_DS_BROWSE_FILE_PROFILE, CDialogSettings::OnDsBrowseFileProfileUpdate )

    EVT_UPDATE_UI( ID_DS_SERVERNAME, CDialogSettings::OnDsServernameUpdate )

    EVT_UPDATE_UI( ID_DS_USERNAME, CDialogSettings::OnDsUsernameUpdate )

    EVT_UPDATE_UI( ID_DS_PASSWORD, CDialogSettings::OnDsPasswordUpdate )

    EVT_BUTTON( ID_DS_TEST_CONNECTION, CDialogSettings::OnDsTestConnectionClick )

////@end CDialogSettings event table entries

    EVT_BUTTON( wxID_HELP, CDialogSettings::OnHelpClick )

END_EVENT_TABLE()


/*!
 * CDialogSettings constructors
 */

CDialogSettings::CDialogSettings()
{
    Init();
}

CDialogSettings::CDialogSettings( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * CDialogSettings creator
 */

bool CDialogSettings::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CDialogSettings creation
    SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY|wxWS_EX_BLOCK_EVENTS);
    SetSheetStyle(wxPROPSHEET_DEFAULT);
    wxPropertySheetDialog::Create( parent, id, caption, pos, size, style );

    CreateButtons(wxOK|wxCANCEL|wxHELP);
    CreateControls();
    LayoutDialog();
    Centre();
////@end CDialogSettings creation

	m_WindowPosition.SetWindow( this );
	m_WindowPosition.RestorePosition();

	return true;
}


/*!
 * CDialogSettings destructor
 */

CDialogSettings::~CDialogSettings()
{
////@begin CDialogSettings destruction
////@end CDialogSettings destruction

	m_WindowPosition.SavePosition();
}


/*!
 * Member initialisation
 */

void CDialogSettings::Init()
{
////@begin CDialogSettings member initialisation
    m_ThumbnailsSizeCtrl = NULL;
    m_ThumbnailViewSmallSizeCtrl = NULL;
    m_ThumbnailViewMediumSizeCtrl = NULL;
    m_ThumbnailViewLargeSizeCtrl = NULL;
    m_UseJpegPreviewCtrl = NULL;
    m_EnableColorManagementCtrl = NULL;
    m_UsesRGBProfileCtrl = NULL;
    m_UseFileProfileCtrl = NULL;
    m_FileProfileNameCtrl = NULL;
    m_BrowseFileProfileCtrl = NULL;
    m_ConnectServerCtrl = NULL;
////@end CDialogSettings member initialisation
}


/*!
 * Control creation for CDialogSettings
 */

void CDialogSettings::CreateControls()
{    
////@begin CDialogSettings content construction
    CDialogSettings* itemPropertySheetDialog1 = this;

    wxPanel* itemPanel2 = new wxPanel( GetBookCtrl(), ID_DS_GENERAL, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxVERTICAL);
    itemPanel2->SetSizer(itemBoxSizer3);

    wxBoxSizer* itemBoxSizer4 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer3->Add(itemBoxSizer4, 0, wxGROW|wxALL, 5);
    wxCheckBox* itemCheckBox5 = new wxCheckBox( itemPanel2, ID_DS_REOPEN_CATALOG, _("Reopen the last used catalog at startup"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox5->SetValue(false);
    itemBoxSizer4->Add(itemCheckBox5, 0, wxALIGN_LEFT|wxALL, 5);

    wxBoxSizer* itemBoxSizer6 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer4->Add(itemBoxSizer6, 0, wxALIGN_LEFT|wxRIGHT|wxTOP|wxBOTTOM, 5);
    wxStaticText* itemStaticText7 = new wxStaticText( itemPanel2, wxID_STATIC, _("Beep for tasks longer than (seconds):"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer6->Add(itemStaticText7, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxSpinCtrl* itemSpinCtrl8 = new wxSpinCtrl( itemPanel2, ID_SPIN_BEEP_TIME, _T("0"), wxDefaultPosition, wxSize(60, -1), wxSP_ARROW_KEYS, 0, 100, 0 );
    itemBoxSizer6->Add(itemSpinCtrl8, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton9 = new wxButton( itemPanel2, ID_BTN_EXTERNAL_PROGRAMS, _("External Programs..."), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer4->Add(itemButton9, 0, wxALIGN_LEFT|wxALL, 5);

    GetBookCtrl()->AddPage(itemPanel2, _("General"));

    wxPanel* itemPanel10 = new wxPanel( GetBookCtrl(), ID_DS_THUMBNAILS, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
    wxBoxSizer* itemBoxSizer11 = new wxBoxSizer(wxVERTICAL);
    itemPanel10->SetSizer(itemBoxSizer11);

    wxBoxSizer* itemBoxSizer12 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer11->Add(itemBoxSizer12, 0, wxGROW|wxALL, 5);
    wxFlexGridSizer* itemFlexGridSizer13 = new wxFlexGridSizer(4, 2, 0, 0);
    itemFlexGridSizer13->AddGrowableRow(0);
    itemBoxSizer12->Add(itemFlexGridSizer13, 0, wxGROW|wxRIGHT|wxTOP|wxBOTTOM, 5);
    wxStaticText* itemStaticText14 = new wxStaticText( itemPanel10, wxID_STATIC, _("Size of the thumbnails stored in the catalog:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer13->Add(itemStaticText14, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_ThumbnailsSizeCtrl = new wxSpinCtrl( itemPanel10, ID_SPIN_THUMBNAILS_SIZE, _T("0"), wxDefaultPosition, wxSize(60, -1), wxSP_ARROW_KEYS, 0, 5000, 0 );
    itemFlexGridSizer13->Add(m_ThumbnailsSizeCtrl, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText16 = new wxStaticText( itemPanel10, wxID_STATIC, _("Screen thumbnails size in small view"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer13->Add(itemStaticText16, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_ThumbnailViewSmallSizeCtrl = new wxSpinCtrl( itemPanel10, ID_SPIN_THUMBNAILS_SMALL_VIEW, _T("0"), wxDefaultPosition, wxSize(60, -1), wxSP_ARROW_KEYS, 0, 5000, 0 );
    itemFlexGridSizer13->Add(m_ThumbnailViewSmallSizeCtrl, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText18 = new wxStaticText( itemPanel10, wxID_STATIC, _("Screen thumbnails size in medium view"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer13->Add(itemStaticText18, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_ThumbnailViewMediumSizeCtrl = new wxSpinCtrl( itemPanel10, ID_SPIN_THUMBNAILS_MEDIUM_VIEW, _T("0"), wxDefaultPosition, wxSize(60, -1), wxSP_ARROW_KEYS, 0, 5000, 0 );
    itemFlexGridSizer13->Add(m_ThumbnailViewMediumSizeCtrl, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText20 = new wxStaticText( itemPanel10, wxID_STATIC, _("Screen thumbnails size in large view"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer13->Add(itemStaticText20, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_ThumbnailViewLargeSizeCtrl = new wxSpinCtrl( itemPanel10, ID_SPIN_THUMBNAILS_LARGE_VIEW, _T("0"), wxDefaultPosition, wxSize(60, -1), wxSP_ARROW_KEYS, 0, 5000, 0 );
    itemFlexGridSizer13->Add(m_ThumbnailViewLargeSizeCtrl, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_UseJpegPreviewCtrl = new wxCheckBox( itemPanel10, ID_CHK_TH_USE_PREVIEW, _("Use JPEG preview if available (faster, possibly lower quality)"), wxDefaultPosition, wxDefaultSize, 0 );
    m_UseJpegPreviewCtrl->SetValue(false);
    itemBoxSizer12->Add(m_UseJpegPreviewCtrl, 0, wxALIGN_LEFT|wxALL, 5);

    GetBookCtrl()->AddPage(itemPanel10, _("Thumbnails"));

    wxPanel* itemPanel23 = new wxPanel( GetBookCtrl(), ID_DS_COLOR, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
    wxBoxSizer* itemBoxSizer24 = new wxBoxSizer(wxVERTICAL);
    itemPanel23->SetSizer(itemBoxSizer24);

    wxBoxSizer* itemBoxSizer25 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer24->Add(itemBoxSizer25, 0, wxGROW|wxALL, 5);
    m_EnableColorManagementCtrl = new wxCheckBox( itemPanel23, ID_DS_ENABLE_COLOR_MANAGEMENT, _("Enable color management"), wxDefaultPosition, wxDefaultSize, 0 );
    m_EnableColorManagementCtrl->SetValue(false);
    itemBoxSizer25->Add(m_EnableColorManagementCtrl, 0, wxALIGN_LEFT|wxALL, 5);

    wxStaticBox* itemStaticBoxSizer27Static = new wxStaticBox(itemPanel23, wxID_ANY, _("Monitor profile"));
    wxStaticBoxSizer* itemStaticBoxSizer27 = new wxStaticBoxSizer(itemStaticBoxSizer27Static, wxVERTICAL);
    itemBoxSizer25->Add(itemStaticBoxSizer27, 0, wxGROW|wxALL, 5);
    m_UsesRGBProfileCtrl = new wxRadioButton( itemPanel23, ID_DS_USE_SRGB_PROFILE, _("Use sRGB profile"), wxDefaultPosition, wxDefaultSize, 0 );
    m_UsesRGBProfileCtrl->SetValue(false);
    itemStaticBoxSizer27->Add(m_UsesRGBProfileCtrl, 0, wxALIGN_LEFT|wxALL, 5);

    wxBoxSizer* itemBoxSizer29 = new wxBoxSizer(wxHORIZONTAL);
    itemStaticBoxSizer27->Add(itemBoxSizer29, 0, wxGROW, 5);
    m_UseFileProfileCtrl = new wxRadioButton( itemPanel23, ID_DS_USE_FILE_PROFILE, _("Use profile from file:"), wxDefaultPosition, wxDefaultSize, 0 );
    m_UseFileProfileCtrl->SetValue(false);
    itemBoxSizer29->Add(m_UseFileProfileCtrl, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_FileProfileNameCtrl = new wxTextCtrl( itemPanel23, ID_DS_FILE_PROFILE, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer29->Add(m_FileProfileNameCtrl, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_BrowseFileProfileCtrl = new wxButton( itemPanel23, ID_DS_BROWSE_FILE_PROFILE, _("..."), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT );
    itemBoxSizer29->Add(m_BrowseFileProfileCtrl, 0, wxALIGN_CENTER_VERTICAL|wxRIGHT|wxTOP|wxBOTTOM, 5);

    GetBookCtrl()->AddPage(itemPanel23, _("Color"));

    wxPanel* itemPanel33 = new wxPanel( GetBookCtrl(), ID_DS_SERVER, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
    wxBoxSizer* itemBoxSizer34 = new wxBoxSizer(wxVERTICAL);
    itemPanel33->SetSizer(itemBoxSizer34);

    wxBoxSizer* itemBoxSizer35 = new wxBoxSizer(wxVERTICAL);
    itemBoxSizer34->Add(itemBoxSizer35, 0, wxGROW|wxALL, 5);
    wxStaticText* itemStaticText36 = new wxStaticText( itemPanel33, wxID_STATIC, _("Check the checkbox below if you want to access catalogs stored\nin a network server. Each computer in the network will be able to\naccess the same catalogs stored in the server.\nDO NOT check the checkbox if you only want to access catalogs\nstored in this computer and you do not want to share them\nwith other computers."), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer35->Add(itemStaticText36, 0, wxGROW|wxALL, 5);

    itemBoxSizer35->Add(4, 4, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    m_ConnectServerCtrl = new wxCheckBox( itemPanel33, ID_DS_CONNECT_SERVER, _("Connect to a network server"), wxDefaultPosition, wxDefaultSize, 0 );
    m_ConnectServerCtrl->SetValue(false);
    itemBoxSizer35->Add(m_ConnectServerCtrl, 0, wxALIGN_LEFT|wxALL, 5);

    wxFlexGridSizer* itemFlexGridSizer39 = new wxFlexGridSizer(3, 2, 0, 0);
    itemFlexGridSizer39->AddGrowableCol(1);
    itemBoxSizer35->Add(itemFlexGridSizer39, 0, wxGROW|wxALL, 5);
    wxStaticText* itemStaticText40 = new wxStaticText( itemPanel33, wxID_STATIC, _("Server"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer39->Add(itemStaticText40, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxTextCtrl* itemTextCtrl41 = new wxTextCtrl( itemPanel33, ID_DS_SERVERNAME, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer39->Add(itemTextCtrl41, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText42 = new wxStaticText( itemPanel33, wxID_STATIC, _("Username"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer39->Add(itemStaticText42, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxTextCtrl* itemTextCtrl43 = new wxTextCtrl( itemPanel33, ID_DS_USERNAME, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer39->Add(itemTextCtrl43, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText44 = new wxStaticText( itemPanel33, wxID_STATIC, _("Password"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer39->Add(itemStaticText44, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxTextCtrl* itemTextCtrl45 = new wxTextCtrl( itemPanel33, ID_DS_PASSWORD, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_PASSWORD );
    itemFlexGridSizer39->Add(itemTextCtrl45, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    itemBoxSizer35->Add(4, 4, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxButton* itemButton47 = new wxButton( itemPanel33, ID_DS_TEST_CONNECTION, _("Test server connection"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer35->Add(itemButton47, 0, wxALIGN_RIGHT|wxALL, 5);

    GetBookCtrl()->AddPage(itemPanel33, _("Server"));

    // Set validators
    itemCheckBox5->SetValidator( wxGenericValidator(& GetData().m_ReopenLastUsedCatalog) );
    itemSpinCtrl8->SetValidator( wxGenericValidator(& GetData().m_BeepTime) );
    m_ThumbnailsSizeCtrl->SetValidator( wxGenericValidator(& GetData().m_ThumbnailSize) );
    m_ThumbnailViewSmallSizeCtrl->SetValidator( wxGenericValidator(& GetData().m_ThumbnailViewSmallSize) );
    m_ThumbnailViewMediumSizeCtrl->SetValidator( wxGenericValidator(& GetData().m_ThumbnailViewMediumSize) );
    m_ThumbnailViewLargeSizeCtrl->SetValidator( wxGenericValidator(& GetData().m_ThumbnailViewLargeSize) );
    m_UseJpegPreviewCtrl->SetValidator( wxGenericValidator(& GetData().m_UseJpegPreview) );
    m_EnableColorManagementCtrl->SetValidator( wxGenericValidator(& GetData().m_EnableColorManagement) );
    m_FileProfileNameCtrl->SetValidator( wxTextValidator(wxFILTER_NONE, & GetData().m_FileProfileName) );
    m_ConnectServerCtrl->SetValidator( wxGenericValidator(& GetData().dbConnectToServer) );
    itemTextCtrl41->SetValidator( wxGenericValidator(& GetData().dbServerName) );
    itemTextCtrl43->SetValidator( wxGenericValidator(& GetData().dbUserName) );
    itemTextCtrl45->SetValidator( wxGenericValidator(& GetData().dbPassword) );
////@end CDialogSettings content construction
}


/*!
 * Should we show tooltips?
 */

bool CDialogSettings::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CDialogSettings::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CDialogSettings bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CDialogSettings bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CDialogSettings::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CDialogSettings icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CDialogSettings icon retrieval
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DS_TEST_CONNECTION
 */

void CDialogSettings::OnDsTestConnectionClick( wxCommandEvent& event )
{
////@begin wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DS_TEST_CONNECTION in CDialogSettings.
    // Before editing this code, remove the block markers.
    event.Skip();
////@end wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DS_TEST_CONNECTION in CDialogSettings. 
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_DS_SERVERNAME
 */

void CDialogSettings::OnDsServernameUpdate( wxUpdateUIEvent& event )
{
	event.Enable( m_ConnectServerCtrl->IsChecked() );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_DS_USERNAME
 */

void CDialogSettings::OnDsUsernameUpdate( wxUpdateUIEvent& event )
{
	event.Enable( m_ConnectServerCtrl->IsChecked() );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_DS_PASSWORD
 */

void CDialogSettings::OnDsPasswordUpdate( wxUpdateUIEvent& event )
{
	event.Enable( m_ConnectServerCtrl->IsChecked() );
}


/*!
 * Transfer data to the window
 */

bool CDialogSettings::TransferDataToWindow()
{
	switch( m_data.m_MonitorProfile ) {
		case CAppSettings::mp_sRGB:
			m_UsesRGBProfileCtrl->SetValue( true );
			break;
		case CAppSettings::mp_FileProfile:
			m_UseFileProfileCtrl->SetValue( true );
			break;
		default:
			wxFAIL;
	}
		
	return wxPropertySheetDialog::TransferDataToWindow();
}

/*!
 * Transfer data from the window
 */

bool CDialogSettings::TransferDataFromWindow()
{
	if( m_UsesRGBProfileCtrl->GetValue() ) {
		m_data.m_MonitorProfile = CAppSettings::mp_sRGB;
	}
	else {
		m_data.m_MonitorProfile = CAppSettings::mp_FileProfile;
	}
	
	return wxPropertySheetDialog::TransferDataFromWindow();
}

bool CDialogSettings::Validate()
{
	if( !wxPropertySheetDialog::Validate() )
		return false;

	int thCatalog = m_ThumbnailsSizeCtrl->GetValue();
	int thSmall = m_ThumbnailViewSmallSizeCtrl->GetValue();
	int thMedium = m_ThumbnailViewMediumSizeCtrl->GetValue();
	int thLarge = m_ThumbnailViewLargeSizeCtrl->GetValue();

	if( thCatalog > 512 ) {
		if( !CUtils::MsgAskNo( _("The size of the thumbnails stored in the catalog is a rather large number: are you sure that you want to use that value?") ) )
			return false;
	}
	if( thSmall > thMedium ) {
		CUtils::MsgErr( _("The small thumbnails size is larger than the medium size") );
		return false;
	}
	if( thMedium > thLarge ) {
		CUtils::MsgErr( _("The medium thumbnails size is larger than the large size") );
		return false;
	}

	// check that a valid monitor profile file has been given
	wxString profileFile = m_FileProfileNameCtrl->GetValue();
	if( !profileFile.IsEmpty() ) {
		if( !wxFileExists(profileFile) ) {
			CUtils::MsgErr( _("The monitor color profile file does not exist") );
			return false;
		}
	}
	if( m_UseFileProfileCtrl->GetValue() && profileFile.IsEmpty() ) {
		CUtils::MsgErr( _("You have selected to use a monitor color profile from a file, but you have not entered the file name") );
		return false;
	}

    return true;
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_HELP
 */

void CDialogSettings::OnHelpClick( wxCommandEvent& WXUNUSED(event) )
{
	int nPageSelected = GetBookCtrl()->GetSelection();
	switch( nPageSelected ) {
		case 0:
			wxGetApp().GetHelpController()->DisplaySection( wxT("settings_general.htm") );
			break;
		case 1:
			wxGetApp().GetHelpController()->DisplaySection( wxT("settings_thumbnails.htm") );
			break;
		case 2:
			wxGetApp().GetHelpController()->DisplaySection( wxT("settings_color.htm") );
			break;
		case 3:
			wxGetApp().GetHelpController()->DisplaySection( wxT("settings_server.htm") );
			break;
		default:
			wxFAIL;
	}
}




/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BTN_EXTERNAL_PROGRAMS
 */

void CDialogSettings::OnBtnExternalProgramsClick( wxCommandEvent& event )
{
	CExternalProgramsDlg dlg( this );
	dlg.SetData( m_data );
	dlg.ShowModal();
	m_data = dlg.GetData();
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_DS_USE_SRGB_PROFILE
 */

void CDialogSettings::OnDsUseSrgbProfileUpdate( wxUpdateUIEvent& event )
{
	event.Enable( m_EnableColorManagementCtrl->GetValue() );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_DS_USE_FILE_PROFILE
 */

void CDialogSettings::OnDsUseFileProfileUpdate( wxUpdateUIEvent& event )
{
	event.Enable( m_EnableColorManagementCtrl->GetValue() );
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_DS_FILE_PROFILE
 */

void CDialogSettings::OnDsFileProfileUpdate( wxUpdateUIEvent& event )
{
	event.Enable( m_EnableColorManagementCtrl->GetValue() && m_UseFileProfileCtrl->GetValue() );
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DS_BROWSE_FILE_PROFILE
 */

void CDialogSettings::OnDsBrowseFileProfileClick( wxCommandEvent& event )
{
	wxString defaultPath = wxEmptyString;

	// set the default profile location for each operating system
#ifdef __WXMSW__
	defaultPath = GetSystemDirectory_wx();
	if( !defaultPath.IsEmpty() ) {
		defaultPath += wxT("spool\\drivers\\color");
	}
#endif
#ifdef __WXGTK__
	defaultPath = wxT("/usr/share/color/icc");
#endif
#ifdef __WXMAC__
	defaultPath = wxT("/Library/ColorSync/Profiles");
#endif

	if( !wxDirExists(defaultPath) ) {
		defaultPath = wxEmptyString;
	}

	wxString wildcard = _("ICC color profiles (*.icc;*.icm)|*.icc;*.icm|All files (*.*)|*.*");
	wxFileDialog dlg( this, _("Select the monitor color profile"), defaultPath, wxEmptyString, wildcard, wxFD_OPEN|wxFD_FILE_MUST_EXIST );
	if( dlg.ShowModal() == wxID_OK ) {
		m_FileProfileNameCtrl->SetValue( dlg.GetPath() );
	}
}


/*!
 * wxEVT_UPDATE_UI event handler for ID_DS_BROWSE_FILE_PROFILE
 */

void CDialogSettings::OnDsBrowseFileProfileUpdate( wxUpdateUIEvent& event )
{
	event.Enable( m_EnableColorManagementCtrl->GetValue() && m_UseFileProfileCtrl->GetValue() );
}

