/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        settings.h
// Purpose:     Settings dialog
// Author:      
// Modified by: 
// Created:     06/08/2008 11:00:08
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _SETTINGS_H_
#define _SETTINGS_H_


/*!
 * Includes
 */

////@begin includes
#include "app_settings.h"
#include "wx/propdlg.h"
#include "wx/valgen.h"
#include "wx/spinctrl.h"
#include "wx/valtext.h"
////@end includes

#include "window_position.h"

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxSpinCtrl;
////@end forward declarations

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_DIALOG_SETTINGS 10011
#define ID_DS_GENERAL 10012
#define ID_DS_REOPEN_CATALOG 10014
#define ID_SPIN_BEEP_TIME 10015
#define ID_BTN_EXTERNAL_PROGRAMS 10016
#define ID_DS_THUMBNAILS 10047
#define ID_SPIN_THUMBNAILS_SIZE 10022
#define ID_SPIN_THUMBNAILS_SMALL_VIEW 10002
#define ID_SPIN_THUMBNAILS_MEDIUM_VIEW 10030
#define ID_SPIN_THUMBNAILS_LARGE_VIEW 10029
#define ID_CHK_TH_USE_PREVIEW 10138
#define ID_DS_COLOR 10162
#define ID_DS_ENABLE_COLOR_MANAGEMENT 10163
#define ID_DS_USE_SRGB_PROFILE 10164
#define ID_DS_USE_FILE_PROFILE 10165
#define ID_DS_FILE_PROFILE 10166
#define ID_DS_BROWSE_FILE_PROFILE 10005
#define ID_DS_SERVER 10013
#define ID_DS_CONNECT_SERVER 10017
#define ID_DS_SERVERNAME 10018
#define ID_DS_USERNAME 10019
#define ID_DS_PASSWORD 10020
#define ID_DS_TEST_CONNECTION 10021
#define SYMBOL_CDIALOGSETTINGS_STYLE wxCAPTION|wxRESIZE_BORDER|wxSYSTEM_MENU|wxCLOSE_BOX
#define SYMBOL_CDIALOGSETTINGS_TITLE _("Settings")
#define SYMBOL_CDIALOGSETTINGS_IDNAME ID_DIALOG_SETTINGS
#define SYMBOL_CDIALOGSETTINGS_SIZE wxSize(399, 299)
#define SYMBOL_CDIALOGSETTINGS_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CDialogSettings class declaration
 */

class CDialogSettings: public wxPropertySheetDialog
{    
    DECLARE_DYNAMIC_CLASS( CDialogSettings )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CDialogSettings();
    CDialogSettings( wxWindow* parent, wxWindowID id = SYMBOL_CDIALOGSETTINGS_IDNAME, const wxString& caption = SYMBOL_CDIALOGSETTINGS_TITLE, const wxPoint& pos = SYMBOL_CDIALOGSETTINGS_POSITION, const wxSize& size = SYMBOL_CDIALOGSETTINGS_SIZE, long style = SYMBOL_CDIALOGSETTINGS_STYLE );

    /// Creation
    bool Create( wxWindow* parent, wxWindowID id = SYMBOL_CDIALOGSETTINGS_IDNAME, const wxString& caption = SYMBOL_CDIALOGSETTINGS_TITLE, const wxPoint& pos = SYMBOL_CDIALOGSETTINGS_POSITION, const wxSize& size = SYMBOL_CDIALOGSETTINGS_SIZE, long style = SYMBOL_CDIALOGSETTINGS_STYLE );

    /// Destructor
    ~CDialogSettings();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

    /// Transfer data to the window
    virtual bool TransferDataToWindow();

    /// Transfer data from the window
    virtual bool TransferDataFromWindow();

	/// Validate input data
	virtual bool Validate();

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for wxID_HELP
    void OnHelpClick( wxCommandEvent& event );

////@begin CDialogSettings event handler declarations

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BTN_EXTERNAL_PROGRAMS
    void OnBtnExternalProgramsClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_DS_USE_SRGB_PROFILE
    void OnDsUseSrgbProfileUpdate( wxUpdateUIEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_DS_USE_FILE_PROFILE
    void OnDsUseFileProfileUpdate( wxUpdateUIEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_DS_FILE_PROFILE
    void OnDsFileProfileUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DS_BROWSE_FILE_PROFILE
    void OnDsBrowseFileProfileClick( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_DS_BROWSE_FILE_PROFILE
    void OnDsBrowseFileProfileUpdate( wxUpdateUIEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_DS_SERVERNAME
    void OnDsServernameUpdate( wxUpdateUIEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_DS_USERNAME
    void OnDsUsernameUpdate( wxUpdateUIEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_DS_PASSWORD
    void OnDsPasswordUpdate( wxUpdateUIEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DS_TEST_CONNECTION
    void OnDsTestConnectionClick( wxCommandEvent& event );

////@end CDialogSettings event handler declarations

////@begin CDialogSettings member function declarations

    /// Data access
    CAppSettings& GetData() { return m_data; }
    const CAppSettings& GetData() const { return m_data; }
    void SetData(const CAppSettings& data) { m_data = data; }

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CDialogSettings member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CDialogSettings member variables
    wxSpinCtrl* m_ThumbnailsSizeCtrl;
    wxSpinCtrl* m_ThumbnailViewSmallSizeCtrl;
    wxSpinCtrl* m_ThumbnailViewMediumSizeCtrl;
    wxSpinCtrl* m_ThumbnailViewLargeSizeCtrl;
    wxCheckBox* m_UseJpegPreviewCtrl;
    wxCheckBox* m_EnableColorManagementCtrl;
    wxRadioButton* m_UsesRGBProfileCtrl;
    wxRadioButton* m_UseFileProfileCtrl;
    wxTextCtrl* m_FileProfileNameCtrl;
    wxButton* m_BrowseFileProfileCtrl;
    wxCheckBox* m_ConnectServerCtrl;
    /// The data edited by this window
    CAppSettings m_data;
////@end CDialogSettings member variables

	CWindowPosition m_WindowPosition;

};

#endif
    // _SETTINGS_H_
