/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        treectrl_physical.h
// Purpose:     
// Author:      
// Modified by: 
// Created:     12/08/2008 17:28:28
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

#ifndef _TREECTRL_PHYSICAL_H_
#define _TREECTRL_PHYSICAL_H_


/*!
 * Includes
 */

////@begin includes
#include "wx/treectrl.h"
////@end includes

#include "fsdb.h"

using namespace FSDB;
/*!
 * Forward declarations
 */

////@begin forward declarations
class CTreeCtrlPhysical;
////@end forward declarations

class CMainFrame;

/*!
 * Control identifiers
 */

////@begin control identifiers
#define SYMBOL_CTREECTRLPHYSICAL_STYLE wxTR_HAS_BUTTONS |wxTR_HIDE_ROOT|wxTR_SINGLE|wxNO_BORDER|wxTR_DEFAULT_STYLE
#define SYMBOL_CTREECTRLPHYSICAL_IDNAME wxID_ANY
#define SYMBOL_CTREECTRLPHYSICAL_SIZE wxSize(100, 100)
#define SYMBOL_CTREECTRLPHYSICAL_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CTreeCtrlPhysical class declaration
 */

class CTreeCtrlPhysical: public wxTreeCtrl
{    
    DECLARE_DYNAMIC_CLASS( CTreeCtrlPhysical )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CTreeCtrlPhysical();
    CTreeCtrlPhysical(wxWindow* parent, wxWindowID id, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxTR_HAS_BUTTONS);

    /// Creation
    bool Create(wxWindow* parent, wxWindowID id, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxTR_HAS_BUTTONS);

    /// Destructor
    ~CTreeCtrlPhysical();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CTreeCtrlPhysical event handler declarations

    /// wxEVT_COMMAND_TREE_SEL_CHANGED event handler for wxID_ANY
    void OnAnySelChanged( wxTreeEvent& event );

    /// wxEVT_COMMAND_TREE_ITEM_EXPANDING event handler for wxID_ANY
    void OnAnyItemExpanding( wxTreeEvent& event );

    /// wxEVT_COMMAND_TREE_ITEM_MENU event handler for wxID_ANY
    void OnAnyItemMenu( wxTreeEvent& event );

    /// wxEVT_SET_FOCUS event handler for wxID_ANY
    void OnSetFocus( wxFocusEvent& event );

////@end CTreeCtrlPhysical event handler declarations

////@begin CTreeCtrlPhysical member function declarations

////@end CTreeCtrlPhysical member function declarations

////@begin CTreeCtrlPhysical member variables
////@end CTreeCtrlPhysical member variables

	/// load data from the database.
	/// it will not load the tree completely for speed reasons
	/// it wil only load two levels to show leaves and to know which leaf contains othe leafs
	void LoadData();

	// set the volume type to handle
	void SetVolumeType( wxString volumeType );

	// return the ID of che currently selected volume, or -1 if no volume is selected
	int GetSelectedVolumeID();

	// return the path ID of che currently selected item, or -1 if nothing is selected
	int GetSelectedPathID();

	// return the ID of the volume that contains the selected item
	int GetVolumeIDFromSelectedPath();

	// return the name (stored in the database) of the selected volume or folder, or an empty string is nothing is selected
	wxString GetSelectedItemName();

	// return the description of the selected volume or folder, or an empty string is nothing is selected
	wxString GetSelectedItemDescription();

	// set the description of the selected volume or folder
	void SetSelectedItemDescription( wxString descr );

	// remove the selected item from the treeview control
	void RemoveSelectedItem();

private:

	/// create a label for a volume in the tree view control
	wxString CreateVolumeLabel( const wxString& VolumeName, const wxString& VolumeDescription );

	/// load the data of a single volume
	void LoadVolume( long volumeID, wxString volumeName, wxString volumeDescription, wxTreeItemId rootID );

	/// load the data of a single folder
	void LoadFolder( long volumeID, wxTreeItemId fatherTreeID, long fatherID );

	/// return a pointer to the main frame
	CMainFrame* GetMainFrame() { return (CMainFrame*) GetParent()->GetParent(); }

	// the type of volume that will be handled by this instance ("R" == removable, "F" == fixed)
	wxString volumeType;

	// if true ignore events that would slow down the program
	// used because calling DeleteAllItems() causes a selection changed event for all the elements of the control
	// and each one would fill the right pane with data that would not be seen slowing things a lot
	bool m_IgnoreEvents;

	// return true if a node is selected
	bool IsNodeSelected();

};

#endif
    // _TREECTRL_PHYSICAL_H_
