/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef _TREECTRL_VIRTUAL_H_
#define _TREECTRL_VIRTUAL_H_


/*!
 * Includes
 */

////@begin includes
#include "wx/treectrl.h"
////@end includes

#include "fsdb.h"

using namespace FSDB;
/*!
 * Forward declarations
 */

////@begin forward declarations
class CTreeCtrlVirtual;
////@end forward declarations

class CMainFrame;

/*!
 * Control identifiers
 */

////@begin control identifiers
#define SYMBOL_CTREECTRLVIRTUAL_STYLE wxTR_HAS_BUTTONS |wxTR_HIDE_ROOT|wxTR_SINGLE|wxNO_BORDER|wxTR_DEFAULT_STYLE
#define SYMBOL_CTREECTRLVIRTUAL_IDNAME wxID_ANY
#define SYMBOL_CTREECTRLVIRTUAL_SIZE wxSize(100, 100)
#define SYMBOL_CTREECTRLVIRTUAL_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CTreeCtrlPhysical class declaration
 */

class CTreeCtrlVirtual: public wxTreeCtrl
{    
    DECLARE_DYNAMIC_CLASS( CTreeCtrlVirtual )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CTreeCtrlVirtual();
    CTreeCtrlVirtual(wxWindow* parent, wxWindowID id, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxTR_HAS_BUTTONS);

    /// Creation
    bool Create(wxWindow* parent, wxWindowID id, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxTR_HAS_BUTTONS);

    /// Destructor
    ~CTreeCtrlVirtual();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CTreeCtrlPhysical event handler declarations

    /// wxEVT_COMMAND_TREE_SEL_CHANGED event handler for wxID_ANY
    void OnAnySelChanged( wxTreeEvent& event );

    /// wxEVT_COMMAND_TREE_ITEM_EXPANDING event handler for wxID_ANY
    void OnAnyItemExpanding( wxTreeEvent& event );

    /// wxEVT_COMMAND_TREE_ITEM_MENU event handler for wxID_ANY
    void OnAnyItemMenu( wxTreeEvent& event );

////@end CTreeCtrlPhysical event handler declarations

    void OnSetFocus( wxFocusEvent& event );

////@begin CTreeCtrlPhysical member function declarations

////@end CTreeCtrlPhysical member function declarations

////@begin CTreeCtrlPhysical member variables
////@end CTreeCtrlPhysical member variables

	/// load data from the database.
	/// it will not load the tree completely for speed reasons
	/// it wil only load two levels to show leaves and to know which leaf contains othe leafs
	void LoadData();

	// return the (virtual) path ID of che currently selected item, or -1 if nothing is selected
	int GetSelectedPathID();

	// return the name (stored in the database) of the selected volume or folder, or an empty string is nothing is selected
	wxString GetSelectedItemName();

	// set a new name for the selected item, also changes the database
	void SetSelectedItemName( wxString newName );

	// return the description of the selected volume or folder, or an empty string is nothing is selected
	wxString GetSelectedItemDescription();

	// set the description of the selected volume or folder
	void SetSelectedItemDescription( wxString descr );

	// remove the selected item from the treeview control
	void RemoveSelectedItem();

	// return true if a node is selected
	bool IsNodeSelected();

	// set to true to make this tree control update the right pane when selection changes
	// this control is also used in the "choode virtual folder" dialog where there is no need to update
	void SetUpdateRightPane( bool doUpdate ) { m_updateRightPane = doUpdate; }

private:

	/// create a label for a volume in the tree view control
	wxString CreateVolumeLabel( const wxString& VolumeName, const wxString& VolumeDescription );

	/// load the data of a single volume
//	void LoadVolume( long volumeID, wxString volumeName, wxString volumeDescription, wxTreeItemId rootID );

	/// load the data of a single folder
	void LoadFolder( wxTreeItemId fatherTreeID, long fatherID );

	/// return a pointer to the main frame
	CMainFrame* GetMainFrame() { return (CMainFrame*) GetParent()->GetParent(); }

	// if true ignore events that would slow down the program
	// used because calling DeleteAllItems() causes a selection changed event for all the elements of the control
	// and each one would fill the right pane with data that would not be seen slowing things a lot
	bool m_IgnoreEvents;

	// true if ve must update the right pane in the main frame
	bool m_updateRightPane;

};

#endif
    // _TREECTRL_VIRTUAL_H_
