/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <wx/app.h>
#include <wx/menu.h>

#include "vvvpapp.h"
#include "vvvp_thumbnailctrl.h"
#include "sqlstrings.h"
#include "app_settings.h"
#include "fsdb.h"
#include "image_information_dialog.h"
#include "object_description_dialog.h"
#include "left_right_panes_events.h"
#include "fs_image.h"

#include "wx/dcmemory.h"

using namespace FSDB;


IMPLEMENT_CLASS(vvvPImageThumbnailItem, wxImageThumbnailItem)
IMPLEMENT_CLASS(vvvPThumbnailCtrl, wxThumbnailCtrl)
BEGIN_EVENT_TABLE( vvvPThumbnailCtrl, wxThumbnailCtrl )
    EVT_CONTEXT_MENU( vvvPThumbnailCtrl::OnContextMenu )
    EVT_SET_FOCUS( vvvPThumbnailCtrl::OnSetFocus )
END_EVENT_TABLE()


/*!
 * vvvPImageThumbnailItem
 */

vvvPImageThumbnailItem::vvvPImageThumbnailItem( CFileData* fileData ) {

	m_fileData = fileData;
	m_filename = m_fileData->GetName();
	m_isThumbnailDirty = true;
}

bool vvvPImageThumbnailItem::Load(wxThumbnailCtrl* ctrl, bool forceLoad) {

	if( forceLoad )
		m_isThumbnailDirty = true;

	return true;
}

void vvvPImageThumbnailItem::CreateCachedBitmap( wxThumbnailCtrl* ctrl ) {

	if( !m_fileData->HasThumbnail() ) {
		// this object does not have a thumbnail, so we use a default image
		m_cachedBitmap = ctrl->GetMissingThumbnailImage();
	}
	else {
		// compute the thumbnail
		wxImage image = m_fileData->GetThumbnailImage();
        wxSize thumbnailSize = ctrl->GetThumbnailImageSize();
        double scaleX = ((double) thumbnailSize.x)/((double) image.GetWidth());
        double scaleY = ((double) thumbnailSize.y)/((double) image.GetHeight());

        if (scaleX < 1.0 || scaleY < 1.0)
        {
            double scale = wxMin(scaleX, scaleY);
            int newWidth = (int) (scale * image.GetWidth());
            int newHeight = (int) (scale * image.GetHeight());
			//fsImage *ip = static_cast<fsImage*>( &image );
			//wxImage imgResized = ip->ScaleFS( newWidth, newHeight );
			//image = imgResized;
			image.Rescale(newWidth, newHeight, wxIMAGE_QUALITY_HIGH);
        }
		// rotate the image
		wxImage rotImage = CUtils::RotateImage( image, m_fileData->GetRotation() );
		m_cachedBitmap = wxBitmap(rotImage);
	}
}

bool vvvPImageThumbnailItem::Draw( wxDC& dc, wxThumbnailCtrl* ctrl, const wxRect& rect, int style ) {

	if( m_isThumbnailDirty ) {
		CreateCachedBitmap( ctrl );		m_isThumbnailDirty = false;
	}

	return wxImageThumbnailItem::Draw( dc, ctrl, rect, style );

}

/// Draw the item background
bool vvvPImageThumbnailItem::DrawBackground(wxDC& dc, wxThumbnailCtrl* ctrl, const wxRect& rect, const wxRect& imageRect, int style, int WXUNUSED(index))
{
    wxColour mediumGrey = ctrl->GetUnselectedThumbnailBackgroundColour();
    wxColour unfocussedDarkGrey = ctrl->GetSelectedThumbnailUnfocussedBackgroundColour();
    wxColour focussedDarkGrey = ctrl->GetSelectedThumbnailFocussedBackgroundColour();
    wxColour darkGrey ;
    if (style & wxTHUMBNAIL_FOCUSSED)
        darkGrey = focussedDarkGrey;
    else
        darkGrey = unfocussedDarkGrey;

    if (style & wxTHUMBNAIL_SELECTED)
    {
        wxBrush brush(darkGrey);
        wxPen pen(darkGrey);
        dc.SetBrush(brush);
        dc.SetPen(pen);
    }
    else
    {
        wxBrush brush(mediumGrey);
        wxPen pen(mediumGrey);
        dc.SetBrush(brush);
        dc.SetPen(pen);
    }

    dc.DrawRectangle(rect);

    if (style & wxTHUMBNAIL_TAGGED)
    {
        wxPen bluePen = ctrl->GetTagColour();
        dc.SetPen(bluePen);

        dc.DrawLine(rect.GetRight(), rect.GetTop(), rect.GetRight(), rect.GetBottom());
        dc.DrawLine(rect.GetLeft(), rect.GetBottom(), rect.GetRight()+1, rect.GetBottom());

        dc.DrawLine(rect.GetLeft(), rect.GetTop(), rect.GetRight(), rect.GetTop());
        dc.DrawLine(rect.GetLeft(), rect.GetTop(), rect.GetLeft(), rect.GetBottom());
    }
    else if (style & wxTHUMBNAIL_SELECTED)
    {
        dc.SetPen(*wxWHITE_PEN);
        dc.DrawLine(rect.GetRight(), rect.GetTop(), rect.GetRight(), rect.GetBottom());
        dc.DrawLine(rect.GetLeft(), rect.GetBottom(), rect.GetRight()+1, rect.GetBottom());

        dc.SetPen(*wxBLACK_PEN);
        dc.DrawLine(rect.GetLeft(), rect.GetTop(), rect.GetRight(), rect.GetTop());
        dc.DrawLine(rect.GetLeft(), rect.GetTop(), rect.GetLeft(), rect.GetBottom());
    }
    else
    {
        dc.SetPen(*wxBLACK_PEN);
        dc.DrawLine(rect.GetRight(), rect.GetTop(), rect.GetRight(), rect.GetBottom());
        dc.DrawLine(rect.GetLeft(), rect.GetBottom(), rect.GetRight()+1, rect.GetBottom());

        dc.SetPen(*wxWHITE_PEN);
        dc.DrawLine(rect.GetLeft(), rect.GetTop(), rect.GetRight(), rect.GetTop());
        dc.DrawLine(rect.GetLeft(), rect.GetTop(), rect.GetLeft(), rect.GetBottom());
    }

	wxString filename = m_fileData->GetName();
    if (!filename.IsEmpty() && (ctrl->GetWindowStyle() & wxTH_TEXT_LABEL))
    {
		// write the file name
        dc.SetFont(ctrl->GetFont());
        if (style & wxTHUMBNAIL_SELECTED)
            dc.SetTextForeground(*wxWHITE);
        else
            dc.SetTextForeground(*wxBLACK);
        dc.SetBackgroundMode(wxTRANSPARENT);

        int margin = ctrl->GetThumbnailMargin();

        wxRect fRect;
        fRect.x = rect.x + margin;
        fRect.y = rect.y + margin;
        fRect.width = rect.width - 2*margin;
        fRect.height = (rect.height - imageRect.height)/2 - 2*margin;

        wxCoord textW, textH;
        dc.GetTextExtent(filename, & textW, & textH);

        dc.SetClippingRegion(fRect);
        int x = fRect.x + wxMax(0, (fRect.width - textW)/2);
        int y = fRect.y ;
        dc.DrawText(filename, x, y);
        dc.DestroyClippingRegion();
    }

	// write the image datetime
	CMetadata md = m_fileData->GetMetadata();
	wxDateTime dt = m_fileData->GetImageDateTime();
	if( dt.IsValid() && (ctrl->GetWindowStyle() & wxTH_EXTENSION_OR_DATETIME_LABEL) )
    {
        wxString sdt = dt.FormatDate() + wxT(" ") + dt.FormatTime();

		dc.SetFont(ctrl->GetFont());
        if (style & wxTHUMBNAIL_SELECTED)
            dc.SetTextForeground(*wxWHITE);
        else
            dc.SetTextForeground(ctrl->GetTypeColour());
        dc.SetBackgroundMode(wxTRANSPARENT);
        
        int margin = ctrl->GetThumbnailMargin();
        
        wxRect fRect;
        fRect.x = rect.x + margin;
	    fRect.y = rect.y + rect.height - (rect.height - imageRect.height)/2 + margin;
        fRect.width = rect.width - 2*margin;
        fRect.height = (rect.height - imageRect.height)/2 - 2*margin;
        
        wxCoord textW, textH;
        dc.GetTextExtent(sdt, & textW, & textH);
        
        dc.SetClippingRegion(fRect);
        int x = fRect.x ;
        int y = fRect.y ;
        dc.DrawText(sdt, x, y);
        dc.DestroyClippingRegion();            
    }

	// Draw tag bitmap
    if (style & wxTHUMBNAIL_TAGGED)
    {
        const wxBitmap& tagBitmap = ctrl->GetTagBitmap();
        if (tagBitmap.Ok())
        {
            int x = rect.x + rect.width - tagBitmap.GetWidth() - ctrl->GetThumbnailMargin();
            int y = rect.y + ctrl->GetThumbnailMargin();
            dc.DrawBitmap(tagBitmap, x, y, true);
        }
    }

    // If the item itself is the focus, draw a dotted
    // rectangle around it
    if (style & wxTHUMBNAIL_IS_FOCUS)
    {
        wxPen dottedPen(ctrl->GetFocusRectColour(), 1, wxDOT);
        dc.SetPen(dottedPen);
        dc.SetBrush(*wxTRANSPARENT_BRUSH);
        wxRect focusRect = imageRect;
        focusRect.x --;
        focusRect.y --;
        focusRect.width += 2;
        focusRect.height += 2;
        dc.DrawRectangle(focusRect);
    }
    
    return true;
}



/*!
 * vvvPThumbnailCtrl
 */



vvvPThumbnailCtrl::vvvPThumbnailCtrl( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style ) 
	:wxThumbnailCtrl( parent, id, pos, size, style )
{
	numFilesTot = 0;
	sizeFilesTot = 0;
}

void vvvPThumbnailCtrl::RotateSelectedThumbnails( CUtils::ImageRotation rotation ) {

	Database db = wxGetApp().GetDatabase();
	db->TransactionStart();

	const wxArrayInt& thumbnails = GetSelections();
	for( int k = 0; k < (int) thumbnails.GetCount(); k++ ) {
		RotateSingleThumbnail( rotation, thumbnails[k] );
		Reload( thumbnails[k] );
	}

	db->TransactionCommit();
	Refresh();

}

void vvvPThumbnailCtrl::RotateSingleThumbnail( CUtils::ImageRotation rotation, int nThumbnail ) {

	vvvPImageThumbnailItem* th = static_cast<vvvPImageThumbnailItem*>( GetItem(nThumbnail) );
	CUtils::ImageRotation oldRotation = th->GetFileData()->GetRotation();
	CUtils::ImageRotation newRotation = CUtils::GetNewImageRotation( oldRotation, rotation );
	th->GetFileData()->SetRotation( newRotation );

	int fileID = th->GetFileData()->GetID();
	std::string sql = CSqlStrings::UpdateFileRotation( fileID, newRotation );
	Database db = wxGetApp().GetDatabase();
	db->ExecuteNoReturn(sql);

}

bool vvvPThumbnailCtrl::IsAnythingSelected() {

	const wxArrayInt& thumbnails = GetSelections();
	return (thumbnails.GetCount() > 0);
}

std::vector<int> vvvPThumbnailCtrl::GetSelectedItemsIndexes() {

	std::vector<int> indexes;
	const wxArrayInt& thumbnails = GetSelections();
	int nSel = (int) thumbnails.GetCount();
	for( int k = 0; k < nSel; k++ ) {
		int selectedIndex = thumbnails[k];
		indexes.push_back( selectedIndex );
	}
	return indexes;
}

void vvvPThumbnailCtrl::OnContextMenu( wxContextMenuEvent& event ) {
	wxPoint point = event.GetPosition();
	// If from keyboard
	if (point.x == -1 && point.y == -1) {
		wxSize size = GetSize();
		point.x = size.x / 2;
		point.y = size.y / 2;
	} else {
		point = ScreenToClient(point);
		// if there is a toolbar we need to correct the coordinates
		wxPoint origin = GetClientAreaOrigin();
		point += origin;
	}

	// if unselected, select the current thumbnail
	int focusItem = GetFocusItem();
	if( focusItem >= 0 && !IsSelected(focusItem) ) {
		SelectNone();
		Select( focusItem );
	}

	CAppSettings::CurrentView cv = wxGetApp().GetMainFrame()->GetCurrentView();

	wxMenu menu;
	if( cv == CAppSettings::cvFixed || cv == CAppSettings::cvRemovable ) {
		menu.Append( ID_ADD_VIRTUAL_FOLDER, _("Add To Virtual Folder") );
		menu.AppendSeparator();
	}
	menu.Append( ID_TOOLS_SHOW_IMAGE, _("Show Image") );
	menu.Append( ID_EDIT_OBJECT_INFORMATION, _("Image information\tCtrl-I") );
	menu.Append( ID_EDIT_ROTATE_RIGHT, _("Rotate right\tCtrl-R") );
	menu.Append( ID_EDIT_ROTATE_UPSIDE_DOWN, _("Upside down") );
	menu.Append( ID_EDIT_ROTATE_LEFT, _("Rotate left\tCtrl-L") );

	// add external programs
	CExternalPrograms& ep = wxGetApp().GetAppSettings().m_ExternalPrograms;
	if( ep.size() > 0 ) {
		menu.AppendSeparator();
		CMainFrame* mf = dynamic_cast<CMainFrame*>( GetGrandParent() );
		mf->AppendExternalProgramsMenus( &menu, ep );
	}

	PopupMenu( &menu, point );
}

void vvvPThumbnailCtrl::ShowInfoDialog()
{
	std::vector<int> selectedIndexes = GetSelectedItemsIndexes();

	if( selectedIndexes.empty() ) return;

	if( selectedIndexes.size() == 1 ) {
		// only one item, show the image info dialog
		vvvPImageThumbnailItem* th = static_cast<vvvPImageThumbnailItem*>( GetItem(selectedIndexes[0]) );
		CFileData *fd = th->GetFileData();
		CImageInformationDialog dialog;
		dialog.SetFileData( fd );
		dialog.Create( GetGrandParent(), ID_CIMAGEINFORMATIONDIALOG, _("Image information") );
		if( dialog.ShowModal() != wxID_OK ) return;
	}
	else {
		// multiple items, show the object description dialog
		wxString orgDescr = wxEmptyString;
		CDialogObjectDescription dialog( GetGrandParent(), ID_CDIALOGOBJECTDESCRIPTION, _("Object description") );
		wxString objectName = _("Multiple items") + wxString::Format( wxT(" (%d)"), selectedIndexes.size() );
		dialog.SetObjectName( objectName );
		dialog.SetDescription( orgDescr );
		if( dialog.ShowModal() != wxID_OK ) return;

		wxString newDescr = dialog.GetDescription();
		if( newDescr == orgDescr) return;

		// set the new description
		for( int k = 0; k < static_cast<int>(selectedIndexes.size()); k++ ) {
			vvvPImageThumbnailItem* th = static_cast<vvvPImageThumbnailItem*>( GetItem(selectedIndexes[k]) );
			CFileData *fd = th->GetFileData();
			int fileID = fd->GetID();
			std::string sql = CSqlStrings::UpdatePhysicalFileDescription( fileID, newDescr );
			FSDB::Database db = wxGetApp().GetDatabase();
			db->ExecuteNoReturn(sql);
			fd->SetDescription( newDescr );
		}
	}
}

void vvvPThumbnailCtrl::ShowContainerData( CFileDataArray &fileDataArray ) {

	PrepareForLoading();
	LoadNewData( fileDataArray );
}

void vvvPThumbnailCtrl::PrepareForLoading()
{
	numFilesTot = 0;
	sizeFilesTot = 0;

	Clear();
	Update();
	Scroll( 0, 0 );
}

void vvvPThumbnailCtrl::LoadNewData( CFileDataArray &fileDataArray )
{
	int nCurrentItems = GetCount();
	Freeze();
	for( int k = nCurrentItems; k < fileDataArray.size(); k++ ) {
		numFilesTot++;
		sizeFilesTot += fileDataArray[k].GetSize();

		Append( new vvvPImageThumbnailItem(&(fileDataArray[k])));
	}
	Thaw();
}

std::vector<CFileData*> vvvPThumbnailCtrl::GetSelectedItemsCFileData()
{
	std::vector<CFileData*> cfd;
	const wxArrayInt& thumbnails = GetSelections();
	int nSel = (int) thumbnails.GetCount();
	for( int k = 0; k < nSel; k++ ) {
		int selectedIndex = thumbnails[k];
		vvvPImageThumbnailItem* th = static_cast<vvvPImageThumbnailItem*>( GetItem(selectedIndex) );
		cfd.push_back( th->GetFileData() );
	}
	return cfd;
}

int vvvPThumbnailCtrl::GetFirstVisibleItemIndex()
{
	wxPoint pt( 8, 8 );
	// try with a position where the first item should be
	int idx = 0;
	if( !HitTest(pt, idx) ) {
		// try another point
		pt.y = 16;
		if( !HitTest(pt, idx) ) {
			// give up
			idx = -1;
		}
	}
	return idx;
}

void vvvPThumbnailCtrl::SetThumbnailImageSize(const wxSize& sz)
{
	// try to show the same thumbnails after changing size
	int firstIndex = GetFirstVisibleItemIndex();
	
	wxThumbnailCtrl::SetThumbnailImageSize( sz );

	if( firstIndex >= 0 )
		EnsureVisible( firstIndex );
}

int vvvPThumbnailCtrl::Append(wxThumbnailItem* item)
{
    int sz = (int) GetCount();
    m_items.Add(item);
    m_firstSelection = -1;
    m_lastSelection = -1;
    m_focusItem = -1;

    item->Load(this, false);

    if (m_freezeCount == 0)
    {
        SetupScrollbars();
        Refresh();
    }
    return sz;
}

void vvvPThumbnailCtrl::OnSetFocus( wxFocusEvent& WXUNUSED(event) )
{
	// send the event to notify that the window received the focus
	fsRightPaneFocusedEvent event( fsEVT_RIGHT_PANE_FOCUSED, GetId() );
	event.SetEventObject( this );
	GetEventHandler()->ProcessEvent( event );
}

