/*
	This file is part of vvvP (Virtual Volumes View Photo edition)

	Copyright (C) 2008, the vvvP Development team

	Author: Fulvio Senore

    vvvP is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    vvvP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with vvvP; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/////////////////////////////////////////////////////////////////////////////
// Name:        vvvpapp.cpp
// Purpose:     
// Author:      
// Modified by: 
// Created:     30/07/2008 21:52:10
// RCS-ID:      
// Copyright:   
// Licence:     
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "vvvpapp.h"
#include "utils.h"
#include "sqlstrings.h"
#include "uuidwrapper.h"
#include "app_settings.h"
#include "fs_imagjpeg.h"
#include "fs_imagtiff.h"

#include "wx/log.h"
#include "wx/cmdline.h"
#include "wx/filename.h"
#include "wx/config.h"
#include "wx/stdpaths.h"
#include "wx/fileconf.h"
#include "wx/sysopt.h"

////@begin XPM images
////@end XPM images


/*!
 * Application instance implementation
 */

////@begin implement app
IMPLEMENT_APP( CVvvPApp )
////@end implement app


/*!
 * CVvvPApp type definition
 */

IMPLEMENT_CLASS( CVvvPApp, wxApp )


/*!
 * CVvvPApp event table definition
 */

BEGIN_EVENT_TABLE( CVvvPApp, wxApp )

////@begin CVvvPApp event table entries
////@end CVvvPApp event table entries

END_EVENT_TABLE()

static const wxCmdLineEntryDesc g_cmdLineDesc[] = 
{
	{ wxCMD_LINE_PARAM, NULL, NULL, "input catalog", wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },
	{ wxCMD_LINE_NONE }
};


/*!
 * Constructor for CVvvPApp
 */

CVvvPApp::CVvvPApp()
{
    Init();
}


/*!
 * Member initialisation
 */

void CVvvPApp::Init()
{
////@begin CVvvPApp member initialisation
////@end CVvvPApp member initialisation

	m_DbConnected = false;
}

/*!
 * Initialisation for CVvvPApp
 */

bool CVvvPApp::OnInit()
{    

#ifdef __WXMAC__
	// in OSX we cannot know which will be the current working directory (depends on how we started the program)
	// but the firebird embedded database (used the way we use it) needs the current working directory to be same as the executable file so we set it accordingly
	wxSystemOptions::SetOption( wxT("mac.listctrl.always_use_generic"), 1 );
	wxString appPath = wxStandardPaths::Get().GetExecutablePath();
	wxFileName fn( appPath );
	appPath = fn.GetPath();
	wxSetWorkingDirectory( appPath );
	
	s_macHelpMenuTitleName = _("Help");
#endif	

	// set the config object
	SetVendorName(wxT("VVVP"));
	SetAppName(wxT("VVVP"));

	// initialize help
	wxFileSystem::AddHandler( new wxZipFSHandler );
//	m_HelpController = new wxHtmlHelpController;
	m_HelpController = new wxHelpController;
	m_HelpController->Initialize( CUtils::GetHelpFileName() );

	// parse the command line
	// available command line options:
	// -s <file name>
	//    specifies the name of a file used to store the program's settings. Used to make VVVP a portable app. If relative it is relative to the exe location.
	// -d <folder name>
	//    specifies the default folder when creating a new catalog. Used to make VVVP a portable app. If relative it is relative to the exe location.
	wxCmdLineParser cmdParser( g_cmdLineDesc, argc, argv );
	cmdParser.AddOption( wxT("s"), wxT("SettingsFile") );
	cmdParser.AddOption( wxT("d"), wxT("DefaultDataFolder") );
	int res;
	{
		wxLogNull log;
		res = cmdParser.Parse( false );
	}
	if( res == -1 || res > 0 ) {
		m_CatalogName = wxEmptyString;
		m_SettingsFileName = wxEmptyString;
	}
	else {
		if( cmdParser.GetParamCount() > 0 ) {
			m_CatalogName = cmdParser.GetParam(0);
			wxFileName fName( m_CatalogName );
			fName.Normalize( wxPATH_NORM_LONG | wxPATH_NORM_DOTS | wxPATH_NORM_TILDE | wxPATH_NORM_ABSOLUTE );
			m_CatalogName = fName.GetFullPath();
		}
		else 
			m_CatalogName = wxEmptyString;
		m_SettingsFileName = wxEmptyString;
		m_DefaultDataFolder = wxEmptyString;
		if( cmdParser.Found( wxT("s"), &m_SettingsFileName ) ) {
			if( !wxIsAbsolutePath(m_SettingsFileName) ) {
				// the path must be relative to the application's path
				wxString appPath = wxStandardPaths::Get().GetExecutablePath();
				wxFileName tmp(appPath );
				appPath = tmp.GetPath();	// removes the program's name
				wxFileName fn( m_SettingsFileName );
				fn.MakeAbsolute( appPath );
				m_SettingsFileName = fn.GetFullPath();
			}
		}
		if( cmdParser.Found( wxT("d"), &m_DefaultDataFolder ) ) {
			if( !wxIsAbsolutePath(m_DefaultDataFolder) ) {
				// the path must be relative to the application's path
				wxString appPath = wxStandardPaths::Get().GetExecutablePath();
				wxFileName tmp(appPath );
				appPath = tmp.GetPath();	// removes the program's name
				wxFileName fn( m_DefaultDataFolder );
				fn.MakeAbsolute( appPath );
				m_DefaultDataFolder = fn.GetFullPath();
			}
		}
	}

	// if a settings file name has been passed as an option, create the corresponding config object
	if( !m_SettingsFileName.empty() ) {
		wxConfigBase *pConfig = new wxFileConfig( wxEmptyString, wxEmptyString, m_SettingsFileName );
		wxConfigBase::Set( pConfig );
	}

	m_AppSettings.Load();

	// set up the locale
	m_locale.Init( wxLANGUAGE_DEFAULT, wxLOCALE_LOAD_DEFAULT );

#if wxUSE_XPM
	wxImage::AddHandler(new wxXPMHandler);
#endif
#if wxUSE_LIBPNG
	wxImage::AddHandler(new wxPNGHandler);
#endif
#if wxUSE_GIF
	wxImage::AddHandler(new wxGIFHandler);
#endif
#if wxUSE_ICO_CUR
  wxImage::AddHandler( new wxICOHandler );
  wxImage::AddHandler( new wxCURHandler );
  wxImage::AddHandler( new wxANIHandler );
#endif

//	wxInitAllImageHandlers();
	// add support for custom jpeg and tiff handlers to handle color profiles
	wxImage::AddHandler(new fsJPEGHandler);
	wxImage::AddHandler(new fsTIFFHandler);

	try {
		mainWindow = new CMainFrame( NULL );
		mainWindow->Show(true);
	}
	catch( CDataErrorException& e ) {
		wxString s;
		if( e.GetErrorCause() == CDataErrorException::ecServerNotFound )
			s = _("Unable to connect to the server\n\n");
		else
			s = _("An unexpected error has occurred. Here is a description of the error:\n\n");
		s += CUtils::std2wx( e.what() );
		wxLogError( wxT("%s"), s.c_str() );
		return false;
	}
	catch( IBPP::Exception& e ) {
		wxString s = _("An unexpected error has occurred. Here is a description of the error:\n\n");
		s += CUtils::std2wx( e.ErrorMessage() );
		wxLogError( wxT("%s"), s.c_str() );
		return false;
	}
	catch( ... ) {
		wxString s = _("An unexpected error has occurred. This application will be terminated");
		wxLogError( wxT("%s"), s.c_str() );
		return false;
	}

    return true;
}


/*!
 * Cleanup for CVvvPApp
 */

int CVvvPApp::OnExit()
{    
	delete m_HelpController;

	m_AppSettings.Save();

	// it looks like some resources are not released automatically, so we release them here
	// probably the reason is that wxWidgets uses an older version of expat
	Exiv2::XmpParser::terminate();

////@begin CVvvPApp cleanup
	return wxApp::OnExit();
////@end CVvvPApp cleanup
}


bool CVvvPApp::OpenDatabase( wxString fileName, int expectedVersion ) {

	wxBusyCursor wait;
	wxString serverName, userName, password;
	bool errorOpeningDB;

	if( m_DbConnected ) {
		m_db->Disconnect();
		m_DbConnected = false;
	}

	m_AppSettings.GetConnectionData( serverName, userName, password );
	m_db = FSDB::DatabaseFirebirdFactory( CUtils::wx2std(serverName), CUtils::wx2std(fileName), CUtils::wx2std(userName), CUtils::wx2std(password) );
	errorOpeningDB = false;
	try {
		m_db->Connect();
	}
	catch( CDataErrorException e ) {
		switch( e.GetErrorCause()  ) {
			case CDataErrorException::ecDatabaseNotFound:
				CUtils::MsgErr( _("Catalog not found:\n\n") + fileName );
				errorOpeningDB = true;
				break;
			case CDataErrorException::ecWrongUsernameOrPassword:
				CUtils::MsgErr( _("Incorrect username or password for the following database:\n\n") + fileName );
				errorOpeningDB = true;
				break;
			case CDataErrorException::ecServerNotFound:
				if( m_AppSettings.dbConnectToServer )
					CUtils::MsgErr( _("Unable to connect to the following server:\n\n") + serverName );
				else
					CUtils::MsgErr( _("Unable to open the catalog.\nPlease note that you are not connected to a database server so you cannot open a catalog located on another computer in the network.\nIf you want to open a catalog on another computer you must connect to the database server (Tools/Options menu).") );
				errorOpeningDB = true;
				break;
			default:
				throw;
		}
	}

	if( errorOpeningDB ) return false;

	// read the current database version
	std::string sql = CSqlStrings::SelectServiceData();
	FSDB_CREATE_RS_READWRITE_FWDONLY(m_db, rs)
	rs.Open( sql, "SERVICE", "SERVICE_ID" );
	int currentDbVersion = rs["DB_VERSION"];
	wxString flUUID;
	if( rs["FL_UUID"].IsNull() )
		flUUID = wxEmptyString;
	else
		flUUID = CUtils::DBstd2wx( rs["FL_UUID"] );
	if( flUUID == wxEmptyString ) {
		// add a UUID to the database
		rs.Edit();
		flUUID = CUUIDWrapper::ComputeUUID().MakeUpper();
		rs["FL_UUID"] = CUtils::DBwx2std( flUUID );
		rs.Update();
	}
	rs.Close();
	// read the fixed locations
	m_FixedLocations.SetUUID( flUUID );

	if( currentDbVersion < expectedVersion ) {
		// we need to upgrade the database

		if( !wxFileExists(CUtils::GetStructUpdateDbName() ) ) {
			CUtils::MsgErr( _("This catalog has been created with a previous version of the program and it needs to be upgraded, but the file containing the upgrades list is missing.\n\nYou can try reinstalling the program to solve this problem.") );
			return false;
		}

	}

	m_DbConnected = true;

	return true;
}

// derived to add exception handling
int CVvvPApp::OnRun() {

	try {
		return wxApp::OnRun();
	}
	catch( CDataErrorException& e ) {
		wxString s;
		if( e.GetErrorCause() == CDataErrorException::ecServerNotFound )
			s = _("Unable to connect to the server\n\n");
		else
			s = _("An unexpected error has occurred. Here is a description of the error:\n\n");
		s += CUtils::std2wx( e.what() );
		wxLogError( wxT("%s"), s.c_str() );
		return 3;
	}
	catch( IBPP::Exception& e ) {
		wxString s = _("An unexpected error has occurred. Here is a description of the error:\n\n");
		s += CUtils::std2wx( e.ErrorMessage() );
		wxLogError( wxT("%s"), s.c_str() );
		return 3;
	}
	catch( ... ) {
		wxString s = _("An unexpected error has occurred. This application will be terminated");
		wxLogError( wxT("%s"), s.c_str() );
		return 3;
	}
}

bool CVvvPApp::OnExceptionInMainLoop() {

	try {
		throw;
	}
    catch (const std::exception& e) {
		wxString msg = _("An unexpected error has happened. Here is a description of the error:\n\n");
		msg += CUtils::std2wx(e.what());
		wxLogError( wxT("%s"), msg.c_str() );
    }
	return false;
}

void CVvvPApp::UpdateDBStatisticsPhysical() {

	std::vector<std::string> tables;
	tables.push_back( "VOLUMES" );
	tables.push_back( "PATHS" );
	tables.push_back( "PATHS_FILES" );
	tables.push_back( "FILES" );
	tables.push_back( "METADATA" );
	tables.push_back( "THUMBNAILS" );
	m_db->UpdateStatistics( tables );

}

void CVvvPApp::UpdateDBStatisticsVirtual() {

	std::vector<std::string> tables;
	tables.push_back( "VIRTUAL_PATHS" );
	tables.push_back( "VIRTUAL_FILES" );
	m_db->UpdateStatistics( tables );

}

void CVvvPApp::HandleEvent(wxEvtHandler* handler, wxEventFunction func, wxEvent& event) const
{
    try {
        wxApp::HandleEvent(handler, func, event);
    }
    catch (const std::exception& e) {
		wxString msg = _("An unexpected error has happened. Here is a description of the error:\n\n");
		msg += CUtils::std2wx(e.what());
		wxLogError( wxT("%s"), msg.c_str() );
    }
}


